@@ -633,6 +633,7 @@ action_element_equal( Reduce *rc, PElement *p1, PElement *p2 )
 		return( TRUE );
 
 	case ELEMENT_NOVAL:
+	case ELEMENT_STATIC:
 	default:
 		assert( FALSE );
 
@@ -1339,16 +1340,16 @@ action_proc_uop( Reduce *rc, Compile *compile,
 		if( PEISREAL( a ) ) {
 			double v = PEGETREAL( a );
 
-			v = (int) IM_CLIP( 0, v, UCHAR_MAX );
+			v = IM_CLIP( 0, v, UCHAR_MAX );
 
-			PEPUTP( out, ELEMENT_CHAR, (unsigned char) v );
+			PEPUTP( out, ELEMENT_CHAR, (int) v );
 		}
 		else if( PEISCOMPLEX( a ) ) {
 			double v = PEGETREALPART( a );
 
-			v = (int) IM_CLIP( 0, v, UCHAR_MAX );
+			v = IM_CLIP( 0, v, UCHAR_MAX );
 
-			PEPUTP( out, ELEMENT_CHAR, (unsigned char) v );
+			PEPUTP( out, ELEMENT_CHAR, (int) v );
 		}
 		else if( PEISCHAR( a ) ) {
 			PEPUTPE( out, a );
