@@ -76,6 +76,7 @@ box_build( iDialog *idlg,
 	gtk_label_set_markup( GTK_LABEL( lab ), s );
         gtk_label_set_justify( GTK_LABEL( lab ), GTK_JUSTIFY_LEFT );
         gtk_label_set_selectable( GTK_LABEL( lab ), TRUE );
+	gtk_label_set_line_wrap( GTK_LABEL( lab ), TRUE );
         gtk_box_pack_start( GTK_BOX( hb ), lab, FALSE, FALSE, 0 );
 	gtk_widget_show( lab );
 }
@@ -119,7 +120,7 @@ box_vmarkup( char *out, const char *top, const char *sub, va_list ap )
 	escape_markup( buf2, buf3, MAX_DIALOG_TEXT );
 
 	(void) im_snprintf( out, MAX_DIALOG_TEXT, 
-		"<span weight=\"bold\" size=\"larger\">%s</span>", buf1 );
+		"<b><big>%s</big></b>", buf1 );
 	if( strcmp( buf3, "" ) != 0 ) {
 		int len = strlen( out );
 
@@ -268,42 +269,50 @@ about_build( iDialog *idlg, GtkWidget *work )
 	buf_init_static( &buf, txt, MAX_DIALOG_TEXT );
 
 	im_snprintf( txt2, MAX_DIALOG_TEXT, _( "About %s." ), PACKAGE );
-	buf_appendf( &buf, 
-		"<span weight=\"bold\" size=\"larger\">%s</span>\n\n", txt2 );
+	buf_appendf( &buf, "<b><big>%s</big></b>\n\n", txt2 );
 	im_snprintf( txt2, MAX_DIALOG_TEXT, 
-		_( "%s: (c) 2004 The National Gallery" ), PACKAGE );
+		_( "%s is an image processing package." ), PACKAGE );
 	buf_appendf( &buf, "%s\n\n", txt2 );
+
 	im_snprintf( txt2, MAX_DIALOG_TEXT, 
 		_( "%s comes with ABSOLUTELY NO WARRANTY. This is "
-			"free software\n"
-		"and you are welcome to redistribute it under certain "
-			"conditions,\n"
-		"see http://www.gnu.org." ), PACKAGE );
+		"free software and you are welcome to redistribute "
+		"it under certain conditions, see http://www.gnu.org." ), 
+		PACKAGE );
 	buf_appendf( &buf, "%s\n\n", txt2 );
 
-{
-	char start[FILENAME_MAX];
+	im_snprintf( txt2, MAX_DIALOG_TEXT, 
+		_( "%s: (c) 2005 The National Gallery" ), PACKAGE );
+	buf_appendf( &buf, "%s\n", txt2 );
 
-	expand_variables( IP_SAVEDIR IM_DIR_SEP_STR "start", start );
-        nativeize_path( start );
-	buf_appendf( &buf, "%s %s\n\n", _( "Personal start folder:" ), start );
+{
+	char buf1[FILENAME_MAX];
+	char buf2[FILENAME_MAX];
+
+	im_snprintf( buf1, FILENAME_MAX, "%s" IM_DIR_SEP_STR "start",
+		get_savedir() );
+	expand_variables( buf1, buf2 );
+        nativeize_path( buf2 );
+	buf_appendf( &buf, "%s %s\n", _( "Personal start folder:" ), buf2 );
 }
 
 	buf_appendf( &buf, "%s %s\n", _( "Homepage:" ), VIPS_HOMEPAGE );
 	escape_markup( im_version_string(), txt2, MAX_DIALOG_TEXT );
 	buf_appendf( &buf, "%s %s\n", 
-		_( "Linked to VIPS library version:" ), txt2 );
+		_( "Linked to VIPS:" ), txt2 );
 	escape_markup( IM_VERSION_STRING, txt2, MAX_DIALOG_TEXT );
-	buf_appendf( &buf, "%s %s\n\n", 
-		_( "Built against VIPS library version:" ), txt2 );
+	buf_appendf( &buf, "%s %s\n", 
+		_( "Built against VIPS:" ), txt2 );
 	escape_markup( PACKAGE, txt2, MAX_DIALOG_TEXT );
 	buf_appendf( &buf, "$PACKAGE: %s\n", txt2 );
 	escape_markup( VERSION, txt2, MAX_DIALOG_TEXT );
 	buf_appendf( &buf, "$VERSION: %s\n", txt2 );
-	escape_markup( NN( getenv( "VIPSHOME" ) ), txt2, MAX_DIALOG_TEXT );
+	escape_markup( NN( g_getenv( "VIPSHOME" ) ), txt2, MAX_DIALOG_TEXT );
 	buf_appendf( &buf, "$VIPSHOME: %s\n", txt2 );
-	escape_markup( NN( getenv( "HOME" ) ), txt2, MAX_DIALOG_TEXT );
+	escape_markup( NN( g_getenv( "HOME" ) ), txt2, MAX_DIALOG_TEXT );
 	buf_appendf( &buf, "$HOME: %s\n", txt2 );
+	escape_markup( NN( g_getenv( "SAVEDIR" ) ), txt2, MAX_DIALOG_TEXT );
+	buf_appendf( &buf, "$SAVEDIR: %s\n", txt2 );
 	escape_markup( PATH_TMP, txt2, MAX_DIALOG_TEXT );
 	buf_appendf( &buf, "%s %s", 
 		_( "Temp files in:" ), txt2 );
@@ -326,6 +335,7 @@ about_build( iDialog *idlg, GtkWidget *work )
 	gtk_label_set_markup( GTK_LABEL( lab ), buf_all( &buf ) );
         gtk_label_set_justify( GTK_LABEL( lab ), GTK_JUSTIFY_LEFT );
         gtk_label_set_selectable( GTK_LABEL( lab ), TRUE );
+	gtk_label_set_line_wrap( GTK_LABEL( lab ), TRUE );
         gtk_box_pack_start( GTK_BOX( hb ), lab, FALSE, FALSE, 2 );
 	gtk_widget_show( lab );
 }
@@ -376,18 +386,6 @@ box_help( GtkWidget *par, const char *name )
 	box_alert( par );
 }
 
-/* Callback from ItemFactory ... display help page.
- */
-/*ARGSUSED*/
-void
-box_help_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *par )
-{
-	const char *str = GUINT_TO_POINTER( callback_action );
-
-	box_help( par, str );
-}
-
 /* Name + caption dialog ... for new workspace / new column.
  */
 
@@ -398,10 +396,10 @@ stringset_child_destroy( StringsetChild *ssc )
 {
 	ssc->ss->children = g_slist_remove( ssc->ss->children, ssc );
 
-	FREE( ssc->label );
-	FREE( ssc->text );
-	FREE( ssc->tooltip );
-	FREE( ssc );
+	IM_FREE( ssc->label );
+	IM_FREE( ssc->text );
+	IM_FREE( ssc->tooltip );
+	IM_FREE( ssc );
 
 	return( NULL );
 }
@@ -704,12 +702,10 @@ imageheader_refresh( Imageheader *imageheader )
 		set_glabel( imageheader->yoff, "%d", im->Yoffset );
 		set_glabel( imageheader->filename, "%s", im->filename );
 
-		gtk_text_freeze( GTK_TEXT( imageheader->history ) );
-		gtk_editable_delete_text( GTK_EDITABLE( imageheader->history ),
-			0, -1 );
-		gtk_text_insert( GTK_TEXT( imageheader->history ),
-			NULL, NULL, NULL, im->Hist, strlen( im->Hist ) );
-		gtk_text_thaw( GTK_TEXT( imageheader->history ) );
+		gtk_text_buffer_set_text( 
+			gtk_text_view_get_buffer( 
+				GTK_TEXT_VIEW( imageheader->history ) ),
+			im->Hist, -1 );
 	}
 	else {
 		set_glabel( imageheader->xsize, "--" );
@@ -756,8 +752,7 @@ imageheader_build( GtkWidget *widget )
 
 	GtkWidget *table;
 	GtkWidget *swin;
-	GtkAdjustment *hadj;
-	GtkAdjustment *vadj;
+	PangoFontDescription *font_desc;
 
 #ifdef DEBUG
 	printf( "imageheader_build: %s\n", IWINDOW( imageheader )->title );
@@ -788,12 +783,14 @@ imageheader_build( GtkWidget *widget )
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( swin ),
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
         gtk_box_pack_start( GTK_BOX( idlg->work ), swin, TRUE, TRUE, 2 );
-	hadj = gtk_scrolled_window_get_hadjustment( 
-		GTK_SCROLLED_WINDOW( swin ) );
-	vadj = gtk_scrolled_window_get_vadjustment( 
-		GTK_SCROLLED_WINDOW( swin ) );
-	imageheader->history = gtk_text_new( hadj, vadj );
-	gtk_text_set_editable( GTK_TEXT( imageheader->history ), FALSE );
+	imageheader->history = gtk_text_view_new();
+	gtk_text_view_set_editable( GTK_TEXT_VIEW( imageheader->history ), 
+		FALSE );
+	gtk_text_view_set_cursor_visible( GTK_TEXT_VIEW( imageheader->history ),
+		FALSE );
+	font_desc = pango_font_description_from_string( "Mono" );
+	gtk_widget_modify_font( imageheader->history, font_desc );
+	pango_font_description_free( font_desc );
 	gtk_container_add( GTK_CONTAINER( swin ), imageheader->history );
 
 	imageheader_refresh( imageheader );
@@ -894,8 +891,8 @@ box_url( GtkWidget *par, const char *url )
 	v = (int) ShellExecute( NULL, "open", url2, NULL, NULL, SW_SHOWNORMAL );
 	if( v <= 32 ) {
 		error_top( _( "Unable to open location." ) );
-		error_sub( _( "Unable to open URL \"%s\"\n"
-			"windows error code = %d" ), url, v );
+		error_sub( _( "Unable to open URL \"%s\", "
+			"windows error code = %d." ), url, v );
 		box_alert( par );
 	}
 #elif defined MAC_OSX
@@ -906,9 +903,7 @@ box_url( GtkWidget *par, const char *url )
 	if( !shown ) {
 		box_info( par, _( "Browser window opened." ),
 			_( "Opened window for URL:\n"
-			"\n"
 			"  %s\n"
-			"\n"
 			"This may take a few seconds." ), url );
 		shown = TRUE;
 	}
@@ -933,9 +928,7 @@ box_url( GtkWidget *par, const char *url )
 		error_top( _( "Unable to open location." ) );
 		error_sub( _( 
 			"Attempted to launch browser with command:\n"
-			"\n"
 			"  %s\n"
-			"\n"
 			"You can change this command in Preferences." ),
 			buf_all( &buf2 ) );
 		box_alert( par );
@@ -944,9 +937,7 @@ box_url( GtkWidget *par, const char *url )
 		box_info( par, 
 			_( "Browser window opened." ),
 			_( "Opened window for URL:\n"
-			"\n"
 			"  %s\n"
-			"\n"
 			"You may need to switch desktops to see the "
 			"new window." ), 
 			url2 );
@@ -958,23 +949,6 @@ box_url( GtkWidget *par, const char *url )
 #endif /*HAVE_WINDOWS_H*/
 }
 
-/* From an itemfactory menu.
- */
-void
-box_url_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
-{
-	GtkWidget *par = GTK_WIDGET( callback_data );
-	const char *tag = (const char *) callback_action;
-	BufInfo buf;
-	char txt[512];
-
-	buf_init_static( &buf, txt, 512 );
-	buf_appendf( &buf, "file://%s/%s", NIP_DOCPATH, tag );
-
-	box_url( par, buf_all( &buf ) );
-}
-
 /* Progress dialog.
  */
 
@@ -1044,12 +1018,13 @@ progress_build( GtkWidget *widget )
 
 	box_markup( buf,
 		_( "Computing image." ),
-		_( "Calculating all the pixels in an image. Press the\n"
+		_( "Calculating all the pixels in an image. Press the "
 		"Cancel button to stop computation early." ) );
 	lab = gtk_label_new( NULL );
 	gtk_label_set_markup( GTK_LABEL( lab ), buf );
         gtk_label_set_justify( GTK_LABEL( lab ), GTK_JUSTIFY_LEFT );
         gtk_label_set_selectable( GTK_LABEL( lab ), TRUE );
+	gtk_label_set_line_wrap( GTK_LABEL( lab ), TRUE );
         gtk_box_pack_start( GTK_BOX( idlg->work ), lab, TRUE, TRUE, 0 );
         gtk_misc_set_alignment( GTK_MISC( lab ), 0.0, 0.5 );
 	gtk_widget_show( lab );
@@ -1140,11 +1115,21 @@ progress_update( Progress *progress, int percent, int eta )
 	GtkProgressBar *bar = GTK_PROGRESS_BAR( progress->bar );
 	char buf[1000];
 
-	(void) im_snprintf( buf, 1000, _( "About %d seconds left" ), eta );
+	if( eta > 30 ) {
+		int minutes = (eta + 30) / 60;
+
+		im_snprintf( buf, 1000, ngettext( 
+			"About %d minute left", 
+			"About %d minutes left", 
+			minutes ), minutes );
+	}
+	else
+		im_snprintf( buf, 1000, _( "About %d seconds left" ), eta );
 	gtk_progress_bar_set_text( bar, buf );
 	gtk_progress_bar_set_fraction( bar, 
 		IM_CLIP( 0.0, (double) percent / 100.0, 1.0 ) );
 	animate_hourglass();
+
 }
 
 /* Splash screen.
@@ -1195,9 +1180,8 @@ splash_init( Splash *splash )
 
 	title = gtk_label_new( NULL );
 	(void) im_snprintf( buf1, 1000, _( "Version %s" ), VERSION );
-	(void) im_snprintf( buf2, 1000, 
-		"<span weight=\"bold\" size=\"larger\">%s</span>\n" 
-		"%s", PACKAGE, buf1 );
+	(void) im_snprintf( buf2, 1000, "<b><big>%s</big></b>\n%s", 
+		PACKAGE, buf1 );
 	gtk_label_set_markup( GTK_LABEL( title ), buf2 );
 	gtk_label_set_justify( GTK_LABEL( title ), GTK_JUSTIFY_CENTER );
         gtk_box_pack_start( GTK_BOX( vbox ), title, FALSE, FALSE, 0 );
@@ -1252,10 +1236,10 @@ void
 splash_updatev( Splash *splash, const char *fmt, va_list ap )
 {
 	char buf1[100];
-	char buf2[100];
+	char buf2[150];
 
 	(void) im_vsnprintf( buf1, 100, fmt, ap );
-	(void) im_snprintf( buf2, 100, 
+	(void) im_snprintf( buf2, 150, 
 		"<span style=\"italic\" size=\"smaller\">%s</span>" , buf1 );
 	gtk_label_set_markup( GTK_LABEL( splash->label ), buf2 );
 
@@ -1405,7 +1389,7 @@ fontbutton_finalize( GObject *gobject )
 
 	fontbutton = FONTBUTTON( gobject );
 
-	FREE( fontbutton->font_name );
+	IM_FREE( fontbutton->font_name );
 
 	G_OBJECT_CLASS( fontbutton_parent_class )->finalize( gobject );
 }
