@@ -850,8 +850,7 @@ class_new( Heap *heap, Compile *compile, HeapNode **arg, PElement *out )
 	 */
 	if( compile->nparam + compile->nsecret >= MAX_SYSTEM ) {
 		error_top( _( "Too many arguments." ) );
-		error_sub( _( "Too many arguments to class "
-			"constructor \"%s\".\n"
+		error_sub( _( "Too many arguments to class constructor \"%s\". "
 			"No more than %d arguments are supported." ),
 			symbol_name( compile->sym ), MAX_SYSTEM );
 		return( FALSE );
@@ -949,9 +948,8 @@ class_newv( Heap *heap, const char *name, PElement *out, ... )
 	compile = sym->expr->compile;
 	if( compile->nparam >= MAX_SYSTEM ) {
 		error_top( _( "Too many arguments." ) );
-		error_sub( _( "Too many arguments to class "
-			"constructor \"%s\".\n"
-			"No more than %d arguments are possible." ),
+		error_sub( _( "Too many arguments to class constructor \"%s\". "
+			"No more than %d arguments are supported." ),
 			symbol_name( compile->sym ), MAX_SYSTEM );
 		return( FALSE );
 	}
