@@ -64,7 +64,7 @@ colour_finalize( GObject *gobject )
 {
 	Colour *colour = COLOUR( gobject );
 
-	FREE( colour->colour_space );
+	IM_FREE( colour->colour_space );
 	buf_destroy( &colour->caption );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
@@ -131,7 +131,7 @@ colour_set_colour( Colour *colour,
 
 	for( i = 0; i < 3; i++ )
 		colour->value[i] = value[i];
-	SETSTR( colour->colour_space, colour_space )
+	SETSTR( colour->colour_space, colour_space );
 
 	colour_refresh( colour );
         classmodel_update( CLASSMODEL( colour ) );
