@@ -93,10 +93,10 @@ columnview_clone_cb( GtkWidget *wid, GtkWidget *host, Columnview *cview )
 {
 	Column *col = COLUMN( VOBJECT( cview )->iobject );
 	Workspace *ws = col->ws;
-	const char *newname = workspace_column_name_new( ws, NULL );
+	char *newname = workspace_column_name_new( ws, NULL );
         Column *newcol = workspace_column_get( ws, newname );
 
-	FREE( newname );
+	IM_FREE( newname );
         iobject_set( IOBJECT( newcol ), NULL, IOBJECT( col )->caption );
         newcol->x = col->x + 30;
         newcol->y = col->y + 30;
@@ -178,7 +178,8 @@ columnview_filename( char *file, const char *caption )
 
 	for( i = 0; ; i++ ) {
 		im_snprintf( file, FILENAME_MAX, 
-			"$HOME/.$PACKAGE-$VERSION/data/%s-%d.ws", name, i );
+			"$SAVEDIR" IM_DIR_SEP_STR "data" IM_DIR_SEP_STR 
+			"%s-%d.ws", name, i );
 		if( !existsf( "%s", file ) )
 			break;
 	}
@@ -571,11 +572,8 @@ static void
 columnview_updown_cb( GtkWidget *wid, Columnview *cview )
 {
 	Column *col = COLUMN( VOBJECT( cview )->iobject );
-	Workspace *ws = col->ws;
 
-        col->open = !col->open;
-	filemodel_set_modified( FILEMODEL( ws ), TRUE );
-        iobject_changed( IOBJECT( col ) );
+	column_set_open( col, !col->open );
 }
 
 /* Calculate the xywh of the column title bar in the workspaceview.
@@ -804,23 +802,13 @@ columnview_refresh( vObject *vobject )
 	/* Set open/closed.
 	 */
 	if( col->open ) {
-                if( !GTK_WIDGET_VISIBLE( cview->frame ) ) {
-                        gtk_widget_show( cview->frame );
-
-                        /* Pop to front ... if we have opened it, we
-                         * probably want to look at it.
-                         */
-                        columnview_front( cview );
-                }
-
                 gtk_arrow_set( GTK_ARROW( cview->updown ),
-                        GTK_ARROW_DOWN, GTK_SHADOW_OUT );
+                        GTK_ARROW_UP, GTK_SHADOW_OUT );
                 set_tooltip( cview->updownb, _( "Fold the column up" ) );
 	}
 	else {
-                gtk_widget_hide( cview->frame );
                 gtk_arrow_set( GTK_ARROW( cview->updown ),
-                        GTK_ARROW_UP, GTK_SHADOW_OUT );
+                        GTK_ARROW_DOWN, GTK_SHADOW_OUT );
                 set_tooltip( cview->updownb, _( "Fold the column down" ) );
 	}
 
@@ -942,7 +930,7 @@ columnview_class_init( ColumnviewClass *class )
 	view_class->link = columnview_link;
 
 	pane = columnview_menu = popup_build( _( "Column menu" ) );
-	popup_add_but( pane, _( "_Edit Caption ..." ), 
+	popup_add_but( pane, _( "_Edit Caption" ), 
 		POPUP_FUNC( columnview_caption_cb ) );
 	popup_add_but( pane, _( "Select _All" ), 
 		POPUP_FUNC( columnview_select_cb ) );
@@ -951,7 +939,7 @@ columnview_class_init( ColumnviewClass *class )
 	popup_add_but( pane, GTK_STOCK_SAVE_AS,
 		POPUP_FUNC( columnview_save_as_cb ) );
 	menu_add_sep( pane );
-	popup_add_but( pane, _( "Make Column Into _Menu Item ..." ),
+	popup_add_but( pane, _( "Make Column Into _Menu Item" ),
 		POPUP_FUNC( columnview_to_menu_cb ) );
 	menu_add_sep( pane );
 	popup_add_but( pane, GTK_STOCK_DELETE,
@@ -1029,6 +1017,7 @@ columnview_init( Columnview *cview )
         gtk_container_add( GTK_CONTAINER( but ), icon );
         gtk_signal_connect( GTK_OBJECT( but ), "clicked",
                 GTK_SIGNAL_FUNC( columnview_destroy2_cb ), cview );
+
 	/*
 		
 		FIXME ... turning this on makes nip scroll to a funny place
