@@ -50,6 +50,10 @@
 
 #include "ip.h"
 
+/* Turn on to output i18n strings at the end of compile.
+ */
+static gboolean compile_find_i18n = FALSE;
+
 static iContainerClass *parent_class = NULL;
 
 Compile *
@@ -64,7 +68,7 @@ compile_get_parent( Compile *compile )
 void *
 compile_name_print( Compile *compile )
 {
-	printf( "compile(0x%x) ", (unsigned int) compile );
+	printf( "compile(0x%x) ", GPOINTER_TO_UINT( compile ) );
 	symbol_name_print( compile->sym );
 	printf( " " );
 
@@ -254,20 +258,26 @@ compile_finalize( GObject *gobject )
 	/* Junk parse tree.
 	 */
 	slist_map( compile->treefrag, (SListMapFn) tree_node_destroy, NULL );
-	FREEF( g_slist_free, compile->treefrag );
+	IM_FREEF( g_slist_free, compile->treefrag );
 	compile->tree = NULL;
 
 	/* Break links to all locals.
 	 */
-	FREEF( g_slist_free, compile->param );
+	IM_FREEF( g_slist_free, compile->param );
 	compile->nparam = 0;
-	FREEF( g_slist_free, compile->secret );
+	IM_FREEF( g_slist_free, compile->secret );
 	compile->nsecret = 0;
 	compile->this = NULL;
 	compile->super = NULL;
 	(void) slist_map( compile->children, 
 		(SListMapFn) symbol_link_break, compile );
-	FREEF( g_slist_free, compile->children );
+	IM_FREEF( g_slist_free, compile->children );
+
+	/* Remove static strings we created.
+	 */
+	slist_map( compile->statics, 
+		(SListMapFn) heap_static_string_destroy, NULL );
+	IM_FREEF( g_slist_free, compile->statics );
 
 	/* Junk heap.
 	 */
@@ -280,9 +290,9 @@ compile_finalize( GObject *gobject )
 
 	/* Junk text.
 	 */
-	FREE( compile->text );
-	FREE( compile->prhstext );
-	FREE( compile->rhstext );
+	IM_FREE( compile->text );
+	IM_FREE( compile->prhstext );
+	IM_FREE( compile->rhstext );
 
 	compile->sym = NULL;
 
@@ -339,6 +349,7 @@ compile_init( Compile *compile )
 
 	compile->base.type = ELEMENT_NOVAL;
 	compile->heap = NULL;
+	compile->statics = NULL;
 }
 
 GType
@@ -818,9 +829,20 @@ compile_graph( Compile *compile, ParseNode *pn, PElement *out )
 		 */
 		switch( pn->con.type ) {
 		case PARSE_CONST_STR:
-			if( !heap_string_new( heap, pn->con.val.str, out ) )
+		{
+			HeapStaticString *string;
+
+			if( !(string = heap_static_string_new( 
+				reduce_context->heap,
+				pn->con.val.str )) )
 				return( FALSE );
+			compile->statics = g_slist_prepend( compile->statics,
+				string );
+
+			PEPUTP( out, ELEMENT_STATIC, string );
+
 			break;
+		}
 
 		case PARSE_CONST_CHAR:
 			PEPUTP( out, ELEMENT_CHAR, pn->con.val.ch );
@@ -956,6 +978,7 @@ compile_abstract_body( Compile *compile,
 	case ELEMENT_COMPILEREF:
 	case ELEMENT_NOVAL:
 	case ELEMENT_TAG:
+	case ELEMENT_STATIC:
 		/* Leave alone.
 		 */
 		break;
@@ -1035,13 +1058,19 @@ static int compile_node_sum;
 
 /* A hash code we calculate from a bit of heap.
  */
-typedef unsigned int CompileHash;
+typedef gpointer CompileHash;
+
+/* Combine two hashes.
+ */
+#define COMPILEHASH_ADD( A, B ) \
+	GUINT_TO_POINTER( GPOINTER_TO_UINT( A ) +  GPOINTER_TO_UINT( B ) )
 
 /* Build one of these during sharing analysis. From node pointers to
  * hash codes, and from hash codes to a list of matching node pointers.
  */
 typedef struct _CompileShare {
 	Compile *compile;
+
 	GHashTable *node2hash;
 	GHashTable *hash2nodel;
 } CompileShare;
@@ -1086,15 +1115,14 @@ compile_share_remove( HeapNode *hn, CompileShare *share )
 {
 	CompileHash hash;
 
-	if( (hash = (CompileHash) 
-		g_hash_table_lookup( share->node2hash, hn )) ) {
+	if( (hash = g_hash_table_lookup( share->node2hash, hn )) ) {
 		GSList *nodel;
 
 		if( (nodel = g_hash_table_lookup( share->hash2nodel, 
-			(gpointer) hash )) ) {
+			hash )) ) {
 			nodel = slist_remove_all( nodel, hn );
 			g_hash_table_replace( share->hash2nodel, 
-				(gpointer) hash, nodel );
+				hash, nodel );
 		}
 
 		g_hash_table_remove( share->node2hash, hn );
@@ -1111,23 +1139,20 @@ compile_share_add( CompileShare *share, HeapNode *hn, CompileHash hash )
 	/* Make sure hash is non-zero (very unlikely).
 	 */
 	if( !hash )
-		hash = 1;
+		hash = GUINT_TO_POINTER( 1 );
 
 	if( !g_hash_table_lookup( share->node2hash, hn ) ) {
 		GSList *nodel;
 
-		g_hash_table_insert( share->node2hash, hn, (gpointer) hash ); 
+		g_hash_table_insert( share->node2hash, hn, hash ); 
 
-		if( (nodel = g_hash_table_lookup( share->hash2nodel,
-			(gpointer) hash )) ) {
+		if( (nodel = g_hash_table_lookup( share->hash2nodel, hash )) ) {
 			nodel = g_slist_prepend( nodel, hn );
-			g_hash_table_replace( share->hash2nodel, 
-				(gpointer) hash, nodel );
+			g_hash_table_replace( share->hash2nodel, hash, nodel );
 		}
 		else {
 			nodel = g_slist_prepend( NULL, hn );
-			g_hash_table_insert( share->hash2nodel, 
-				(gpointer) hash, nodel );
+			g_hash_table_insert( share->hash2nodel, hash, nodel );
 		}
 	}
 }
@@ -1171,16 +1196,19 @@ compile_share_scan_element( CompileShare *share, PElement *e )
 	case ELEMENT_COMB:
 	case ELEMENT_IMAGE:
 	case ELEMENT_CONSTRUCTOR:
-		hash = (CompileHash) PEGETTYPE( e ) + 
-			(CompileHash) PEGETVAL( e );
+		hash = GUINT_TO_POINTER( PEGETTYPE( e ) + PEGETVAL( e ) );
 		break;
 
 	case ELEMENT_ELIST:
-		hash = (CompileHash) ELEMENT_ELIST;
+		hash = GUINT_TO_POINTER( ELEMENT_ELIST );
 		break;
 
 	case ELEMENT_TAG:
-		hash = (CompileHash) g_str_hash( PEGETTAG( e ) );
+		hash = GUINT_TO_POINTER( g_str_hash( PEGETTAG( e ) ) );
+		break;
+
+	case ELEMENT_STATIC:
+		hash = GUINT_TO_POINTER( g_str_hash( PEGETSTATIC( e )->text ) );
 		break;
 
 	case ELEMENT_NOVAL:
@@ -1210,15 +1238,19 @@ compile_share_scan_node( CompileShare *share, HeapNode *hn )
 	case TAG_COMPLEX:
 	case TAG_APPL:
 		PEPOINTLEFT( hn, &a );
-		hash += compile_share_scan_element( share, &a );
+		hash = COMPILEHASH_ADD( hash, 
+			compile_share_scan_element( share, &a ) );
 		PEPOINTRIGHT( hn, &a );
-		hash += compile_share_scan_element( share, &a );
+		hash = COMPILEHASH_ADD( hash, 
+			compile_share_scan_element( share, &a ) );
 		hash += (int) hn->type;
 		break;
 
 	case TAG_DOUBLE:
-		hash += (CompileHash) hn->body.ptrs.left;
-		hash += (CompileHash) hn->body.ptrs.right;
+		hash = COMPILEHASH_ADD( hash, 
+			hn->body.ptrs.left );
+		hash = COMPILEHASH_ADD( hash, 
+			hn->body.ptrs.right );
 		hash += (int) hn->type;
 		break;
 
@@ -1434,7 +1466,8 @@ compile_transform_share( HeapNode *hn, Compile *compile )
 			hn1->type = TAG_SHARED;
 			PPUT( hn1, 
 				ELEMENT_NODE, hn2, 
-				ELEMENT_CHAR, compile_share_number );
+				ELEMENT_CHAR, GUINT_TO_POINTER( 
+					compile_share_number ) );
 
 			compile_share_number++;
 			if( compile_share_number == MAX_RELOC ) {
@@ -1479,8 +1512,13 @@ compile_remove_subexpr( Compile *compile, PElement *root )
 	 */
 	compile_share_number = 0;
 	if( heap_map( rootn, 
-		(heap_map_fn) compile_transform_share, compile, NULL ) )
+		(heap_map_fn) compile_transform_share, compile, NULL ) ) {
+		/* We can't leave the graph half-done, it'll confuse the copier
+		 * later. Zap the graph.
+		 */
+		PEPUTP( root, ELEMENT_NOVAL, NULL );
 		return( FALSE );
+	}
 
 #ifdef DEBUG_COMMON
 	if( compile_node_sum ) {
@@ -1608,6 +1646,85 @@ compile_symbol( Symbol *sym )
 /* Parse support.
  */
 
+static void
+compile_check_i18n( Compile *compile, ParseNode *pn )
+{
+	GSList *l;
+
+	switch( pn->type ) {
+	case NODE_APPLY:
+		if( pn->arg1->type == NODE_LEAF && 
+			strcmp( IOBJECT( pn->arg1->leaf )->name, "_" ) == 0 &&
+			pn->arg2->type == NODE_CONST &&
+			pn->arg2->con.type == PARSE_CONST_STR ) {
+			const char *text = pn->arg2->con.val.str;
+
+			if( compile_find_i18n ) {
+				/* Remove msgid duplicates with this.
+				 */
+				static GHashTable *msgid = NULL;
+
+				if( !msgid ) 
+					msgid = g_hash_table_new( 
+						g_str_hash, g_str_equal );
+
+				if( !g_hash_table_lookup( msgid, text ) ) {
+					char buf[MAX_STRSIZE];
+
+					g_hash_table_insert( msgid, 
+						(void *) text, NULL ); 
+					my_strecpy( buf, text, TRUE );
+					printf( "msgid \"%s\"\n", buf );
+					printf( "msgstr \"\"\n\n" );
+				}
+			}
+
+			/* We can gettext these at compile time. Replace the
+			 * APPLY node with a fixed-up text string.
+			 */
+			pn->type = NODE_CONST;
+			pn->con.type = PARSE_CONST_STR;
+			pn->con.val.str = im_strdupn( _( text ) );
+		}
+		break;
+
+	case NODE_GENERATOR:
+		compile_check_i18n( compile, pn->arg1 );
+		compile_check_i18n( compile, pn->arg2 );
+		compile_check_i18n( compile, pn->arg3 );
+		break;
+
+	case NODE_BINOP:
+	case NODE_COMPOSE:
+		compile_check_i18n( compile, pn->arg1 );
+		compile_check_i18n( compile, pn->arg2 );
+		break;
+
+	case NODE_UOP:
+		compile_check_i18n( compile, pn->arg1 );
+		break;
+
+	case NODE_SUPER:
+	case NODE_LISTCONST:
+		for( l = pn->elist; l; l = l->next ) {
+			ParseNode *arg = (ParseNode *) l->data;
+
+			compile_check_i18n( compile, arg );
+		}
+		break;
+
+	case NODE_LEAF:
+	case NODE_CLASS:
+	case NODE_TAG:
+	case NODE_CONST:
+		break;
+
+	case NODE_NONE:
+	default:
+		assert( FALSE );
+	}
+}
+
 /* Do end-of-parse checks. Called after every 'A=...' style definition (not 
  * just top-level syms). Used to do lots of checks, not much left now.
  */
@@ -1626,6 +1743,16 @@ compile_check( Compile *compile )
 				"\"%s\" should have no arguments." ),
 				MEMBER_CHECK, symbol_name( parent ) );
 	}
+
+	/* Magic stuff for i18n of menus. Look for (_ "string constant") and
+	 * pump it through gettext.
+	 */
+#ifdef DEBUG
+	printf( "compile_check_i18n: " );
+	compile_name_print( compile );
+	printf( "\n" );
+#endif /*DEBUG*/
+	compile_check_i18n( compile, compile->tree );
 }
 
 /* Mark error on all exprs using this compile.
@@ -1720,7 +1847,7 @@ compile_resolve( Symbol *sym, Symbol *zombie )
 
 	/* No one refers to the zombie now.
 	 */
-	FREEF( g_slist_free, zombie->parents );
+	IM_FREEF( g_slist_free, zombie->parents );
 
 	IDESTROY( zombie );
 }
@@ -1856,3 +1983,11 @@ compile_get_member_string( Compile *compile, const char *name )
 
 	return( NULL );
 }
+
+/* Output constant i18n strings during compile.
+ */
+void
+compile_set_i18n( gboolean i18n )
+{
+	compile_find_i18n = i18n;
+}
