@@ -114,13 +114,13 @@ conversion_finalize( GObject *gobject )
 
 	conversion_all = g_slist_remove( conversion_all, conv );
 
-	FREEF( im_region_free, conv->ireg );
-	FREEF( im_region_free, conv->reg );
+	IM_FREEF( im_region_free, conv->ireg );
+	IM_FREEF( im_region_free, conv->reg );
 
-	FREEF( imageinfo_destroy_nonheap, conv->repaint_ii );
-	FREEF( imageinfo_destroy_nonheap, conv->display_ii );
-	FREEF( imageinfo_destroy_nonheap, conv->visual_ii );
-	FREEF( imageinfo_destroy_nonheap, conv->ii );
+	IM_FREEF( imageinfo_destroy_nonheap, conv->repaint_ii );
+	IM_FREEF( imageinfo_destroy_nonheap, conv->display_ii );
+	IM_FREEF( imageinfo_destroy_nonheap, conv->visual_ii );
+	IM_FREEF( imageinfo_destroy_nonheap, conv->ii );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
@@ -373,8 +373,11 @@ conversion_make_display( Conversion *conv, IMAGE *in, IMAGE **mask_out )
 		}
 	}
 	else
-		if( im_render( in, out, mask, conv->tile_size, conv->tile_size, 
-			conversion_get_default_tiles( conv ),
+		if( im_render_fade( in, out, mask, 
+			conv->tile_size, conv->tile_size, 
+				conversion_get_default_tiles( conv ),
+			20, 10,
+			conv->priority,
 			conversion_paint_callback, conv ) ) {
 			im_close( out );
 			return( NULL );
@@ -394,19 +397,19 @@ conversion_make_bg( Conversion *conv, IMAGE *repaint, IMAGE *out )
 	IMAGE *t2 = im_open_local( out, "t2", "p" );
 	int i;
 
-	/* Make a 2x2 pixel image with the 102/153 pixels in.
+	/* Make a 2x2 pixel image with the tile pixels in.
 	 */
-	im_initdesc( tile, 2, 2, 
+	im_initdesc( tile, 2, 2,
 		repaint->Bands, IM_BBITS_BYTE, IM_BANDFMT_UCHAR,
 		IM_CODING_NONE, IM_TYPE_MULTIBAND, 
 		1.0, 1.0, 0, 0 );
 	if( im_setupout( tile ) )
 		return( -1 );
 	for( i = 0; i < tile->Bands; i++ ) {
-		((PEL *)(tile->data))[i] = 153;
-		((PEL *)(tile->data))[i + tile->Bands] = 102;
-		((PEL *)(tile->data))[i + tile->Bands * 2] = 102;
-		((PEL *)(tile->data))[i + tile->Bands * 3] = 153;
+		((PEL *)(tile->data))[i] = 100;
+		((PEL *)(tile->data))[i + tile->Bands] = 90;
+		((PEL *)(tile->data))[i + tile->Bands * 2] = 90;
+		((PEL *)(tile->data))[i + tile->Bands * 3] = 100;
 	}
 
 	/* Blow it up, tile it and trim it.
@@ -848,8 +851,8 @@ conversion_rebuild_repaint( Conversion *conv )
 			return;
 	}
 
-	FREEF( im_region_free, conv->ireg );
-	FREEF( imageinfo_destroy_nonheap, conv->repaint_ii );
+	IM_FREEF( im_region_free, conv->ireg );
+	IM_FREEF( imageinfo_destroy_nonheap, conv->repaint_ii );
 
 	if( display_im ) {
 		conv->repaint_ii = new_repaint_ii;
@@ -897,7 +900,7 @@ conversion_rebuild_display( Conversion *conv )
 		imageinfo_sub_add( new_display_ii, conv->visual_ii );
 	}
 
-	FREEF( imageinfo_destroy_nonheap, conv->display_ii );
+	IM_FREEF( imageinfo_destroy_nonheap, conv->display_ii );
 
 	if( visual_im ) {
 		conv->display_ii = new_display_ii;
@@ -946,15 +949,15 @@ conversion_rebuild_visual( Conversion *conv )
 		imageinfo_sub_add( new_visual_ii, conv->ii );
 
 		if( !(new_reg = im_region_create( im )) ) {
-			FREEF( im_region_free, new_reg );
+			IM_FREEF( im_region_free, new_reg );
 			return;
 		}
 	}
 
 	/* Junk old stuff.
 	 */
-	FREEF( im_region_free, conv->reg );
-	FREEF( imageinfo_destroy_nonheap, conv->visual_ii );
+	IM_FREEF( im_region_free, conv->reg );
+	IM_FREEF( imageinfo_destroy_nonheap, conv->visual_ii );
 
 	/* Install new stuff.
 	 */
@@ -1055,6 +1058,7 @@ conversion_init( Conversion *conv )
 	conv->area_changed_sid = 0;
 	conv->reg = NULL;
 	conv->synchronous = FALSE;
+	conv->priority = 0;
 	conv->visual_ii = NULL;
 	conv->display_ii = NULL;
 	conv->display_mag = 99999999;
@@ -1168,7 +1172,7 @@ conversion_set_image( Conversion *conv, Imageinfo *ii )
 		 */
 		FREESID( conv->changed_sid, conv->ii );
 		FREESID( conv->area_changed_sid, conv->ii );
-		FREEF( imageinfo_destroy_nonheap, conv->ii );
+		IM_FREEF( imageinfo_destroy_nonheap, conv->ii );
 		conv->image.width = -1;
 		conv->image.height = -1;
 
