@@ -178,7 +178,7 @@ filemodel_finalize( GObject *gobject )
 		NN( filemodel->filename ) );
 #endif /*DEBUG*/
 
-	FREE( filemodel->filename );
+	IM_FREE( filemodel->filename );
 	filemodel_unregister( filemodel );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
@@ -278,8 +278,9 @@ filemodel_save_all_xml( Filemodel *filemodel, const char *filename )
 
 	prettify_tree( xdoc );
 
-	if( (int) call_string_filename( (call_string_fn) xmlSaveFile, 
-		filename, xdoc, NULL, NULL ) == -1 ) {
+	if( GPOINTER_TO_INT( call_string_filename( 
+		(call_string_fn) xmlSaveFile, 
+		filename, xdoc, NULL, NULL ) ) == -1 ) {
 		error_top( _( "Save failed." ) );
 		error_sub( _( "Save of %s \"%s\" to file \"%s\" failed.\n%s" ),
 			G_OBJECT_TYPE_NAME( filemodel ), 
@@ -365,7 +366,7 @@ filemodel_class_init( FilemodelClass *class )
 	class->set_modified = filemodel_real_set_modified;
 	class->save_all = filemodel_real_save_all;
 	class->filetype = NULL;
-	class->default_filetype = 0;
+	class->filetype_pref = NULL;
 }
 
 static void
@@ -443,16 +444,16 @@ filemodel_load_all_xml( Filemodel *filemodel,
 		!xnode->nsDef ||
 		!isprefix( NAMESPACE, xnode->nsDef->href ) ) {
 		error_top( _( "Load failed." ) );
-		error_sub( _( "Can't load XML file \"%s\",\n"
-			"it's not a %s save file" ), 
+		error_sub( _( "Can't load XML file \"%s\", "
+			"it's not a %s save file." ), 
 			state->filename, PACKAGE );
 		return( FALSE );
 	}
 	if( sscanf( xnode->nsDef->href + strlen( NAMESPACE ) + 1, "%d.%d.%d",
 		&state->major, &state->minor, &state->micro ) != 3 ) {
 		error_top( _( "Load failed." ) );
-		error_sub( _( "Can't load XML file \"%s\",\n"
-			"Unable to extract version information from "
+		error_sub( _( "Can't load XML file \"%s\", "
+			"unable to extract version information from "
 			"namespace." ), state->filename );
 		return( FALSE );
 	}
@@ -464,8 +465,8 @@ filemodel_load_all_xml( Filemodel *filemodel,
 
 	if( !(xstart = get_node( xnode, tname )) ) {
 		error_top( _( "Load failed." ) );
-		error_sub( _( "Can't load XML file \"%s\",\n"
-			"The file does not contain a %s." ), 
+		error_sub( _( "Can't load XML file \"%s\", "
+			"the file does not contain a %s." ), 
 			state->filename, tname );
 		return( FALSE );
 	}
@@ -643,7 +644,7 @@ filemodel_inter_saveas_cb( iWindow *iwnd, void *client,
 	iWindowNotifyFn nfn, void *sys )
 {
 	Filemodel *filemodel = FILEMODEL( client );
-	FilemodelClass *filemodel_class = FILEMODEL_GET_CLASS( filemodel );
+	FilemodelClass *class = FILEMODEL_GET_CLASS( filemodel );
 
 	Filesel *filesel = FILESEL( filesel_new() );
 
@@ -654,7 +655,8 @@ filemodel_inter_saveas_cb( iWindow *iwnd, void *client,
 		NN( IOBJECT( filemodel )->name ) );
 	filesel_set_flags( filesel, FALSE, TRUE );
 	filesel_set_filetype( filesel, 
-		filemodel_class->filetype, filemodel_class->default_filetype ); 
+		class->filetype, 
+		watch_int_get( main_watchgroup, class->filetype_pref, 0 ) );
 	idialog_set_parent( IDIALOG( filesel ), GTK_WIDGET( iwnd ) );
 	filesel_set_done( filesel, filemodel_inter_save_cb, filemodel );
 	idialog_set_notify( IDIALOG( filesel ), nfn, sys );
@@ -682,7 +684,7 @@ filemodel_inter_save( iWindow *parent, Filemodel *filemodel )
 			filemodel_set_modified( filemodel, FALSE );
 			box_info( GTK_WIDGET( parent ), 
 				_( "Save successful." ),
-				_( "%s \"%s\" successfully saved\n"
+				_( "%s \"%s\" successfully saved "
 				"to file \"%s\"." ),
 				G_OBJECT_TYPE_NAME( filemodel ), 
 				NN( IOBJECT( filemodel )->name ),
@@ -733,7 +735,7 @@ filemodel_inter_savenempty_cb( iWindow *iwnd, void *client,
 				nfn, sys, 
 				_( "Object has been modified." ),
 				_( "%s \"%s\" has been modified since you "
-				"loaded it\nfrom file \"%s\".\n\n"
+				"loaded it from file \"%s\".\n\n"
 				"Do you want to save your changes?" ),
 				tname, 
 				NN( IOBJECT( filemodel )->name ),
@@ -823,7 +825,7 @@ filemodel_inter_loadas_cb( iWindow *iwnd, void *client,
 	iWindowNotifyFn nfn, void *sys )
 {
 	Filemodel *filemodel = FILEMODEL( client );
-	FilemodelClass *filemodel_class = FILEMODEL_GET_CLASS( filemodel );
+	FilemodelClass *class = FILEMODEL_GET_CLASS( filemodel );
 
 	Filesel *filesel = FILESEL( filesel_new() );
 
@@ -831,7 +833,8 @@ filemodel_inter_loadas_cb( iWindow *iwnd, void *client,
 		G_OBJECT_TYPE_NAME( filemodel ) );
 	filesel_set_flags( filesel, FALSE, TRUE );
 	filesel_set_filetype( filesel, 
-		filemodel_class->filetype, filemodel_class->default_filetype ); 
+		class->filetype, 
+		watch_int_get( main_watchgroup, class->filetype_pref, 0 ) );
 	idialog_set_parent( IDIALOG( filesel ), GTK_WIDGET( iwnd ) );
 	filesel_set_done( filesel, filemodel_inter_load_cb, filemodel );
 	idialog_set_notify( IDIALOG( filesel ), nfn, sys );
@@ -911,8 +914,7 @@ filemodel_set_auto_load( Filemodel *filemodel )
 
 		assert( p );
 
-		im_snprintf( buf, FILENAME_MAX, "$HOME" IM_DIR_SEP_STR
-			".$PACKAGE-$VERSION" IM_DIR_SEP_STR 
+		im_snprintf( buf, FILENAME_MAX, "$SAVEDIR" IM_DIR_SEP_STR
 			"start" IM_DIR_SEP_STR "%s", p + 1 );
 		filemodel_set_filename( filemodel, buf );
 	}
