@@ -33,14 +33,16 @@
 #define DEBUG
  */
 
-/* TIFF save possibilities.
+/* TIFF save possibilities. Needs to be kept in sync with the Option in
+ * preferences.
  */
 typedef enum {
 	TIFF_COMPRESSION_NONE = 0,	/* No compression */
 	TIFF_COMPRESSION_LZW,		/* Lempel-Ziv compression */
 	TIFF_COMPRESSION_DEFLATE,	/* Zip (deflate) compression */
 	TIFF_COMPRESSION_PACKBITS,	/* Packbits compression */
-	TIFF_COMPRESSION_JPEG		/* JPEG compression */
+	TIFF_COMPRESSION_JPEG,		/* JPEG compression */
+	TIFF_COMPRESSION_CCITTFAX4	/* Fax compression */
 } TiffCompression;
 
 typedef enum {
@@ -174,6 +176,7 @@ decode_tiff_compression( TiffCompression tc )
 	case TIFF_COMPRESSION_DEFLATE:	return( "deflate" );
 	case TIFF_COMPRESSION_PACKBITS:	return( "packbits" );
 	case TIFF_COMPRESSION_JPEG:	return( "jpeg" );
+	case TIFF_COMPRESSION_CCITTFAX4:return( "ccittfax4" );
 
 	case TIFF_COMPRESSION_NONE:
 	default:
@@ -339,7 +342,7 @@ filesel_destroy( GtkObject *object )
 	filesel = FILESEL( object );
 
 	filesel_all = g_slist_remove( filesel_all, filesel );
-	FREEF( g_free, filesel->current_dir );
+	IM_FREEF( g_free, filesel->current_dir );
 
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
@@ -380,6 +383,7 @@ filesel_add_volume( const char *dir, Filesel *filesel )
         expand_variables( dir, buf );
         nativeize_path( buf );
 	absoluteize_path( buf );
+	canonicalize_path( buf );
 	gtk_file_chooser_add_shortcut_folder( 
 		GTK_FILE_CHOOSER( filesel->chooser ), buf, NULL );
 
@@ -606,11 +610,11 @@ filesel_map_filename_multi( Filesel *filesel,
 		void *res;
 
 		if( (res = fn( filesel, filename, a, b )) ) {
-			FREEF( slist_free_all, names );
+			IM_FREEF( slist_free_all, names );
 			return( res );
 		}
 	}
-	FREEF( slist_free_all, names );
+	IM_FREEF( slist_free_all, names );
 
 	return( NULL );
 }
@@ -821,13 +825,11 @@ filesel_build( GtkWidget *widget )
 static void
 filesel_class_init( FileselClass *class )
 {
-	GtkObjectClass *object_class;
-	iWindowClass *iwindow_class;
-
-	object_class = (GtkObjectClass *) class;
-	iwindow_class = (iWindowClass *) class;
+	GtkObjectClass *object_class = (GtkObjectClass *) class;
+	iWindowClass *iwindow_class = (iWindowClass *) class;
 
 	object_class->destroy = filesel_destroy;
+
 	iwindow_class->build = filesel_build;
 
 	parent_class = g_type_class_peek_parent( class );
@@ -920,6 +922,12 @@ filesel_trigger( iWindow *iwnd, Filesel *filesel,
 	 */
 	iWindowSusp *susp = iwindow_susp_new( NULL, iwnd, filesel, nfn, sys );
 
+	/* If there's a filetype pref, update it.
+	 */
+	if( filesel->type_pref )
+		prefs_set( filesel->type_pref, 
+			"%d", filesel_get_filetype( filesel ) );
+
 	set_hourglass();
 	filesel->done_cb( IWINDOW( filesel ), 
 		filesel->client, filesel_trigger2, susp );
@@ -954,6 +962,7 @@ filesel_init( Filesel *filesel )
 	filesel->browse = NULL;
 	filesel->type = NULL;
 	filesel->default_type = 0;
+	filesel->type_pref = NULL;
 	filesel->current_dir = NULL;
 	filesel->done_cb = NULL;
 	filesel->client = NULL;
@@ -1035,10 +1044,9 @@ filesel_done_cb( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 		g_free( filename );
 	}
 	else
-		/* No file selected, or not a valid file. Could pop an error,
-		 * but prettier to ignore.
+		/* No file selected, or not a valid file. 
 		 */
-		nfn( sys, IWINDOW_FALSE );
+		nfn( sys, IWINDOW_ERROR );
 }
 
 void 
@@ -1057,6 +1065,13 @@ filesel_set_filetype( Filesel *filesel,
 {
 	int i;
 
+	/* Reset the widget if it's there.
+	 */
+	if( filesel->chooser )
+		gtk_file_chooser_set_filter( 
+			GTK_FILE_CHOOSER( filesel->chooser ),
+			filesel->filter[default_type] );
+
 	filesel->type = type;
 	filesel->default_type = default_type;
 
@@ -1065,6 +1080,13 @@ filesel_set_filetype( Filesel *filesel,
 	filesel->ntypes = i;
 }
 
+void 
+filesel_set_filetype_pref( Filesel *filesel, 
+	const char *type_pref )
+{
+	filesel->type_pref = type_pref;
+}
+
 void
 filesel_set_multi( Filesel *filesel, gboolean multi )
 {
