@@ -162,17 +162,19 @@ iimageview_drag_data_received( GtkWidget *widget, GdkDragContext *context,
 			row_qualified_name_relative( row->top_row->sym, 
 				from_row, &buf );
 
-			itext_set_formula( itext, buf_all( &buf ) );
-			itext_set_edited( itext, TRUE );
-			(void) expr_dirty( row->expr, link_serial_new() );
-			filemodel_set_modified( FILEMODEL( row->ws ), TRUE );
+			if( itext_set_formula( itext, buf_all( &buf ) ) ) {
+				itext_set_edited( itext, TRUE );
+				(void) expr_dirty( row->expr, 
+					link_serial_new() );
+				filemodel_set_modified( FILEMODEL( row->ws ), 
+					TRUE );
+				symbol_recalculate_all();
+			}
 
 			/* Usually the drag-from row will be selected, very
 			 * annoying. Select the drag-to row.
 			 */
 			row_select( row );
-
-			symbol_recalculate_all();
 		}
 	}
 }
@@ -214,6 +216,7 @@ iimageview_refresh( vObject *vobject )
 {
 	iImageview *iimageview = IIMAGEVIEW( vobject );
 	iImage *iimage = IIMAGE( vobject->iobject );
+	Row *row = HEAPMODEL( iimage )->row;
 	int w, h;
 	gboolean enabled;
 	double scale, offset;
@@ -235,8 +238,8 @@ iimageview_refresh( vObject *vobject )
 	 * there's a setting for this image, override with that.
 	 */
 	enabled = DISPLAY_CONVERT;
-	scale = CONVERSION_SCALE;
-	offset = CONVERSION_OFFSET;
+	scale = row->ws->scale;
+	offset = row->ws->offset;
 
 	if( iimage->window_width != -1 ) {
 		enabled = iimage->show_convert;
