@@ -89,7 +89,7 @@ imageinfogroup_finalize( GObject *gobject )
 {
 	Imageinfogroup *imageinfogroup = IMAGEINFOGROUP( gobject );
 
-	FREEF( g_hash_table_destroy, imageinfogroup->filename_hash );
+	IM_FREEF( g_hash_table_destroy, imageinfogroup->filename_hash );
 
 	G_OBJECT_CLASS( imageinfogroup_parent_class )->finalize( gobject );
 }
@@ -234,7 +234,8 @@ imageinfogroup_lookup( Imageinfogroup *imageinfogroup, const char *filename )
 /* Our signals. 
  */
 enum {
-	SIG_AREA_CHANGED,	/* Area of image has changed */
+	SIG_AREA_CHANGED,	/* Area of image has changed: update screen */
+	SIG_AREA_PAINTED,	/* Area of image has been painted */
 	SIG_UNDO_CHANGED,	/* Undo/redo state has changed */
 	SIG_FILE_CHANGED,	/* Underlying file seems to have changed */
 	SIG_LAST
@@ -252,7 +253,7 @@ imageinfo_print( Imageinfo *imageinfo )
 	printf( " \"%s\" mtime = %d (0x%x)\n",
 		IOBJECT( imageinfo )->name,
 		(int) imageinfo->mtime,
-		(unsigned int) imageinfo );
+		GPOINTER_TO_UINT( imageinfo ) );
 }
 #endif
 
@@ -273,6 +274,23 @@ imageinfo_area_changed( Imageinfo *imageinfo, Rect *dirty )
 	return( NULL );
 }
 
+void *
+imageinfo_area_painted( Imageinfo *imageinfo, Rect *dirty )
+{
+	assert( IS_IMAGEINFO( imageinfo ) );
+
+#ifdef DEBUG
+	printf( "imageinfo_area_painted: left = %d, top = %d, "
+		"width = %d, height = %d\n",
+		dirty->left, dirty->top, dirty->width, dirty->height );
+#endif /*DEBUG*/
+
+	g_signal_emit( G_OBJECT( imageinfo ), 
+		imageinfo_signals[SIG_AREA_PAINTED], 0, dirty );
+
+	return( NULL );
+}
+
 static void *
 imageinfo_undo_changed( Imageinfo *imageinfo )
 {
@@ -305,7 +323,8 @@ imageinfo_file_changed( Imageinfo *imageinfo )
 void *
 imageinfo_dump( Imageinfo *imageinfo )
 {
-	printf( "imageinfo->heap = 0x%x\n", (unsigned int) imageinfo->heap );
+	printf( "imageinfo->heap = 0x%x\n", 
+		GPOINTER_TO_UINT( imageinfo->heap ) );
 	if( imageinfo->heap ) 
 		iobject_dump( IOBJECT( imageinfo->heap ) );
 
@@ -314,13 +333,14 @@ imageinfo_dump( Imageinfo *imageinfo )
 	printf( "imageinfo->count (# non-heap pointers) = %d\n", 
 		imageinfo->count );
 
-	printf( "imageinfo->im = 0x%x\n", (unsigned int) imageinfo->im );
+	printf( "imageinfo->im = 0x%x\n", 
+		GPOINTER_TO_UINT( imageinfo->im ) );
 	printf( "imageinfo->mapped_im  = 0x%x\n", 
-		(unsigned int) imageinfo->mapped_im );
+		GPOINTER_TO_UINT( imageinfo->mapped_im ) );
 	printf( "imageinfo->identity_lut = 0x%x\n", 
-		(unsigned int) imageinfo->identity_lut );
+		GPOINTER_TO_UINT( imageinfo->identity_lut ) );
 	printf( "imageinfo->underlying = 0x%x\n", 
-		(unsigned int) imageinfo->underlying );
+		GPOINTER_TO_UINT( imageinfo->underlying ) );
 
 	printf( "imageinfo->subii (imageinfo depending on us) = %d\n",
 		g_slist_length( imageinfo->subii ) );
@@ -453,8 +473,8 @@ imageinfo_get_underlying( Imageinfo *imageinfo )
 static void
 imageinfo_undofragment_free( Undofragment *frag )
 {
-	FREEF( im_close, frag->im );
-	FREE( frag );
+	IM_FREEF( im_close, frag->im );
+	IM_FREE( frag );
 }
 
 /* Free an undo buffer.
@@ -464,8 +484,8 @@ imageinfo_undobuffer_free( Undobuffer *undo )
 {
 	slist_map( undo->frags, 
 		(SListMapFn) imageinfo_undofragment_free, NULL );
-	FREEF( g_slist_free, undo->frags );
-	FREE( undo );
+	IM_FREEF( g_slist_free, undo->frags );
+	IM_FREE( undo );
 }
 
 /* Free all undo information attached to an imageinfo.
@@ -475,11 +495,11 @@ imageinfo_undo_free( Imageinfo *imageinfo )
 {	
 	slist_map( imageinfo->redo, 
 		(SListMapFn) imageinfo_undobuffer_free, NULL );
-	FREEF( g_slist_free, imageinfo->redo );
+	IM_FREEF( g_slist_free, imageinfo->redo );
 	slist_map( imageinfo->undo, 
 		(SListMapFn) imageinfo_undobuffer_free, NULL );
-	FREEF( g_slist_free, imageinfo->undo );
-	FREEF( imageinfo_undobuffer_free, imageinfo->cundo );
+	IM_FREEF( g_slist_free, imageinfo->undo );
+	IM_FREEF( imageinfo_undobuffer_free, imageinfo->cundo );
 }
 
 static void
@@ -489,7 +509,7 @@ imageinfo_dispose_eval( Imageinfo *imageinfo )
 		gtk_widget_destroy( GTK_WIDGET( imageinfo->eval_progress ) );
 		imageinfo->eval_progress = NULL;
 	}
-	FREEF( g_timer_destroy, imageinfo->eval_timer );
+	IM_FREEF( g_timer_destroy, imageinfo->eval_timer );
 	if( imageinfo->eval_parent ) {
 		g_object_remove_weak_pointer( 
 			G_OBJECT( imageinfo->eval_parent ), 
@@ -522,7 +542,7 @@ imageinfo_dispose( GObject *gobject )
 
 	imageinfo_dispose_eval( imageinfo );
 
-	FREEFI( g_source_remove, imageinfo->check_tid );
+	IM_FREEF( g_source_remove, imageinfo->check_tid );
 
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
@@ -548,9 +568,9 @@ imageinfo_finalize( GObject *gobject )
 		 */
 		im_strncpy( name, im->filename, FILENAME_MAX - 5 );
 
-	FREEF( im_close, imageinfo->im );
-	FREEF( im_close, imageinfo->mapped_im );
-	FREEF( im_close, imageinfo->identity_lut );
+	IM_FREEF( im_close, imageinfo->im );
+	IM_FREEF( im_close, imageinfo->mapped_im );
+	IM_FREEF( im_close, imageinfo->identity_lut );
 
 	if( imageinfo->dfile && isfile ) {
 #ifdef DEBUG_OPEN
@@ -563,7 +583,7 @@ imageinfo_finalize( GObject *gobject )
 		iobject_changed( IOBJECT( main_imageinfogroup ) );
 	}
 
-	FREEF( imageinfo_destroy_nonheap, imageinfo->underlying );
+	IM_FREEF( imageinfo_destroy_nonheap, imageinfo->underlying );
 
 	imageinfo_undo_free( imageinfo );
 
@@ -628,6 +648,15 @@ imageinfo_real_area_changed( Imageinfo *imageinfo, Rect *dirty )
 {
 }
 
+static void
+imageinfo_real_area_painted( Imageinfo *imageinfo, Rect *dirty )
+{
+	/* Cache attaches to this signal and invalidates on paint. Trigger a
+	 * repaint in turn.
+	 */
+	imageinfo_area_changed( imageinfo, dirty );
+}
+
 static void
 imageinfo_real_undo_changed( Imageinfo *imageinfo )
 {
@@ -649,6 +678,7 @@ imageinfo_class_init( ImageinfoClass *class )
 	gobject_class->finalize = imageinfo_finalize;
 
 	class->area_changed = imageinfo_real_area_changed;
+	class->area_painted = imageinfo_real_area_painted;
 	class->undo_changed = imageinfo_real_undo_changed;
 	class->file_changed = imageinfo_real_file_changed;
 
@@ -662,6 +692,14 @@ imageinfo_class_init( ImageinfoClass *class )
 		g_cclosure_marshal_VOID__POINTER,
 		G_TYPE_NONE, 1,
 		G_TYPE_POINTER );
+	imageinfo_signals[SIG_AREA_PAINTED] = g_signal_new( "area_painted",
+		G_OBJECT_CLASS_TYPE( gobject_class ),
+		G_SIGNAL_RUN_FIRST,
+		G_STRUCT_OFFSET( ImageinfoClass, area_painted ),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__POINTER,
+		G_TYPE_NONE, 1,
+		G_TYPE_POINTER );
 	imageinfo_signals[SIG_UNDO_CHANGED] = g_signal_new( "undo_changed",
 		G_OBJECT_CLASS_TYPE( gobject_class ),
 		G_SIGNAL_RUN_FIRST,
@@ -682,7 +720,7 @@ static void
 imageinfo_init( Imageinfo *imageinfo )
 {
 #ifdef DEBUG_MAKE
-	printf( "imageinfo_init: 0x%x\n", (unsigned int) imageinfo );
+	printf( "imageinfo_init: 0x%x\n", GPOINTER_TO_UINT( imageinfo ) );
 #endif /*DEBUG_MAKE*/
 
 	imageinfo->heap = NULL;
@@ -761,7 +799,11 @@ imageinfo_new( Imageinfogroup *imageinfogroup,
 	heap->itable = g_slist_prepend( heap->itable, imageinfo );
 
 	if( !name ) {
-		temp_name( buf, "v" );
+		if( !temp_name( buf, "v" ) ) 
+			/* Will be freed on next GC.
+			 */
+			return( NULL );
+
 		name = buf;
 	}
 	iobject_set( IOBJECT( imageinfo ), name, NULL );
@@ -1041,7 +1083,7 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 		/* Junk the old history, we don't want stuff from before this
 		 * session confusing the balancer.
 		 */
-		FREE( im->Hist );
+		IM_FREE( im->Hist );
 		im_histlin( im, "" );
 
 		if( !(imageinfo = imageinfo_new( open->imageinfogroup, 
@@ -1133,7 +1175,7 @@ imageinfo_attach_check( Imageinfo *imageinfo )
 #endif /*DEBUG_CHECK*/
 	}
 	else
-		FREEFI( g_source_remove, imageinfo->check_tid );
+		IM_FREEF( g_source_remove, imageinfo->check_tid );
 }
 
 /* Open a filename for input.
@@ -1194,7 +1236,7 @@ imageinfo_new_output( Imageinfogroup *imageinfogroup,
 
         if( !(imageinfo = (Imageinfo *) call_string_filename( 
 		(call_string_fn) imageinfo_open_image_output, 
-		filename, imageinfogroup, heap, (char *) filename )) ) {
+		name, imageinfogroup, heap, (char *) filename )) ) {
                 error_top( _( "Unable to write to file." ) );
 		error_sub( _( "Unable to open \"%s\" for write." ), filename );
 		error_vips();
@@ -1375,9 +1417,9 @@ imageinfo_file( Imageinfo *imageinfo, GtkWidget *parent )
 		return( FALSE );
 	}
 
-	FREEF( im_close, imageinfo->im );
-	FREEF( im_close, imageinfo->mapped_im );
-	FREEF( imageinfo_destroy_nonheap, imageinfo->underlying );
+	IM_FREEF( im_close, imageinfo->im );
+	IM_FREEF( im_close, imageinfo->mapped_im );
+	IM_FREEF( imageinfo_destroy_nonheap, imageinfo->underlying );
 
 	imageinfo->im = im;
 	imageinfo->dfile = TRUE;
@@ -1439,12 +1481,11 @@ imageinfo_check_paintable( Imageinfo *imageinfo, GtkWidget *parent,
 			nfn, sys,
 			_( "Modify" ),
 			_( "Modify disc file?" ),
-			_( "This image is being shown directly from "
-				"the disc file:\n\n"
+			_( "This image is being shown directly from the "
+			"disc file:\n\n"
 			"   %s\n\n"
-			"If you paint on this file, it will be "
-				"permanently changed.\n"
-			"If something goes wrong, you may lose work.\n"
+			"If you paint on this file, it will be permanently "
+			"changed. If something goes wrong, you may lose work. "
 			"Are you sure you want to modify this file?" ),
 			IOBJECT( imageinfo )->name );
 		idialog_set_iobject( idlg, IOBJECT( imageinfo ) );
@@ -1773,7 +1814,7 @@ imageinfo_undo_mark( Imageinfo *imageinfo )
 	 */
 	slist_map( imageinfo->redo, 
 		(SListMapFn) imageinfo_undobuffer_free, NULL );
-	FREEF( g_slist_free, imageinfo->redo );
+	IM_FREEF( g_slist_free, imageinfo->redo );
 
 	/* Trim undo buffer.
 	 */
@@ -1901,7 +1942,7 @@ imageinfo_undofragment_paste( Undofragment *frag )
 	IMAGE *im = imageinfo_get( FALSE, imageinfo ); 
 
 	im_insertplace( im, frag->im, frag->pos.left, frag->pos.top );
-	imageinfo_area_changed( imageinfo, &frag->pos );
+	imageinfo_area_painted( imageinfo, &frag->pos );
 
 	return( NULL );
 }
@@ -2052,7 +2093,7 @@ imageinfo_paint_line( Imageinfo *imageinfo, Imageinfo *ink,
 		return( FALSE );
 	}
 
-	imageinfo_area_changed( imageinfo, &dirty );
+	imageinfo_area_painted( imageinfo, &dirty );
 
 	return( TRUE );
 }
@@ -2086,7 +2127,7 @@ imageinfo_paint_smudge( Imageinfo *imageinfo,
 		return( FALSE );
 	}
 
-	imageinfo_area_changed( imageinfo, &dirty );
+	imageinfo_area_painted( imageinfo, &dirty );
 
 	return( TRUE );
 }
@@ -2124,7 +2165,7 @@ imageinfo_paint_flood( Imageinfo *imageinfo, Imageinfo *ink,
 		return( FALSE );
 	}
 
-	imageinfo_area_changed( imageinfo, &dirty );
+	imageinfo_area_painted( imageinfo, &dirty );
 
 	return( TRUE );
 }
@@ -2147,7 +2188,7 @@ imageinfo_paint_dropper( Imageinfo *imageinfo, Imageinfo *ink, int x, int y )
 	dirty.width = ink_im->Xsize;
 	dirty.height = ink_im->Ysize;
 
-	imageinfo_area_changed( ink, &dirty );
+	imageinfo_area_painted( ink, &dirty );
 
 	return( TRUE );
 }
@@ -2169,7 +2210,7 @@ imageinfo_paint_rect( Imageinfo *imageinfo, Imageinfo *ink, Rect *area )
 		return( FALSE );
 	}
 
-	imageinfo_area_changed( imageinfo, area );
+	imageinfo_area_painted( imageinfo, area );
 
 	return( TRUE );
 }
@@ -2232,7 +2273,7 @@ imageinfo_paint_mask( Imageinfo *imageinfo,
 		return( FALSE );
 	}
 
-	imageinfo_area_changed( imageinfo, &dirty );
+	imageinfo_area_painted( imageinfo, &dirty );
 
 	return( TRUE );
 }
@@ -2432,7 +2473,7 @@ imageinfo_from_text( Imageinfo *imageinfo, const char *text )
 	dirty.top = 0;
 	dirty.width = 1;
 	dirty.height = 1;
-	imageinfo_area_changed( imageinfo, &dirty );
+	imageinfo_area_painted( imageinfo, &dirty );
 
 	return( TRUE );
 }
@@ -2663,7 +2704,7 @@ imageinfo_from_rgb( Imageinfo *imageinfo, double *rgb )
 	dirty.top = 0;
 	dirty.width = 1;
 	dirty.height = 1;
-	imageinfo_area_changed( imageinfo, &dirty );
+	imageinfo_area_painted( imageinfo, &dirty );
 }
 
 /* Widgets for colour edit.
@@ -2688,7 +2729,7 @@ imageinfo_colour_done_cb( iWindow *iwnd, void *client,
 	gtk_color_selection_get_color( 
 		GTK_COLOR_SELECTION( eds->colour_widget ), rgb );
 
-	/* This will emit "area_changed" on our imageinfo.
+	/* This will emit "area_painted" on our imageinfo.
 	 */
 	imageinfo_from_rgb( imageinfo, rgb );
 
@@ -2737,4 +2778,3 @@ imageinfo_colour_edit( GtkWidget *parent, Imageinfo *imageinfo )
 
 	gtk_widget_show( GTK_WIDGET( idlg ) );
 }
-
