@@ -152,9 +152,17 @@ struct _Imageinfo {
 typedef struct _ImageinfoClass {
 	iContainerClass parent_class;
 
-	/* Redisplay on paint.
+	/* An area of the screen needs repainting.
 	 */
 	void (*area_changed)( Imageinfo *, Rect * );
+
+	/* An area of the image has been paintboxed ... invalidate caches and
+	 * trigger area_changed.
+	 */
+	void (*area_painted)( Imageinfo *, Rect * );
+
+	/* Update undo/redo button sensitivities.
+	 */
 	void (*undo_changed)( Imageinfo * );
 
 	/* The underlying file has changed ... higher levels should try to 
@@ -164,6 +172,7 @@ typedef struct _ImageinfoClass {
 } ImageinfoClass;
 
 void *imageinfo_area_changed( Imageinfo *imageinfo, Rect *dirty );
+void *imageinfo_area_painted( Imageinfo *imageinfo, Rect *dirty );
 void *imageinfo_dump( Imageinfo *imageinfo );
 void imageinfo_check_all_destroyed( Imageinfogroup *imageinfogroup );
 
