@@ -42,20 +42,20 @@ static iWindowClass *parent_class = NULL;
 /* All the magnification menus we have.
  */
 typedef struct _ImageviewMagmenu {
-	const char *path;
+	const char *name;
 	int mag;
 } ImageviewMagmenu;
 
 static const ImageviewMagmenu imageview_mags[] = {
-	{ "/View/Zoom/6%", -16 },
-	{ "/View/Zoom/12%", -8 },
-	{ "/View/Zoom/25%", -4 },
-	{ "/View/Zoom/50%", -2 },
-	{ "/View/Zoom/100%", 1 },
-	{ "/View/Zoom/200%", 2 },
-	{ "/View/Zoom/400%", 4 },
-	{ "/View/Zoom/800%", 8 },
-	{ "/View/Zoom/1600%", 16 }
+	{ "Zoom6Mode", -16 },
+	{ "Zoom12Mode", -8 },
+	{ "Zoom25Mode", -4 },
+	{ "Zoom50Mode", -2 },
+	{ "Zoom100Mode", 1 },
+	{ "Zoom200Mode", 2 },
+	{ "Zoom400Mode", 4 },
+	{ "Zoom800Mode", 8 },
+	{ "Zoom1600Mode", 16 }
 };
 
 static void
@@ -75,7 +75,6 @@ imageview_destroy( GtkObject *object )
 	/* My instance destroy stuff.
 	 */
 	UNREF( iv->imagemodel );
-	UNREF( iv->ifac );
 
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
@@ -100,7 +99,6 @@ static void
 imageview_init( Imageview *iv )
 {
 	iv->imagemodel = NULL;
-	iv->ifac = NULL;
 }
 
 GtkType
@@ -130,24 +128,30 @@ static void
 imageview_refresh_title( Imageview *iv )
 {
 	Imagemodel *imagemodel = iv->imagemodel;
-	Conversion *conv = imagemodel->conv;
 	iImage *iimage = imagemodel->iimage;
-	Imageinfo *ii = iimage->instance.ii;
 	Row *row = HEAPMODEL( iimage )->row;
 	Workspace *ws = row_get_workspace( row );
 
-	BufInfo buf;
-	char txt[512];
+	/* Can come here during ws destroy.
+	 */
+	if( ws ) {
+		Conversion *conv = imagemodel->conv;
+		Imageinfo *ii = iimage->instance.ii;
 
-	buf_init_static( &buf, txt, 512 );
-	row_qualified_name_relative( ws->sym, row, &buf );
+		BufInfo buf;
+		char txt[512];
 
-	if( ii && IOBJECT( ii )->name && ii->from_file )
-		buf_appendf( &buf, " - %s", IOBJECT( ii )->name );
+		buf_init_static( &buf, txt, 512 );
+		row_qualified_name_relative( ws->sym, row, &buf );
 
-	buf_appendf( &buf, " - %.0f%%", 100.0 * conversion_dmag( conv->mag ) );
+		if( ii && IOBJECT( ii )->name && ii->from_file )
+			buf_appendf( &buf, " - %s", IOBJECT( ii )->name );
 
-	iwindow_set_title( IWINDOW( iv ), "%s", buf_all( &buf ) );
+		buf_appendf( &buf, " - %.0f%%", 
+			100.0 * conversion_dmag( conv->mag ) );
+
+		iwindow_set_title( IWINDOW( iv ), "%s", buf_all( &buf ) );
+	}
 }
 
 /* The model has changed ... update our menus and titlebar.
@@ -156,56 +160,40 @@ static void
 imageview_imagemodel_changed_cb( Imagemodel *imagemodel, Imageview *iv )
 {
 	Conversion *conv = imagemodel->conv;
-	GtkWidget *item;
+
+	GtkAction *action;
 	int i;
 
+	action = gtk_action_group_get_action( iv->action_group, 
+		"Status" );
+	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
+		imagemodel->show_status );
+
+	action = gtk_action_group_get_action( iv->action_group, 
+		"Control" );
+	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
+		imagemodel->show_convert );
+
+	action = gtk_action_group_get_action( iv->action_group, 
+		"Paint" );
+	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
+		imagemodel->show_paintbox );
+
+	action = gtk_action_group_get_action( iv->action_group, 
+		"Rulers" );
+	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
+		imagemodel->show_rulers );
+
 	for( i = 0; i < IM_NUMBER( imageview_mags ); i++ )
 		if( conv->mag == imageview_mags[i].mag ) {
-			item = gtk_item_factory_get_widget( iv->ifac,
-				imageview_mags[i].path );
-			gtk_check_menu_item_set_active( 
-				GTK_CHECK_MENU_ITEM( item ), TRUE );
+			action = gtk_action_group_get_action( iv->action_group,
+				imageview_mags[i].name );
+			gtk_toggle_action_set_active( 
+				GTK_TOGGLE_ACTION( action ),
+				TRUE );
+			break;
 		}
 
-	item = gtk_item_factory_get_widget( iv->ifac, 
-		"/View/Toolbar/Display control" );
-	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( item ), 
-		imagemodel->show_convert );
-
-	item = gtk_item_factory_get_widget( iv->ifac, "/View/Toolbar/Status" );
-	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( item ), 
-		imagemodel->show_status );
-
-	item = gtk_item_factory_get_widget( iv->ifac, "/View/Toolbar/Paint" );
-	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( item ), 
-		imagemodel->show_paintbox );
-
-	item = gtk_item_factory_get_widget( iv->ifac, "/View/Toolbar/Rulers" );
-	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( item ),
-		imagemodel->show_rulers );
-
-	if( imagemodel->state == IMAGEMODEL_SELECT )
-		item = gtk_item_factory_get_widget( iv->ifac, 
-			"/View/Mode/Select" );
-	else if( imagemodel->state == IMAGEMODEL_PAN )
-		item = gtk_item_factory_get_widget( iv->ifac, 
-			"/View/Mode/Pan" );
-	else if( imagemodel->state == IMAGEMODEL_MAGIN )
-		item = gtk_item_factory_get_widget( iv->ifac, 
-			"/View/Mode/Zoom In" );
-	else if( imagemodel->state == IMAGEMODEL_MAGOUT )
-		item = gtk_item_factory_get_widget( iv->ifac, 
-			"/View/Mode/Zoom Out" );
-	else if( imagemodel_state_paint( imagemodel->state ) )
-		item = gtk_item_factory_get_widget( iv->ifac, 
-			"/View/Mode/Paint" );
-	else
-		item = NULL;
-
-	if( item )
-		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( item ),
-			TRUE );
-
 	imageview_refresh_title( iv );
 }
 
@@ -217,17 +205,8 @@ imageview_imagemodel_destroy_cb( Imagemodel *imagemodel, Imageview *iv )
 	gtk_widget_destroy( GTK_WIDGET( iv ) );
 }
 
-static void
-imageview_replace_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
-{
-	Imageview *iv = IMAGEVIEW( callback_data );
-	Imagemodel *imagemodel = iv->imagemodel;
-	iImage *iimage = imagemodel->iimage;
-
-	classmodel_graphic_replace( CLASSMODEL( iimage ), GTK_WIDGET( iv ) );
-}
-
+/* Region class names indexed by iRegionType.
+ */
 static const char *imageview_region_name[] = {
 	CLASS_MARK,
 	CLASS_HGUIDE,
@@ -237,17 +216,41 @@ static const char *imageview_region_name[] = {
 	CLASS_AREA
 };
 
-static void
-imageview_new_arrow_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+/* Look up a iRegionType from an action name.
+ */
+static iRegionType
+imageview_get_region_type( GtkAction *action )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
+	/* Action names indexed by iRegionType.
+	 */
+	static const char *action_names[] = {
+		"NewMark",
+		"NewHGuide",
+		"NewVGuide",
+		"NewArrow",
+		"NewRegion"
+	};
+
+	const char *name = gtk_action_get_name( action );
+
+	int i;
+
+	for( i = 0; i < IM_NUMBER( action_names ); i++ )
+		if( strcmp( name, action_names[i] ) == 0 )
+			return( (iRegionType) i );
+
+	assert( FALSE );
+}
+
+static void
+imageview_new_arrow2_action_cb( GtkAction *action, Imageview *iv )
+{
+	iRegionType rt = imageview_get_region_type( action );
 	Imagemodel *imagemodel = iv->imagemodel;
 	iImage *iimage = imagemodel->iimage;
 	Row *row = HEAPMODEL( iimage )->row;
 	Workspace *ws = row_get_workspace( row );
 	Conversion *conv = imagemodel->conv;
-	iRegionType rt = (iRegionType) callback_action;
 	int dx = imagemodel->visible.left + imagemodel->visible.width / 2;
 	int dy = imagemodel->visible.top + imagemodel->visible.height / 2;
 
@@ -287,16 +290,14 @@ imageview_new_arrow_ifcb( gpointer callback_data, guint callback_action,
 }
 
 static void
-imageview_new_region_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_new_arrow4_action_cb( GtkAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
+	iRegionType rt = imageview_get_region_type( action );
 	Imagemodel *imagemodel = iv->imagemodel;
 	iImage *iimage = imagemodel->iimage;
 	Row *row = HEAPMODEL( iimage )->row;
 	Workspace *ws = row_get_workspace( row );
 	Conversion *conv = imagemodel->conv;
-	iRegionType rt = (iRegionType) callback_action;
 
 	Rect dr, ir;
 	char txt[MAX_STRSIZE];
@@ -326,10 +327,17 @@ imageview_new_region_ifcb( gpointer callback_data, guint callback_action,
 }
 
 static void
-imageview_save_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_replace_action_cb( GtkAction *action, Imageview *iv )
+{
+	Imagemodel *imagemodel = iv->imagemodel;
+	iImage *iimage = imagemodel->iimage;
+
+	classmodel_graphic_replace( CLASSMODEL( iimage ), GTK_WIDGET( iv ) );
+}
+
+static void
+imageview_save_action_cb( GtkAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
 	Imagemodel *imagemodel = iv->imagemodel;
 	iImage *iimage = imagemodel->iimage;
 
@@ -337,10 +345,22 @@ imageview_save_ifcb( gpointer callback_data, guint callback_action,
 }
 
 static void
-imageview_header_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_recalc_action_cb( GtkAction *action, Imageview *iv )
+{
+	Imagemodel *imagemodel = iv->imagemodel;
+	iImage *iimage = imagemodel->iimage;
+	Row *row = HEAPMODEL( iimage )->row;
+
+        workspace_deselect_all( row->ws );
+        row_select( row );
+        if( !workspace_selected_recalc( row->ws ) )
+                box_alert( GTK_WIDGET( iv ) );
+        workspace_deselect_all( row->ws );
+}
+
+static void
+imageview_header_action_cb( GtkAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
 	Imagemodel *imagemodel = iv->imagemodel;
 	iImage *iimage = imagemodel->iimage;
 	Row *row = HEAPMODEL( iimage )->row;
@@ -364,77 +384,8 @@ imageview_header_ifcb( gpointer callback_data, guint callback_action,
 }
 
 static void
-imageview_quit_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_zoom_in_action_cb( GtkAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
-
-	iwindow_kill( IWINDOW( iv ) );
-}
-
-static void
-imageview_recalc_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
-{
-	Imageview *iv = IMAGEVIEW( callback_data );
-	Imagemodel *imagemodel = iv->imagemodel;
-	iImage *iimage = imagemodel->iimage;
-	Row *row = HEAPMODEL( iimage )->row;
-
-        workspace_deselect_all( row->ws );
-        row_select( row );
-        if( !workspace_selected_recalc( row->ws ) )
-                box_alert( GTK_WIDGET( iv ) );
-        workspace_deselect_all( row->ws );
-}
-
-static void
-imageview_mag_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
-{
-	Imageview *iv = IMAGEVIEW( callback_data );
-	int mag = (int) callback_action;
-	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM( widget );
-
-	if( item->active && iv->ip ) 
-		imagepresent_zoom_to( iv->ip, mag );
-}
-
-/* ... make sure the specified mode is set.
- */
-static void
-imageview_mode_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
-{
-	Imageview *iv = IMAGEVIEW( callback_data );
-	Imagemodel *imagemodel = iv->imagemodel;
-	ImagemodelState state = (ImagemodelState) callback_action;
-	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM( widget );
-
-	if( item->active ) 
-		imagemodel_set_state( imagemodel, state, widget );
-}
-
-/* Make sure we're in one of the paint modes.
- */
-static void
-imageview_mode_paint_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
-{
-	Imageview *iv = IMAGEVIEW( callback_data );
-	Imagemodel *imagemodel = iv->imagemodel;
-	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM( widget );
-
-	if( item->active && 
-		!imagemodel_state_paint( imagemodel->state ) )
-		imagemodel_set_state( imagemodel, IMAGEMODEL_PEN, widget );
-}
-
-static void
-imageview_zoom_in_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
-{
-	Imageview *iv = IMAGEVIEW( callback_data );
 	Imagemodel *imagemodel = iv->imagemodel;
 	Conversion *conv = imagemodel->conv;
 
@@ -442,10 +393,8 @@ imageview_zoom_in_ifcb( gpointer callback_data, guint callback_action,
 }
 
 static void
-imageview_zoom_out_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_zoom_out_action_cb( GtkAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
 	Imagemodel *imagemodel = iv->imagemodel;
 	Conversion *conv = imagemodel->conv;
 
@@ -453,186 +402,283 @@ imageview_zoom_out_ifcb( gpointer callback_data, guint callback_action,
 }
 
 static void
-imageview_zoom_100_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_zoom_100_action_cb( GtkAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
-
 	if( iv->ip )
 		imagepresent_zoom_to( iv->ip, 1 );
 }
 
 static void
-imageview_zoom_fit_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_zoom_fit_action_cb( GtkAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
-
 	imagepresent_zoom_to( iv->ip, 0 );
 }
 
 static void
-imageview_show_rulers_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_show_status_action_cb( GtkToggleAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
-	Imagemodel *imagemodel = iv->imagemodel;
-	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM( widget );
-
-	imagemodel_set_rulers( imagemodel, item->active );
+	imagemodel_set_status( iv->imagemodel, 
+		gtk_toggle_action_get_active( action ) );
 }
 
 static void
-imageview_show_status_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_show_convert_action_cb( GtkToggleAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
-	Imagemodel *imagemodel = iv->imagemodel;
-	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM( widget );
-
-	imagemodel_set_status( imagemodel, item->active );
+	imagemodel_set_convert( iv->imagemodel, 
+		gtk_toggle_action_get_active( action ) );
 }
 
 static void
-imageview_show_paintbox_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_show_paintbox_action_cb( GtkToggleAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
-	Imagemodel *imagemodel = iv->imagemodel;
-	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM( widget );
-
-	imagemodel_set_paintbox( imagemodel, item->active );
+	imagemodel_set_paintbox( iv->imagemodel, 
+		gtk_toggle_action_get_active( action ) );
 }
 
 static void
-imageview_show_convert_ifcb( gpointer callback_data, guint callback_action,
-        GtkWidget *widget )
+imageview_show_rulers_action_cb( GtkToggleAction *action, Imageview *iv )
 {
-	Imageview *iv = IMAGEVIEW( callback_data );
-	Imagemodel *imagemodel = iv->imagemodel;
-	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM( widget );
-
-	imagemodel_set_convert( imagemodel, item->active );
+	imagemodel_set_rulers( iv->imagemodel, 
+		gtk_toggle_action_get_active( action ) );
 }
 
-/* Menu bar.
+static void
+imageview_mode_action_cb( GtkRadioAction *action, GtkRadioAction *current,
+	Imageview *iv )
+{
+	ImagemodelState state = (ImagemodelState) 
+		gtk_radio_action_get_current_value( action );
+
+	imagemodel_set_state( iv->imagemodel, state, GTK_WIDGET( iv ) );
+}
+
+static void
+imageview_mag_action_cb( GtkRadioAction *action, GtkRadioAction *current, 
+	Imageview *iv )
+{
+	if( iv->ip ) 
+		imagepresent_zoom_to( iv->ip, 
+			gtk_radio_action_get_current_value( action ) );
+}
+
+/* Our actions.
  */
-static GtkItemFactoryEntry imageview_menu_items[] = {
-	{ N_( "/_File" ),            			NULL,         
-		NULL,			0, "<Branch>" },
-	{ N_( "/File/_New" ), 			NULL, 
-		NULL, 			0, "<Branch>" },
-	{ N_( "/File/New/_Mark" ), 			NULL,
-		imageview_new_arrow_ifcb,	IREGION_MARK, 
-			"<StockItem>", GTK_STOCK_NEW },
-	{ N_( "/File/New/_HGuide" ), 			NULL,
-		imageview_new_arrow_ifcb,	IREGION_HGUIDE, 
-			"<StockItem>", GTK_STOCK_NEW },
-	{ N_( "/File/New/_VGuide" ), 			NULL,
-		imageview_new_arrow_ifcb,	IREGION_VGUIDE, 
-			"<StockItem>", GTK_STOCK_NEW },
-	{ N_( "/File/New/_Arrow" ), 			NULL,
-		imageview_new_region_ifcb,	IREGION_ARROW, 
-			"<StockItem>", GTK_STOCK_NEW },
-	{ N_( "/File/New/_Region" ), 			NULL,
-		imageview_new_region_ifcb,	IREGION_REGION, 
-			"<StockItem>", GTK_STOCK_NEW },
-	{ N_( "/File/sep3" ),        			NULL,         
-		NULL,       		0, "<Separator>" },
-	{ N_( "/File/_Replace Image ..." ),		"<control>R", 
-		imageview_replace_ifcb, 0 },
-	{ N_( "/File/_Save Image As ..." ),     	"<control>S", 
-		imageview_save_ifcb,    0, 
-			"<StockItem>", GTK_STOCK_SAVE_AS },
-	{ N_( "/File/sep1" ),        			NULL,         
-		NULL,       		0, "<Separator>" },
-	{ N_( "/File/Re_calculate Image" ),		"<control>C", 
-		imageview_recalc_ifcb,  0 },
-	{ N_( "/File/sep2" ),        			NULL,         
-		NULL,       		0, "<Separator>" },
-	{ N_( "/File/_Close" ),   			"<control>W", 
-		imageview_quit_ifcb,    0, 
-			"<StockItem>", GTK_STOCK_CLOSE },
+static GtkActionEntry imageview_actions[] = {
+	/* Menu items.
+	 */
+	{ "FileMenu", NULL, "_File" },
+	{ "FileNewMenu", NULL, "_New" },
+	{ "ViewMenu", NULL, "_View" },
+	{ "ViewToolbarMenu", NULL, "_Toolbar" },
+	{ "ViewModeMenu", NULL, "M_ode" },
+	{ "ViewZoomMenu", NULL, "_Zoom" },
+	{ "HelpMenu", NULL, "_Help" },
 
-	{ N_( "/_View" ),       			NULL,		
-		NULL,			0, "<Branch>" },
+	/* Actions.
+	 */
+	{ "NewMark", 
+		GTK_STOCK_NEW, N_( "New _Mark" ), NULL, 
+		N_( "Create a new mark" ), 
+		G_CALLBACK( imageview_new_arrow2_action_cb ) },
 
-	{ N_( "/View/_Toolbar" ), 			NULL,
-		NULL,			0, "<Branch>" },
-	{ N_( "/View/Toolbar/_Status" ), 		NULL, 
-		imageview_show_status_ifcb,    	0, "<CheckItem>" },
-	{ N_( "/View/Toolbar/_Display control" ), 	NULL, 
-		imageview_show_convert_ifcb,   	0, "<CheckItem>" },
-	{ N_( "/View/Toolbar/_Paint" ), 		NULL, 
-		imageview_show_paintbox_ifcb,  	0, "<CheckItem>" },
-	{ N_( "/View/Toolbar/_Rulers" ), 		NULL, 
-		imageview_show_rulers_ifcb,    	0, "<CheckItem>" },
+	{ "NewHGuide", 
+		GTK_STOCK_NEW, N_( "New _Horizontal Guide" ), NULL, 
+		N_( "Create a new horizontal guide" ), 
+		G_CALLBACK( imageview_new_arrow2_action_cb ) },
 
-	{ N_( "/View/M_ode" ),				NULL, 
-		NULL,       		0, "<Branch>" },
-	{ N_( "/View/Mode/_Select" ),			NULL, 
-		imageview_mode_ifcb, 
-			IMAGEMODEL_SELECT, "<RadioItem>" },
-	{ N_( "/View/Mode/_Pan" ),			NULL, 
-		imageview_mode_ifcb, 
-			IMAGEMODEL_PAN, "/View/Mode/Select" },
-	{ N_( "/View/Mode/Zoom _In" ),			NULL, 
-		imageview_mode_ifcb, 
-			IMAGEMODEL_MAGIN, "/View/Mode/Select" },
-	{ N_( "/View/Mode/Zoom _Out" ),			NULL, 
-		imageview_mode_ifcb, 
-			IMAGEMODEL_MAGOUT, "/View/Mode/Select" },
-	{ N_( "/View/Mode/P_aint" ),			NULL, 
-		imageview_mode_paint_ifcb, 
-			0, "/View/Mode/Select" },
+	{ "NewVGuide", 
+		GTK_STOCK_NEW, N_( "New _Vertical Guide" ), NULL, 
+		N_( "Create a new vertical guide" ), 
+		G_CALLBACK( imageview_new_arrow2_action_cb ) },
 
-	{ N_( "/View/Image _Header" ), 	    		NULL, 
-		imageview_header_ifcb,  0 },
+	{ "NewArrow", 
+		GTK_STOCK_NEW, N_( "New _Arrow" ), NULL, 
+		N_( "Create a new arrow" ), 
+		G_CALLBACK( imageview_new_arrow4_action_cb ) },
 
-	{ N_( "/View/sep1" ),        			NULL,         
-		NULL,       		0, "<Separator>" },
+	{ "NewRegion", 
+		GTK_STOCK_NEW, N_( "New _Region" ), NULL, 
+		N_( "Create a new region" ), 
+		G_CALLBACK( imageview_new_arrow4_action_cb ) },
 
-	{ N_( "/View/Zoom _In" ),			"<control>plus", 
-		imageview_zoom_in_ifcb,	0, 
-			"<StockItem>", GTK_STOCK_ZOOM_IN },
-	{ N_( "/View/Zoom _Out" ),			"<control>minus", 
-		imageview_zoom_out_ifcb,0, 
-			"<StockItem>", GTK_STOCK_ZOOM_OUT },
-	{ N_( "/View/Zoom _100%" ),			"<control>equal", 
-		imageview_zoom_100_ifcb,0, 
-			"<StockItem>", GTK_STOCK_ZOOM_100 },
-	{ N_( "/View/Zoom to _Fit" ),			NULL, 
-		imageview_zoom_fit_ifcb,0,
-			"<StockItem>", GTK_STOCK_ZOOM_FIT },
-	{ N_( "/View/_Zoom" ),				NULL, 
-		NULL,       		0, "<Branch>" },
-	{ N_( "/View/Zoom/6%" ),			NULL, 
-		imageview_mag_ifcb,	-16, "<RadioItem>" },
-	{ N_( "/View/Zoom/12%" ),			NULL, 
-		imageview_mag_ifcb, 	-8, "/View/Zoom/6%" },
-	{ N_( "/View/Zoom/25%" ),			NULL, 
-		imageview_mag_ifcb,  	-4, "/View/Zoom/6%" },
-	{ N_( "/View/Zoom/50%" ),			NULL, 
-		imageview_mag_ifcb,  	-2, "/View/Zoom/6%" },
-	{ N_( "/View/Zoom/100%" ),			NULL, 
-		imageview_mag_ifcb,  	1, "/View/Zoom/6%" },
-	{ N_( "/View/Zoom/200%" ),			NULL, 
-		imageview_mag_ifcb,  	2, "/View/Zoom/6%" },
-	{ N_( "/View/Zoom/400%" ),			NULL, 
-		imageview_mag_ifcb,  	4, "/View/Zoom/6%" },
-	{ N_( "/View/Zoom/800%" ),			NULL, 
-		imageview_mag_ifcb,  	8, "/View/Zoom/6%" },
-	{ N_( "/View/Zoom/1600%" ),			NULL, 
-		imageview_mag_ifcb,  	16, "/View/Zoom/6%" },
+	{ "Replace", 
+		NULL, N_( "Replace Image" ), NULL, 
+		N_( "Replace image from file" ), 
+		G_CALLBACK( imageview_replace_action_cb ) },
 
-	{ N_( "/_Help" ),            			NULL,         
-		NULL,			0, "<LastBranch>" },
-	{ N_( "/Help/_Image View" ),  			NULL,         
-		box_help_ifcb,		GPOINTER_TO_UINT( "sec:view" ) },
-	{ N_( "/Help/_Paint Bar" ),  			NULL,         
-		box_help_ifcb,		GPOINTER_TO_UINT( "sec:paintbox" ) }
+	{ "SaveAs", 
+		GTK_STOCK_SAVE_AS, N_( "Save Image As" ), NULL,
+		N_( "Save image to file" ), 
+		G_CALLBACK( imageview_save_action_cb ) },
+
+	{ "Recalculate", 
+		NULL, N_( "Recalculate" ), "<control>C",
+		N_( "Recalculate image" ), 
+		G_CALLBACK( imageview_recalc_action_cb ) },
+
+	{ "Close", 
+		GTK_STOCK_CLOSE, N_( "_Close" ), NULL,
+		N_( "Close" ), 
+		G_CALLBACK( iwindow_kill_action_cb ) },
+
+	{ "Header", 
+		NULL, N_( "Image _Header" ), NULL,
+		N_( "View image header" ), 
+		G_CALLBACK( imageview_header_action_cb ) },
+
+	{ "ZoomIn",
+		GTK_STOCK_ZOOM_IN, N_( "Zoom _In" ), "<control>plus",
+		N_( "Zoom in on mouse cursor" ),
+		G_CALLBACK( imageview_zoom_in_action_cb ) },
+
+	{ "ZoomOut",
+		GTK_STOCK_ZOOM_OUT, N_( "Zoom _Out" ), "<control>minus",
+		N_( "Zoom out" ),
+		G_CALLBACK( imageview_zoom_out_action_cb ) },
+
+	{ "Zoom100",
+		GTK_STOCK_ZOOM_100, N_( "Zoom _100%" ), "<control>equal",
+		N_( "Zoom to 100%" ),
+		G_CALLBACK( imageview_zoom_100_action_cb ) },
+
+	{ "ZoomFit",
+		GTK_STOCK_ZOOM_FIT, N_( "Zoom to _Fit" ), NULL,
+		N_( "Zoom to fit image to window" ),
+		G_CALLBACK( imageview_zoom_fit_action_cb ) },
+
+	{ "Guide", 
+		GTK_STOCK_HELP, N_( "_Contents" ), "F1",
+		N_( "Open the users guide" ), 
+		G_CALLBACK( mainw_guide_action_cb ) },
+
+	{ "About", 
+		NULL, N_( "_About" ), NULL,
+		N_( "About this program" ), 
+		G_CALLBACK( mainw_about_action_cb ) }
 };
 
+static GtkToggleActionEntry imageview_toggle_actions[] = {
+	{ "Status",
+		NULL, N_( "_Status" ), NULL,
+		N_( "Show status bar" ),
+		G_CALLBACK( imageview_show_status_action_cb ), TRUE },
+
+	{ "Control",
+		NULL, N_( "_Display control" ), NULL,
+		N_( "Show display control bar" ),
+		G_CALLBACK( imageview_show_convert_action_cb ), TRUE },
+
+	{ "Paint",
+		NULL, N_( "_Paint" ), NULL,
+		N_( "Show paint bar" ),
+		G_CALLBACK( imageview_show_paintbox_action_cb ), FALSE },
+
+	{ "Rulers",
+		NULL, N_( "_Rulers" ), NULL,
+		N_( "Show rulers" ),
+		G_CALLBACK( imageview_show_rulers_action_cb ), FALSE }
+};
+
+static GtkRadioActionEntry imageview_mode_radio_actions[] = {
+	{ "SelectMode",
+		NULL, N_( "_Select" ), NULL,
+		N_( "Select and modify selections" ),
+		IMAGEMODEL_SELECT },
+
+	{ "PanMode",
+		NULL, N_( "_Pan" ), NULL,
+		N_( "Pan image" ),
+		IMAGEMODEL_PAN },
+
+	{ "ZoomInMode",
+		NULL, N_( "Zoom _In" ), NULL,
+		N_( "Zoom in on mouse cursor" ),
+		IMAGEMODEL_MAGIN },
+
+	{ "ZoomOutMode",
+		NULL, N_( "Zoom _Out" ), NULL,
+		N_( "Zoom out" ),
+		IMAGEMODEL_MAGOUT }
+};
+
+static GtkRadioActionEntry imageview_zoom_radio_actions[] = {
+	{ "Zoom6Mode",
+		NULL, N_( "6%" ), NULL, N_( "Zoom to 6%" ), -16 },
+	{ "Zoom12Mode",
+		NULL, N_( "12%" ), NULL, N_( "Zoom to 12%" ), -8 },
+	{ "Zoom25Mode",
+		NULL, N_( "25%" ), NULL, N_( "Zoom to 25%" ), -4 },
+	{ "Zoom50Mode",
+		NULL, N_( "50%" ), NULL, N_( "Zoom to 50%" ), -2 },
+	{ "Zoom100Mode",
+		NULL, N_( "100%" ), NULL, N_( "Zoom to 100%" ), 1 },
+	{ "Zoom200Mode",
+		NULL, N_( "200%" ), NULL, N_( "Zoom to 200%" ), 2 },
+	{ "Zoom400Mode",
+		NULL, N_( "400%" ), NULL, N_( "Zoom to 400%" ), 4 },
+	{ "Zoom800Mode",
+		NULL, N_( "800%" ), NULL, N_( "Zoom to 800%" ), 8 },
+	{ "Zoom1600Mode",
+		NULL, N_( "1600%" ), NULL, N_( "Zoom to 1600%" ), 16 }
+};
+
+static const char *imageview_menubar_ui_description =
+"<ui>"
+"  <menubar name='ImageviewMenubar'>"
+"    <menu action='FileMenu'>"
+"      <menu action='FileNewMenu'>"
+"        <menuitem action='NewMark'/>"
+"        <menuitem action='NewHGuide'/>"
+"        <menuitem action='NewVGuide'/>"
+"        <menuitem action='NewArrow'/>"
+"        <menuitem action='NewRegion'/>"
+"      </menu>"
+"      <separator/>"
+"      <menuitem action='Replace'/>"
+"      <menuitem action='SaveAs'/>"
+"      <separator/>"
+"      <menuitem action='Recalculate'/>"
+"      <separator/>"
+"      <menuitem action='Close'/>"
+"    </menu>"
+"    <menu action='ViewMenu'>"
+"      <menu action='ViewToolbarMenu'>"
+"        <menuitem action='Status'/>"
+"        <menuitem action='Control'/>"
+"        <menuitem action='Paint'/>"
+"        <menuitem action='Rulers'/>"
+"      </menu>"
+"      <menu action='ViewModeMenu'>"
+"        <menuitem action='SelectMode'/>"
+"        <menuitem action='PanMode'/>"
+"        <menuitem action='ZoomInMode'/>"
+"        <menuitem action='ZoomOutMode'/>"
+"      </menu>"
+"      <menuitem action='Header'/>"
+"      <separator/>"
+"      <menuitem action='ZoomIn'/>"
+"      <menuitem action='ZoomOut'/>"
+"      <menuitem action='Zoom100'/>"
+"      <menuitem action='ZoomFit'/>"
+"      <menu action='ViewZoomMenu'>"
+"        <menuitem action='Zoom6Mode'/>"
+"        <menuitem action='Zoom12Mode'/>"
+"        <menuitem action='Zoom25Mode'/>"
+"        <menuitem action='Zoom50Mode'/>"
+"        <menuitem action='Zoom100Mode'/>"
+"        <menuitem action='Zoom200Mode'/>"
+"        <menuitem action='Zoom400Mode'/>"
+"        <menuitem action='Zoom800Mode'/>"
+"        <menuitem action='Zoom1600Mode'/>"
+"      </menu>"
+"    </menu>"
+"    <menu action='HelpMenu'>"
+"      <menuitem action='Guide'/>"
+"      <menuitem action='About'/>"
+"    </menu>"
+"  </menubar>"
+"</ui>";
+
 /* Spot mouse motion events, to update status bar.
  */
 static gint
@@ -671,7 +717,8 @@ imageview_filedrop( Imageview *iv, const char *file )
 static void
 imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 {
-	iWindow *iwnd = IWINDOW( iv );
+	GtkAccelGroup *accel_group;
+	GError *error;
 	GtkWidget *mbar;
 	GtkWidget *frame;
 	GList *focus_chain;
@@ -690,21 +737,50 @@ imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 		G_OBJECT( iv->imagemodel ), "destroy", 
 		G_CALLBACK( imageview_imagemodel_destroy_cb ), iv );
 
-	/* Make menu bar
-	 */
-	iv->ifac = gtk_item_factory_new( GTK_TYPE_MENU_BAR, 
-		"<imageview>", iwnd->accel_group );
-	g_object_ref( iv->ifac );
-	gtk_object_sink( GTK_OBJECT( iv->ifac ) );
-#ifdef ENABLE_NLS
-	gtk_item_factory_set_translate_func( iv->ifac,
-		(GtkTranslateFunc) gettext, NULL, NULL );
-#endif /* ENABLE_NLS */
-	gtk_item_factory_create_items( iv->ifac, 
-		IM_NUMBER( imageview_menu_items ), imageview_menu_items, iv );
-	mbar = gtk_item_factory_get_widget( iv->ifac, "<imageview>" );
+        /* Make main menu bar
+         */
+	iv->action_group = gtk_action_group_new( "ImageviewActions" );
+	gtk_action_group_set_translation_domain( iv->action_group, 
+		GETTEXT_PACKAGE );
+	gtk_action_group_add_actions( iv->action_group, 
+		imageview_actions, G_N_ELEMENTS( imageview_actions ), 
+		GTK_WINDOW( iv ) );
+	gtk_action_group_add_toggle_actions( iv->action_group, 
+		imageview_toggle_actions, 
+			G_N_ELEMENTS( imageview_toggle_actions ), 
+		GTK_WINDOW( iv ) );
+	gtk_action_group_add_radio_actions( iv->action_group,
+		imageview_mode_radio_actions, 
+			G_N_ELEMENTS( imageview_mode_radio_actions ), 
+		WORKSPACE_MODE_REGULAR,
+		G_CALLBACK( imageview_mode_action_cb ),
+		GTK_WINDOW( iv ) );
+	gtk_action_group_add_radio_actions( iv->action_group,
+		imageview_zoom_radio_actions, 
+			G_N_ELEMENTS( imageview_zoom_radio_actions ), 
+		1,
+		G_CALLBACK( imageview_mag_action_cb ),
+		GTK_WINDOW( iv ) );
+
+	iv->ui_manager = gtk_ui_manager_new();
+	gtk_ui_manager_insert_action_group( iv->ui_manager, 
+		iv->action_group, 0 );
+
+	accel_group = gtk_ui_manager_get_accel_group( iv->ui_manager );
+	gtk_window_add_accel_group( GTK_WINDOW( iv ), accel_group );
+
+	error = NULL;
+	if( !gtk_ui_manager_add_ui_from_string( iv->ui_manager,
+			imageview_menubar_ui_description, -1, &error ) ) {
+		g_message( "building menus failed: %s", error->message );
+		g_error_free( error );
+		exit( EXIT_FAILURE );
+	}
+
+	mbar = gtk_ui_manager_get_widget( iv->ui_manager, 
+		"/ImageviewMenubar" );
 	gtk_box_pack_start( GTK_BOX( vbox ), mbar, FALSE, FALSE, 0 );
-	gtk_widget_show( mbar );
+        gtk_widget_show( mbar );
 
 	/* Status bar.
 	 */
@@ -761,7 +837,10 @@ imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 			window_x, window_y );
 
 		gtk_window_set_default_size( GTK_WINDOW( iv ),
-			iimage->window_width, iimage->window_height );
+			IM_MIN( iimage->window_width, 	
+				gdk_screen_get_width( screen ) ),
+			IM_MIN( iimage->window_height,
+				gdk_screen_get_height( screen ) ) );
 
 		iv->imagemodel->show_status = iimage->show_status;
 		iv->imagemodel->show_paintbox = iimage->show_paintbox;
@@ -786,7 +865,7 @@ imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 		w = IM_MIN( IMAGE_WINDOW_WIDTH, 
 			iv->imagemodel->conv->image.width + 16 );
 		h = IM_MIN( IMAGE_WINDOW_HEIGHT, 
-			iv->imagemodel->conv->image.height + 43 );
+			iv->imagemodel->conv->image.height + 41 );
 		gtk_window_set_default_size( GTK_WINDOW( iv ), w, h );
 		conversion_set_mag( iv->imagemodel->conv, 1 );
 	}
@@ -861,20 +940,12 @@ imageview_popdown( iWindow *iwnd, void *client,
 	/* Signal changed on iimage if we save the convert settings. This will
 	 * make the thumbnail update.
 	 */
-	if( imagemodel->show_convert ) {
-		if( !iimage->show_convert ||
-			iimage->scale != imagemodel->scale ||
-			iimage->offset != imagemodel->offset ) {
-			iimage->show_convert = imagemodel->show_convert;
-			iimage->scale = imagemodel->scale;
-			iimage->offset = imagemodel->offset;
-			iobject_changed( IOBJECT( iimage ) );
-		}
-	}
-	else if( iimage->show_convert ) {
-		iimage->show_convert = FALSE;
-		iimage->scale = 1.0;
-		iimage->offset = 0.0;
+	if( iimage->show_convert != imagemodel->show_convert ||
+		iimage->scale != imagemodel->scale ||
+		iimage->offset != imagemodel->offset ) {
+		iimage->show_convert = imagemodel->show_convert;
+		iimage->scale = imagemodel->scale;
+		iimage->offset = imagemodel->offset;
 		iobject_changed( IOBJECT( iimage ) );
 	}
 
