@@ -27,13 +27,6 @@
 
  */
 
-/* Turn on gtk's old broken widgets. We need them for program.c
-
-	FIXME ... yuk, redo program.c without GtkText and GtkCTree
-
- */
-#define GTK_ENABLE_BROKEN 
-
 /* Turn off VIPS's old and broken defines, we don't need them.
  */
 #define IM_NO_VIPS7_COMPAT
@@ -154,6 +147,7 @@ extern int statfs();
 #define STACK_SIZE (1000)	/* Depth of eval stack */
 #define LEN_LABEL (512)		/* Label on windows */
 #define MAX_SYSTEM (50)		/* Max number of args we allow */
+#define MAX_BANDS (64)		/* Max number of bands in image */
 #define MAX_CSTACK (10)		/* Max number of cursors we stack */
 #define MAX_STRSIZE (32768)	/* Size of text for user defs */
 #define MAX_TRACE (1024)	/* Biggest thing we print in trace */
@@ -164,7 +158,6 @@ extern int statfs();
 #define VIPS_DOCPATH "$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR \
 	"doc" IM_DIR_SEP_STR "vips" IM_DIR_SEP_STR "html"
 #define IP_NAME PACKAGE "-" VERSION
-#define IP_SAVEDIR "$HOME" IM_DIR_SEP_STR "." IP_NAME
 #define NAMESPACE VIPS_HOMEPAGE "/" "nip" 
 				/* XML namespace ... note, not nip2! */
 #define MAX_LINELENGTH (120)	/* Max chars we display of value */
@@ -412,10 +405,6 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 	(watch_bool_get( main_watchgroup, "DISPLAY_STATUS", FALSE ))
 #define DISPLAY_CONVERT \
 	(watch_bool_get( main_watchgroup, "DISPLAY_CONVERSION", FALSE ))
-#define CONVERSION_SCALE \
-	(watch_double_get( main_watchgroup, "DISPLAY_CONVERSION_SCALE", 1.0 ))
-#define CONVERSION_OFFSET \
-	(watch_double_get( main_watchgroup, "DISPLAY_CONVERSION_OFFSET", 1.0 ))
 
 /* Update children during paint.
  */
@@ -475,13 +464,6 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #define VIPS_CPUS \
 	(watch_int_get( main_watchgroup, "VIPS_CPUS", 1 ))
 
-/* Startup window size.
- */
-#define MAINW_WINDOW_WIDTH \
-	(watch_int_get( main_watchgroup, "MAINW_WINDOW_WIDTH", 640 ))
-#define MAINW_WINDOW_HEIGHT \
-	(watch_int_get( main_watchgroup, "MAINW_WINDOW_HEIGHT", 480 ))
-
 /* Bar prefs.
  */
 #define MAINW_TOOLBAR \
@@ -492,6 +474,8 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 	(watch_bool_get( main_watchgroup, "MAINW_STATUSBAR", TRUE ))
 #define MAINW_TOOLKITBROWSER \
 	(watch_bool_get( main_watchgroup, "MAINW_TOOLKITBROWSER", FALSE ))
+#define MAINW_PANE_POSITION \
+	(watch_int_get( main_watchgroup, "MAINW_PANE_POSITION", 400 ))
 
 /* Heap size.
  */
@@ -565,13 +549,18 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #define PATH_START (watch_path_get( main_watchgroup, "CALC_PATH_START", \
 	path_start_default ))
 #define PATH_TMP (watch_string_get( main_watchgroup, "CALC_PATH_TMP", \
-	"$HOME/.$PACKAGE-$VERSION/tmp" ))
+	path_tmp_default ))
 
 /* Disassemble functions.
  */
 #define TRACE_FUNCTIONS \
 	(watch_bool_get( main_watchgroup, "CALC_TRACE_FUNCTIONS", FALSE ))
 
+/* Program window.
+ */
+#define PROGRAM_PANE_POSITION \
+	(watch_double_get( main_watchgroup, "PROGRAM_PANE_POSITION", 200 ))
+
 /* Global variables from parse.y.
 
 	FIXME move to its own header ... parser.h?
