@@ -51,8 +51,8 @@ itext_finalize( GObject *gobject )
 
 	/* My instance destroy stuff.
 	 */
-	FREE( itext->formula );
-	FREE( itext->formula_default );
+	IM_FREE( itext->formula );
+	IM_FREE( itext->formula_default );
 	buf_destroy( &itext->value );
 	buf_destroy( &itext->decompile );
 
@@ -799,14 +799,18 @@ itext_set_edited( iText *itext, gboolean edited )
 		heapmodel_set_modified( heapmodel, TRUE );
 }
 
-void
+gboolean
 itext_set_formula( iText *itext, const char *formula )
 {
 	if( !itext->formula || strcmp( itext->formula, formula ) != 0 ) {
-		SETSTR( itext->formula, formula )
+		SETSTR( itext->formula, formula );
 
 		heapmodel_set_modified( HEAPMODEL( itext ), TRUE );
 
 		iobject_changed( IOBJECT( itext ) );
+
+		return( TRUE );
 	}
+
+	return( FALSE );
 }
