@@ -84,6 +84,41 @@ static GSList *iwindow_all = NULL;
  */
 static GdkCursor *iwindow_cursor[IWINDOW_SHAPE_LAST] = { NULL };
 
+#ifdef DEBUG
+/* Human-readable names for cursor shapes.
+ */
+static const char *iwindow_cursor_name[] = {
+	"IWINDOW_SHAPE_DROPPER",
+	"IWINDOW_SHAPE_PEN",
+	"IWINDOW_SHAPE_SMUDGE",
+	"IWINDOW_SHAPE_SMEAR",
+	"IWINDOW_SHAPE_TEXT",
+	"IWINDOW_SHAPE_RECT",
+	"IWINDOW_SHAPE_FLOOD",
+	"IWINDOW_SHAPE_MOVE",
+	"IWINDOW_SHAPE_EDIT",
+	"IWINDOW_SHAPE_MAGIN",	
+	"IWINDOW_SHAPE_MAGOUT",	
+	"IWINDOW_SHAPE_TOP",
+	"IWINDOW_SHAPE_BOTTOM",
+	"IWINDOW_SHAPE_LEFT",
+	"IWINDOW_SHAPE_RIGHT",
+	"IWINDOW_SHAPE_TOPRIGHT",
+	"IWINDOW_SHAPE_TOPLEFT",
+	"IWINDOW_SHAPE_BOTTOMRIGHT",
+	"IWINDOW_SHAPE_BOTTOMLEFT",
+	"IWINDOW_SHAPE_HGLASS1",
+	"IWINDOW_SHAPE_HGLASS2",
+	"IWINDOW_SHAPE_HGLASS3",
+	"IWINDOW_SHAPE_HGLASS4",
+	"IWINDOW_SHAPE_HGLASS5",
+	"IWINDOW_SHAPE_HGLASS6",
+	"IWINDOW_SHAPE_HGLASS7",
+	"IWINDOW_SHAPE_HGLASS8",
+	"IWINDOW_SHAPE_NONE"
+};
+#endif /*DEBUG*/
+
 int
 iwindow_number( void )
 {
@@ -213,6 +248,8 @@ iwindow_cursor_update( iWindow *iwnd )
 {
 	if( GTK_WIDGET_REALIZED( GTK_WIDGET( iwnd ) ) ) {
 		GSList *p;
+		iWindowShape best_shape;
+		int best_priority;
 
 		/* Global shape set? Use that for the whole window.
 		 */
@@ -232,21 +269,30 @@ iwindow_cursor_update( iWindow *iwnd )
 		gdk_window_set_cursor( GTK_WIDGET( iwnd )->window, NULL );
 		gdk_window_set_cursor( iwindow_get_work_window( iwnd ), NULL );
 
-		/* And set the work area to the highest non-NONE shape on 
-		 * the stack.
+		/* And set the work area to the highest priority non-NONE 
+		 * shape we can find .
+
+			FIXME ... could avoid the search if we sorted the
+			context list by priority on each context_new(),
+			but not very important.
+
 		 */
+		best_shape = IWINDOW_SHAPE_NONE;
+		best_priority = -1;
 		for( p = iwnd->contexts; p; p = p->next ) {
 			iWindowCursorContext *cntxt = 
 				(iWindowCursorContext *) p->data;
 
-			if( cntxt->shape != IWINDOW_SHAPE_NONE ) {
-				gdk_window_set_cursor( 
-					iwindow_get_work_window( iwnd ),
-					iwindow_cursor[cntxt->shape] );
-				return( NULL );
+			if( cntxt->shape != IWINDOW_SHAPE_NONE &&
+				cntxt->priority > best_priority ) {
+				best_shape = cntxt->shape;
+				best_priority = cntxt->priority;
 			}
 		}
 
+		gdk_window_set_cursor( 
+			iwindow_get_work_window( iwnd ),
+			iwindow_cursor[best_shape] );
 		gdk_flush();
 	}
 
@@ -305,13 +351,19 @@ set_pointer( void )
 }
 
 iWindowCursorContext *
-iwindow_cursor_context_new( iWindow *iwnd )
+iwindow_cursor_context_new( iWindow *iwnd, int priority, const char *name )
 {
 	iWindowCursorContext *cntxt = INEW( NULL, iWindowCursorContext );
 
+#ifdef DEBUG
+	printf( "iwindow_cursor_context_new: %s\n", cntxt->name );
+#endif /*DEBUG*/
+
 	cntxt->iwnd = iwnd;
+	cntxt->priority = priority;
+	cntxt->name = name;
 	cntxt->shape = IWINDOW_SHAPE_NONE;
-	iwnd->contexts = g_slist_append( iwnd->contexts, cntxt );
+	iwnd->contexts = g_slist_prepend( iwnd->contexts, cntxt );
 
 	return( cntxt );
 }
@@ -322,7 +374,7 @@ iwindow_cursor_context_destroy( iWindowCursorContext *cntxt )
 	iWindow *iwnd = cntxt->iwnd;
 
 	iwnd->contexts = g_slist_remove( iwnd->contexts, cntxt );
-	FREE( cntxt );
+	IM_FREE( cntxt );
 	iwindow_cursor_update( iwnd );
 }
 
@@ -331,6 +383,11 @@ iwindow_cursor_context_set_cursor( iWindowCursorContext *cntxt,
 	iWindowShape shape )
 {
 	if( cntxt->shape != shape ) {
+#ifdef DEBUG
+		printf( "iwindow_cursor_context_set_cursor: %s = %s\n",
+			cntxt->name, iwindow_cursor_name[shape] );
+#endif /*DEBUG*/
+
 		cntxt->shape = shape;
 		iwindow_cursor_update( cntxt->iwnd );
 	}
@@ -469,7 +526,7 @@ iwindow_finalize( GObject *gobject )
 	/* My instance destroy stuff.
 	 */
 	iwindow_all = g_slist_remove( iwindow_all, iwnd );
-	FREE( iwnd->title );
+	IM_FREE( iwnd->title );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 
@@ -507,8 +564,8 @@ iwindow_popdown_notify( iWindow *iwnd, iWindowResult result )
 		 */
 		gtk_widget_unmap( GTK_WIDGET( iwnd ) );
 
-	call_string_filename( (call_string_fn) gtk_accel_map_save,
-		IP_SAVEDIR IM_DIR_SEP_STR "accel_map", NULL, NULL, NULL );
+	call_string_filenamef( (call_string_fn) gtk_accel_map_save,
+		"%s" IM_DIR_SEP_STR "accel_map", get_savedir() );
 
 	iwindow_notify_return( iwnd );
 }
@@ -536,10 +593,27 @@ iwindow_delete_event( GtkWidget *widget, GdkEventAny *event )
 	return( TRUE );
 }
 
+static gboolean
+iwindow_configure_event( GtkWidget *widget, GdkEventConfigure *event )
+{
+	iWindow *iwnd = IWINDOW( widget );
+	
+	if( iwnd->width_pref ) {
+		/* Save window size in global prefs.
+		 */
+		prefs_set( iwnd->width_pref, "%d", event->width );
+		prefs_set( iwnd->height_pref, "%d", event->height );
+	}
+
+	return( GTK_WIDGET_CLASS( parent_class )->
+		configure_event( widget, event ) );
+}
+
 static void 
 iwindow_real_build( GtkWidget *widget )
 {
 	iWindow *iwnd = IWINDOW( widget );
+	GdkScreen *screen = gtk_widget_get_screen( GTK_WIDGET( iwnd ) );
 
 #ifdef DEBUG
 	printf( "iwindow_real_build: %s\n", iwnd->title );
@@ -559,6 +633,17 @@ iwindow_real_build( GtkWidget *widget )
 	if( iwnd->title )
 		gtk_window_set_title( GTK_WINDOW( iwnd ), iwnd->title );
 
+	if( iwnd->width_pref ) {
+		int width = watch_int_get( main_watchgroup, 
+			iwnd->width_pref, 640 );
+		int height = watch_int_get( main_watchgroup, 
+			iwnd->height_pref, 480 );
+
+		gtk_window_set_default_size( GTK_WINDOW( iwnd ), 
+			IM_MIN( width, gdk_screen_get_width( screen ) ),
+			IM_MIN( height, gdk_screen_get_height( screen ) ) );
+	}
+
         gtk_widget_show( iwnd->work );
 }
 
@@ -578,6 +663,7 @@ iwindow_class_init( iWindowClass *class )
 	gobject_class->finalize = iwindow_finalize;
 
 	widget_class->delete_event = iwindow_delete_event;
+	widget_class->configure_event = iwindow_configure_event;
 
 	class->build = iwindow_real_build;
 
@@ -618,6 +704,9 @@ iwindow_init( iWindow *iwnd )
 	iwnd->contexts = NULL;
 	iwnd->work_window = NULL;
 
+	iwnd->width_pref = NULL;
+	iwnd->height_pref = NULL;
+
 	iwindow_all = g_slist_prepend( iwindow_all, iwnd );
 }
 
@@ -692,6 +781,14 @@ iwindow_set_popdown( iWindow *iwnd, iWindowFn popdown, void *popdown_a )
 	iwnd->popdown_a = popdown_a;
 }
 
+void 
+iwindow_set_size_prefs( iWindow *iwnd, 
+	const char *width_pref, const char *height_pref )
+{
+	iwnd->width_pref = width_pref;
+	iwnd->height_pref = height_pref;
+}
+
 void 
 iwindow_set_work_window( iWindow *iwnd, GdkWindow *work_window )
 {
@@ -722,6 +819,14 @@ iwindow_kill( iWindow *iwnd )
 	return( NULL );
 }
 
+/* ... as an action.
+ */
+void
+iwindow_kill_action_cb( GtkAction *action, iWindow *iwnd )
+{
+	iwindow_kill( iwnd );
+}
+
 void 
 iwindow_build( iWindow *iwnd )
 {
