@@ -64,6 +64,8 @@ void *main_c_stack_base = NULL;			/* Base of C stack */
 
 gboolean main_starting = TRUE;			/* In startup */
 
+gboolean main_time_save = FALSE;		/* Time save image ops */
+
 /* Action on main_quit()
  */
 typedef enum {
@@ -74,9 +76,9 @@ typedef enum {
 static ExitAction action_on_exit = EXIT_ACTION_NONE;
 
 static const char *main_argv0 = NULL;		/* argv[0] */
-static const char *main_name = NULL;		/* Name invoked as */
 static iOpenFile *main_stdin = NULL;		/* stdin as an iOpenFile */
 static gboolean main_interactive = TRUE;	/* In GUI mode */
+static gboolean main_load_all = TRUE;		/* Load all .defs */
 static GtkIconFactory *main_icon_factory = NULL;/* Add stocks to this */
 static Splash *main_splash = NULL;
 
@@ -115,7 +117,7 @@ main_usage( void )
 "%s: usage:\n"
 "%s filename1 filename2 ...\n"
 "\tstart in GUI mode, loading the named files" ),
-		main_name, main_name );
+		g_get_prgname(), g_get_prgname() );
 	printf( "\n" );
 
 	printf( _(
@@ -124,27 +126,40 @@ main_usage( void )
 "\t[\"filename\", \"arg1\", \"arg2\", ...], set argc to length of list;\n"
 "\tprint the value of symbol \"main\" to stdout; exit; useful for\n"
 "\trunning %s as an interpreter on unix" ),
-		main_name, main_name );
+		g_get_prgname(), g_get_prgname() );
 	printf( "\n" );
 
 	printf( _(
 "%s -main arg1 arg2 ...\n"
 "\tas -script, but read the definitions from stdin rather than from a\n"
 "\tfile; useful for here-is shell scripts" ),
-		main_name );
+		g_get_prgname() );
 	printf( "\n" );
 
 	printf( _(
 "%s -workspace arg1 arg2 ...\n"
 "\tas -main, but read a workspace save file instead; run in GUI mode\n"
 "\tprint the value of the final row in the save file on exit" ),
-		main_name );
+		g_get_prgname() );
+	printf( "\n" );
+
+	printf( _(
+"%s -i18n\n"
+"\toutput all (_"") strings in all toolkits and quit" ),
+		g_get_prgname() );
 	printf( "\n" );
 
 	printf( _(
 "%s -benchmark\n"
 "\tload all start objects and quit; useful for bechmarking the compiler" ),
-		main_name );
+		g_get_prgname() );
+	printf( "\n" );
+
+	printf( _(
+"%s -time_save\n"
+"\ttime all save image operations, useful for benchmarking image\n" 
+"\tprocessing functions" ),
+		g_get_prgname() );
 	printf( "\n" );
 
 	printf( _( "%s version: %s" ), PACKAGE, VERSION );
@@ -225,8 +240,8 @@ main_quit( void )
 	/* Dump wisdom back again.
 	 */
 #if HAVE_FFTW || HAVE_FFTW3
-	if( (of = file_open_write( "%s", 
-		IP_SAVEDIR IM_DIR_SEP_STR "wisdom" )) ) {
+	if( (of = file_open_write( "%s" IM_DIR_SEP_STR "wisdom", 
+		get_savedir() )) ) {
 		fftw_export_wisdom_to_file( of->fp );
 		file_close( of );
 	}
@@ -239,9 +254,9 @@ main_quit( void )
 	/* Junk all symbols. This may remove a bunch of intermediate images
 	 * too.
 	 */
+	UNREF( main_watchgroup );
 	UNREF( main_symbol_root );
 	UNREF( main_toolkitgroup );
-	UNREF( main_watchgroup );
 	UNREF( main_workspacegroup );
 
 	/* Junk reduction machine ... this should remove all image temps.
@@ -295,6 +310,11 @@ main_quit_test_cb( void *sys, iWindowResult result )
 void
 main_quit_test( void )
 {
+	/* Flush any pending preference saves before we look for dirty
+	 * objects.
+	 */
+	watchgroup_flush( main_watchgroup );
+
 	/* Close registered models.
 	 */
 	filemodel_inter_close_registered_cb( IWINDOW( main_window_top ), NULL,
@@ -304,7 +324,8 @@ main_quit_test( void )
 gboolean
 main_splash_enabled( void )
 {
-	return( !existsf( "%s", NO_SPLASH ) );
+	return( !existsf( "%s" IM_DIR_SEP_STR "%s", 
+		get_savedir(), NO_SPLASH ) );
 }
 
 void
@@ -326,11 +347,12 @@ main_watchgroup_changed_cb( void )
 {
 	static int last_max_cpus = -1;
 
-	/* Try to avoid setting too often ... we can get leaks.
+	/* Try to avoid setting too often ... we can get leaks on some
+	 * platforms.
 	 */
 	if( last_max_cpus != VIPS_CPUS ) {
 		last_max_cpus = VIPS_CPUS;
-		putenvf( "IM_CONCURRENCY=%d", VIPS_CPUS );
+		setenvf( "IM_CONCURRENCY", "%d", VIPS_CPUS );
 	}
 }
 
@@ -340,16 +362,10 @@ main_watchgroup_changed_cb( void )
 gboolean
 main_load( Workspace *ws, const char *filename )
 {
-	BufInfo buf;
-	char txt[MAX_STRSIZE];
 	Workspace *new_ws;
 
 	main_splash_update( _( "Loading \"%s\"" ), im_skip_dir( filename ) );
 
-	if( toolkit_new_from_file( main_toolkitgroup, filename ) )
-		return( TRUE );
-	error_clear();
-
 	if( (new_ws = 
 		workspace_new_from_file( main_workspacegroup, filename )) ) {
 		Mainw *new_mainw;
@@ -364,24 +380,13 @@ main_load( Workspace *ws, const char *filename )
 
 	/* Try as matrix or image. Have to do these via definitions.
 	 */
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	buf_appends( &buf, "Matrix_file \"" );
-	buf_appendsc( &buf, TRUE, filename );
-	buf_appends( &buf, "\"" );
-	if( workspace_add_def( ws, buf_all( &buf ) ) ) {
-		mainw_recent_add( &mainw_recent_matrix, filename );
+	if( workspace_load_file( ws, filename ) )
 		return( TRUE );
-	}
-	error_clear();
 
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	buf_appends( &buf, "Image_file \"" );
-	buf_appendsc( &buf, TRUE, filename );
-	buf_appends( &buf, "\"" );
-	if( workspace_add_def( ws, buf_all( &buf ) ) ) {
-		mainw_recent_add( &mainw_recent_image, filename );
+	/* Toolkits are about the least fussy file format ... try this last.
+	 */
+	if( toolkit_new_from_file( main_toolkitgroup, filename ) )
 		return( TRUE );
-	}
 	error_clear();
 
 	error_top( _( "Unknown file type." ) );
@@ -414,7 +419,7 @@ main_load_def( const char *filename )
 	/* If we're in command-line mode, only load toolkits which 
 	 * start with '_'. For interactive mode, load everything.
 	 */
-	if( main_interactive || im_skip_dir( filename )[0] == '_' ) {
+	if( main_load_all || im_skip_dir( filename )[0] == '_' ) {
 		main_splash_update( _( "Loading toolkit \"%s\"" ), 
 			im_skip_dir( filename ) );
 
@@ -487,7 +492,7 @@ main_link_package( im_package *pack)
 /* Load all plugins and defs.
  */
 static void
-main_load_all( void )
+main_load_startup( void )
 {
 	const char *prefix;
 
@@ -516,7 +521,7 @@ main_load_all( void )
 
 	/* Load up any standard plugs.
 	 */
-	if( (prefix = im_guess_prefix( main_argv0, "VIPSHOME" )) &&
+	if( (prefix = get_vipshome( main_argv0 )) &&
 		im_load_plugins( "%s" IM_DIR_SEP_STR "lib", prefix ) ) {
 		error_top( _( "Unable to load plugins." ) );
 		error_sub( _( "Error loading plug-ins in "
@@ -570,7 +575,6 @@ void
 main_reload( void )
 {
 	set_hourglass();
-	program_freeze();
 
 	/* Remove.
 	 */
@@ -581,7 +585,7 @@ main_reload( void )
 
 	/* Reload.
 	 */
-	main_load_all();
+	main_load_startup();
 
 	/* We may have changed our prefs ... link the watches to the
 	 * new prefs workspace.
@@ -589,7 +593,6 @@ main_reload( void )
 	watch_relink_all();
 
 	set_pointer();
-	program_thaw();
 }
 
 /* Use a file to paint a named stock item.
@@ -630,7 +633,7 @@ main_register_icons( void )
 		{ STOCK_DROPPER, N_( "Ink dropper" ), 
 			0, 0, GETTEXT_PACKAGE },
 		{ STOCK_DUPLICATE, N_( "D_uplicate" ), 
-			GDK_CONTROL_MASK, 'u', GETTEXT_PACKAGE },
+			0, 0, GETTEXT_PACKAGE },
 		{ STOCK_PAINTBRUSH, N_( "Pen" ), 
 			0, 0, GETTEXT_PACKAGE },
 		{ STOCK_LINE, N_( "Line" ), 
@@ -715,8 +718,7 @@ main_x_init( int *argc, char ***argv )
 		"ipgtkrc", NULL, NULL, NULL );
 	gtk_init( argc, argv );
 
-	/* Set the default icon. gtk+-2.2.x onwards, ought to check for this
-	 * really.
+	/* Set the default icon. 
 	 */
 	expand_variables( "$VIPSHOME/share/$PACKAGE/data/vips-128.png", buf );
         nativeize_path( buf );
@@ -753,8 +755,8 @@ main_x_init( int *argc, char ***argv )
 
 	/* Load up any saved accelerators.
 	 */
-	call_string_filename( (call_string_fn) gtk_accel_map_load,
-		IP_SAVEDIR IM_DIR_SEP_STR "accel_map", NULL, NULL, NULL );
+	call_string_filenamef( (call_string_fn) gtk_accel_map_load,
+		"%s" IM_DIR_SEP_STR "accel_map", get_savedir() );
 }
 
 static void *
@@ -771,10 +773,17 @@ static void
 main_toobig_done( iWindow *iwnd, 
 	void *client, iWindowNotifyFn nfn, void *sys )
 {
-	path_map_dir( PATH_TMP, "*",
+	/* Don't "rm *", too dangerous. There can (I think?) only be ws and
+	 * VIPS files.
+	 */
+	path_map_dir( PATH_TMP, "*.v",
+		(path_map_fn) main_toobig_done_sub, NULL );
+	path_map_dir( PATH_TMP, "*.desc",
+		(path_map_fn) main_toobig_done_sub, NULL );
+	path_map_dir( PATH_TMP, "*.ws",
 		(path_map_fn) main_toobig_done_sub, NULL );
 
-	/* Tells space-free indicators to update.
+	/* Tell space-free indicators to update.
 	 */
 	if( main_imageinfogroup )
 		iobject_changed( IOBJECT( main_imageinfogroup ) );
@@ -807,8 +816,8 @@ main_check_temp( double total )
 			NULL, NULL,
 			_( "Empty temp area" ),
 			_( "Many files in temp area." ),
-			_( "The temp area \"%s\" contains %s of files.\n"
-			"Would you like to empty the temp area?\n"
+			_( "The temp area \"%s\" contains %s of files. "
+			"Would you like to empty the temp area? "
 			"This will delete any workspace backups and "
 			"cannot be undone." ),
 			tmp, buf_all( &buf ) );
@@ -856,6 +865,20 @@ main( int argc, char *argv[] )
 	printf( "%s: startup timer zeroed ...\n", argv[0] );
 #endif /*DEBUG_TIME*/
 
+	/* Set localised application name.
+	 */
+	g_set_application_name( _( PACKAGE ) );
+
+	/* Set prgname ... do this ourselves in case we don't start gtk.
+	 */
+	{
+		char *prgname;
+
+		prgname = g_path_get_basename( argv[0] );
+		g_set_prgname( prgname );
+		g_free( prgname );
+	}
+
 	/* Init object system. Need to call this explicitly in case we don't
 	 * start the GUI.
 	 */
@@ -885,30 +908,43 @@ main( int argc, char *argv[] )
 	nativeize_path( argv[0] );
 
 	main_argv0 = argv[0];
-	main_name = im_skip_dir( main_argv0 );
 	main_c_stack_base = &argc;
 
 	/* Pass config.h stuff down to .ws files.
 	 */
-	putenvf( "PACKAGE=%s", PACKAGE );
-	putenvf( "VERSION=%s", VERSION );
+	setenvf( "PACKAGE", "%s", PACKAGE );
+	setenvf( "VERSION", "%s", VERSION );
 
 #ifdef HAVE_WINDOWS_H
         /* No HOME on windows ... make one from HOMEDRIVE and HOMEDIR (via
          * glib).
          */
-        if( !getenv( "HOME" ) )
-                putenvf( "HOME=%s", g_get_home_dir() );
+        if( !g_getenv( "HOME" ) )
+                setenvf( "HOME", "%s", g_get_home_dir() );
 #endif /*HAVE_WINDOWS_H*/
 
+	/* Name of the dir we store our config stuff in. This can get used by
+	 * Preferences.ws.
+	 */
+	setenvf( "SAVEDIR", "%s", get_savedir() );
+
 	/* Start up vips.
 	 */
 	if( im_init_world( main_argv0 ) )
 		error_exit( "unable to start VIPS" );
 
+	/* If VIPS has been configured without threads, we will need to start
+	 * the thread system ourselves so that we can make the async queue in
+	 * conversion.c
+	 */
+#ifdef G_THREADS_ENABLED
+        if( !g_thread_supported() )
+                g_thread_init( NULL );
+#endif /*G_THREADS_ENABLED*/
+
 	/* Try to discover our data files.
 	 */
-	if( !(prefix = im_guess_prefix( main_argv0, "VIPSHOME" )) ) 
+	if( !(prefix = get_vipshome( main_argv0 )) ) 
 		error_exit( "unable to guess VIPSHOME" );
 
 	/* Init i18n ... get catalogues from $VIPSHOME/share/locale so we're
@@ -991,13 +1027,13 @@ main( int argc, char *argv[] )
 
 	/* Do we have a save dir? make one if not.
 	 */
-	if( !existsf( "%s", IP_SAVEDIR ) ) {
-		if( !mkdirf( "%s", IP_SAVEDIR ) || 
-			!mkdirf( "%s", IP_SAVEDIR IM_DIR_SEP_STR "tmp" ) || 
-			!mkdirf( "%s", IP_SAVEDIR IM_DIR_SEP_STR "start" ) || 
-			!mkdirf( "%s", IP_SAVEDIR IM_DIR_SEP_STR "data" ) ) 
+	if( !existsf( "%s", get_savedir() ) ) {
+		if( !mkdirf( "%s", get_savedir() ) || 
+			!mkdirf( "%s" IM_DIR_SEP_STR "tmp", get_savedir() ) || 
+			!mkdirf( "%s" IM_DIR_SEP_STR "start", get_savedir() ) || 
+			!mkdirf( "%s" IM_DIR_SEP_STR "data", get_savedir() ) ) 
 			error_exit( _( "unable to make %s area: %s" ),
-				IP_SAVEDIR, g_strerror( errno ) );
+				get_savedir(), g_strerror( errno ) );
 
 		/* Can't pop up the welcome message now, wait until we start X.
 		 */
@@ -1010,8 +1046,8 @@ main( int argc, char *argv[] )
 	fftw_import_system_wisdom();
 #endif /*HAVE_FFTW3*/
 #if HAVE_FFTW || HAVE_FFTW3
-	if( (of = file_open_read( "%s", 
-		IP_SAVEDIR IM_DIR_SEP_STR "wisdom" )) ) {
+	if( (of = file_open_read( "%s" IM_DIR_SEP_STR "wisdom", 
+		get_savedir() )) ) {
 		fftw_import_wisdom_from_file( of->fp );
 		file_close( of );
 	}
@@ -1057,6 +1093,7 @@ main( int argc, char *argv[] )
 			Toolkit *kit;
 
 			main_interactive = FALSE;
+			main_load_all = FALSE;
 			action_on_exit = EXIT_ACTION_PRINT_MAIN;
 
 			/* Load stdin as a load of defs. Good for HEREIS 
@@ -1082,6 +1119,7 @@ main( int argc, char *argv[] )
 		else if( strcmp( argv[1], "-script" ) == 0 ) {
 			action_on_exit = EXIT_ACTION_PRINT_MAIN;
 			main_interactive = FALSE;
+			main_load_all = FALSE;
 
 			if( argc < 3 )
 				error( _( "-script needs an argument" ) );
@@ -1102,6 +1140,18 @@ main( int argc, char *argv[] )
 			 */
 			main_interactive = FALSE;
 		}
+		else if( strcmp( argv[1], "-time_save" ) == 0 ) {
+			/* Pop up save timers.
+			 */
+			main_time_save = TRUE;
+		}
+		else if( strcmp( argv[1], "-i18n" ) == 0 ) {
+			/* Just start up and shutdown, no X. Output constant
+			 * i18n strings.
+			 */
+			main_interactive = FALSE;
+			compile_set_i18n( TRUE );
+		}
 	}
 
 	/* Make sure we have a start workspace.
@@ -1124,7 +1174,7 @@ main( int argc, char *argv[] )
 
 	/* Load start-up stuff.
 	 */
-	main_load_all();
+	main_load_startup();
 
 	/* Recalc to build all classes.
 	 */
@@ -1222,16 +1272,14 @@ main( int argc, char *argv[] )
 
 			im_snprintf( buf, 256, 
 				_( "Welcome to %s-%s!" ), PACKAGE, VERSION );
-			expand_variables( IP_SAVEDIR, save_dir );
+			expand_variables( get_savedir(), save_dir );
 			nativeize_path( save_dir );
 			box_info( NULL, 
 				buf,
-_( "A new directory has been created in your home directory to hold startup,\n"
-"data and temporary files:\n"
-"\n"
-"     %s\n"
-"\n"
-"If you've used previous versions of %s, you will probably want\n"
+_( "A new directory has been created in your home directory to hold startup, "
+"data and temporary files:\n\n"
+"     %s\n\n"
+"If you've used previous versions of %s, you will probably want "
 "to move any files over from your old work area and remove any old temps." ),
 				save_dir, PACKAGE );
 		}
