@@ -51,7 +51,7 @@ matrix_finalize( GObject *gobject )
 
 	/* My instance finalize stuff.
 	 */
-	FREE( matrix->value );
+	IM_FREE( matrix->value );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
@@ -81,7 +81,7 @@ matrix_resize( Matrix *matrix, int width, int height )
 
 	/* Install new values.
 	 */
-	FREE( matrix->value );
+	IM_FREE( matrix->value );
 	matrix->value = value;
 	matrix->width = width;
 	matrix->height = height;
@@ -235,7 +235,7 @@ matrix_load( Model *model,
 
 	assert( sizeof( matrix->display ) == sizeof( int ) );
 
-	FREE( matrix->value );
+	IM_FREE( matrix->value );
 	if( get_dlprop( xnode, "value", &matrix->value ) &&
 		get_iprop( xnode, "width", &matrix->width ) &&
 		get_iprop( xnode, "height", &matrix->height ) &&
@@ -322,10 +322,10 @@ matrix_graphic_save( Classmodel *classmodel,
 
 	if( im_write_dmask_name( dmask, filename ) ) {
 		error_vips_all();
-		FREEF( im_free_dmask, dmask );
+		IM_FREEF( im_free_dmask, dmask );
 		return( FALSE );
 	}
-	FREEF( im_free_dmask, dmask );
+	IM_FREEF( im_free_dmask, dmask );
 
 	mainw_recent_add( &mainw_recent_matrix, filename );
 
@@ -352,9 +352,10 @@ matrix_graphic_replace( Classmodel *classmodel,
 	matrix_dmask_to_ip( dmask, &buf );
 	im_free_dmask( dmask );
 
-	itext_set_formula( itext, buf_all( &buf ) );
-	itext_set_edited( itext, TRUE );
-	(void) expr_dirty( row->expr, link_serial_new() );
+	if( itext_set_formula( itext, buf_all( &buf ) ) ) {
+		itext_set_edited( itext, TRUE );
+		(void) expr_dirty( row->expr, link_serial_new() );
+	}
 
 	mainw_recent_add( &mainw_recent_matrix, filename );
 
@@ -389,7 +390,7 @@ matrix_class_init( MatrixClass *class )
 	classmodel_class->graphic_replace = matrix_graphic_replace;
 
 	classmodel_class->filetype = filesel_type_matrix;
-	classmodel_class->default_filetype = 0;
+	classmodel_class->filetype_pref = "MATRIX_FILE_TYPE";
 
 	/* Static init.
 	 */
@@ -620,7 +621,9 @@ matrix_ip_to_dmask( PElement *root )
 	if( class_get_member_string( root, MEMBER_FILENAME, buf, MAX_STRSIZE ) )
 		filename = buf;
 	else {
-		temp_name( name, "mat" );
+		if( !temp_name( name, "mat" ) )
+			return( NULL );
+
 		filename = name;
 	}
 
@@ -631,7 +634,7 @@ matrix_ip_to_dmask( PElement *root )
 
 	if( !class_get_member_matrix( root, MEMBER_VALUE, 
 		dmask->coeff, width * height, &width, &height ) ) {
-		FREEF( im_free_dmask, dmask );
+		IM_FREEF( im_free_dmask, dmask );
 		return( FALSE );
 	}
 
@@ -657,7 +660,7 @@ matrix_ip_to_imask( PElement *root )
 		return( NULL );
 
 	if( !(imask = matrix_dmask_to_imask( dmask )) ) {
-		FREEF( im_free_dmask, dmask );
+		IM_FREEF( im_free_dmask, dmask );
 		return( NULL );
 	}
 
