@@ -69,9 +69,9 @@ ModelLoadState *model_loadstate = NULL;
 static void *
 model_loadstate_rename_destroy( ModelRename *rename )
 {
-	FREE( rename->old_name );
-	FREE( rename->new_name );
-	FREE( rename );
+	IM_FREE( rename->old_name );
+	IM_FREE( rename->new_name );
+	IM_FREE( rename );
 
 	return( NULL );
 }
@@ -103,12 +103,12 @@ model_loadstate_destroy( ModelLoadState *state )
 	 */
 	xmlSetGenericErrorFunc( NULL, NULL );
 
-	FREE( state->filename );
-	FREEF( xmlFreeDoc, state->xdoc );
+	IM_FREE( state->filename );
+	IM_FREEF( xmlFreeDoc, state->xdoc );
 	slist_map( state->renames, 
 		(SListMapFn) model_loadstate_rename_destroy, NULL );
 	g_slist_free( state->renames );
-	FREE( state );
+	IM_FREE( state );
 }
 
 static void
@@ -128,8 +128,7 @@ model_loadstate_error_get( ModelLoadState *state )
 
 	utf8 = f2utf8( buf_all( &state->error_log ) );
 	error_top( _( "Load failed." ) );
-	error_sub( _( "Unable to load from file \"%s\".\n"
-		"Error log is:\n%s" ),
+	error_sub( _( "Unable to load from file \"%s\". Error log is:\n%s" ),
 		state->filename, utf8 );
 	g_free( utf8 );
 }
@@ -254,7 +253,7 @@ model_loadstate_rewrite( ModelLoadState *state, char *old_rhs, char *new_rhs )
 			break;
 
 		case TK_IDENT:
-			FREE( yylval.yy_name );
+			IM_FREE( yylval.yy_name );
 			break;
 
 		default:
@@ -687,7 +686,7 @@ model_check_destroy_finished( void *client, iWindowResult result )
 	ModelCheckDestroy *mcd = (ModelCheckDestroy *) client;
 
 	FREESID( mcd->destroy_sid, mcd->model );
-	FREE( mcd );
+	IM_FREE( mcd );
 }
 
 void
