@@ -200,9 +200,9 @@ paintboxview_clear_cb( GtkWidget *widget, Paintboxview *pbv )
 		iwindow_notify_null, NULL,
 		GTK_STOCK_CLEAR,
 		_( "Clear undo history?" ), 
-		_( "Are you sure you want to clear all undo and redo?\n"
-		"This will free up memory, but you will no longer be\n"
-		"able to undo or redo any of the painting you have\n"
+		_( "Are you sure you want to clear all undo and redo? "
+		"This will free up memory, but you will no longer be "
+		"able to undo or redo any of the painting you have "
 		"done so far." ) );
 }
 
