@@ -41,6 +41,7 @@ GSList *path_session = NULL;
  */
 GSList *path_start_default = NULL;
 GSList *path_search_default = NULL;
+const char *path_tmp_default = NULL;
 
 /* Turn a search path (eg. "/pics/lr:/pics/hr") into a list of directory names.
  */
@@ -111,9 +112,9 @@ path_add_string( const char *str, char *buf )
 char *
 path_unparse( GSList *path )
 {	
-	int len = (int) slist_fold( path, 0, 
-		(SListFoldFn) path_add_component, NULL );
-	char *buf = im_malloc( NULL, len + 1 );
+	int len = GPOINTER_TO_INT( slist_fold( path, 0, 
+		 (SListFoldFn) path_add_component, NULL ) );
+	char *buf = imalloc( NULL, len + 1 );
 
 	/* Build new string.
 	 */
@@ -229,8 +230,8 @@ path_map_exact( GSList *path, const char *patt, path_map_fn fn, void *a )
 	ru = slist_map5( path, (SListMap5Fn) path_scan_dir, 
 		wild, (void *) path_next_item, &previous, (void *) fn, a );
 
-	FREEF( slist_free_all, previous );
-	FREEF( wild_destroy, wild );
+	IM_FREEF( slist_free_all, previous );
+	IM_FREEF( wild_destroy, wild );
 
 	return( ru );
 }
@@ -255,8 +256,8 @@ path_map( GSList *path, const char *patt, path_map_fn fn, void *a )
 			wild, (void *) path_next_item, &previous, 
 			(void *) fn, a );
 
-	FREEF( slist_free_all, previous );
-	FREEF( wild_destroy, wild );
+	IM_FREEF( slist_free_all, previous );
+	IM_FREEF( wild_destroy, wild );
 
 	return( ru );
 }
@@ -277,7 +278,7 @@ path_map_dir( const char *dir, const char *patt, path_map_fn fn, void *a )
 		error_top( _( "Not found." ) );
 		error_sub( _( "File \"%s\" not found." ), patt );
 	}
-	FREEF( wild_destroy, wild );
+	IM_FREEF( wild_destroy, wild );
 
 	return( ru );
 }
@@ -341,14 +342,27 @@ path_init( void )
 	printf( "path_init: loading start from \".\" only\n" );
 	path_start_default = path_parse( "." );
 	path_search_default = path_parse( "." );
-#else /*DEBUG*/
-	path_start_default = path_parse( 
-		IP_SAVEDIR IM_DIR_SEP_STR "start" IM_PATH_SEP_STR
+	path_tmp_default = im_strdup( NULL, "." );
+#else /*!DEBUG*/
+	char buf[FILENAME_MAX];
+
+	im_snprintf( buf, FILENAME_MAX,
+		"%s" IM_DIR_SEP_STR "start" IM_PATH_SEP_STR
+		"$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR
+			PACKAGE IM_DIR_SEP_STR "start",
+		get_savedir() );
+	path_start_default = path_parse( buf );
+
+	im_snprintf( buf, FILENAME_MAX,
+		"%s" IM_DIR_SEP_STR "data" IM_PATH_SEP_STR
 		"$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR 
-		PACKAGE IM_DIR_SEP_STR "start" );
-	path_search_default = path_parse( 
-		IP_SAVEDIR IM_DIR_SEP_STR "data" IM_PATH_SEP_STR
-		"$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR 
-		PACKAGE IM_DIR_SEP_STR "data" IM_PATH_SEP_STR "." );
+			PACKAGE IM_DIR_SEP_STR "data" IM_PATH_SEP_STR 
+		".",
+		get_savedir() );
+	path_search_default = path_parse( buf );
+
+	im_snprintf( buf, FILENAME_MAX, 
+		"%s" IM_DIR_SEP_STR "tmp", get_savedir() );
+	path_tmp_default = im_strdup( NULL, buf );
 #endif /*DEBUG*/
 }
