@@ -428,7 +428,7 @@ reduce_get_line_size( Reduce *rc, PElement *base, int *w, int *h )
 		*w = l;
 	else if( *w != l ) {
 		error_top( _( "Not rectangular." ) );
-		error_sub( _( "Matrix of real is not rectangular.\n"
+		error_sub( _( "Matrix of real is not rectangular. "
 			"Found row of length %d, should be %d." ), l, *w );
 		reduce_throw( rc );
 	}
@@ -487,7 +487,7 @@ reduce_get_line( Reduce *rc, PElement *base, GetMatrixInfo *gmi )
 		gmi->w = l;
 	else if( l != gmi->w ) {
 		error_top( _( "Not rectangular." ) );
-		error_sub( _( "Matrix of real is not rectangular.\n"
+		error_sub( _( "Matrix of real is not rectangular. "
 			"Found row of length %d, should be %d." ), l, gmi->w );
 		reduce_throw( rc );
 	}
@@ -831,7 +831,7 @@ is_WHNF( PElement *out )
 		PEISCOMPLEX( out ) || PEISNUM( out ) || PEISCHAR( out ) ||
 		PEISBOOL( out ) || PEISTAG( out ) || PEISIMAGE( out ) ||
 		PEISLIST( out ) || PEISCLASS( out ) || PEISSYMREF( out ) ||
-		PEISCOMPILEREF( out ) || PEISNOVAL( out ) )
+		PEISCOMPILEREF( out ) || PEISNOVAL( out ) || PEISSTATIC( out ) )
 		return( TRUE );
 
 	/* Must be a function ... loop down the spine, counting args.
@@ -998,6 +998,17 @@ reduce_start:
 		goto reduce_start;
 	}
 
+	case ELEMENT_STATIC:
+	{
+		HeapStaticString *string = PEGETSTATIC( &np );
+
+		/* Link to compiled string.
+		 */
+		PEPUTE( &np, &string->e );
+
+		goto reduce_start;
+	}
+
 	case ELEMENT_SYMBOL:
 	{
 		Symbol *sym = PEGETSYMBOL( &np );
@@ -1803,7 +1814,7 @@ reduce_destroy( Reduce *rc )
 {
 	heap_unregister_reduce( rc->heap, rc );
 	UNREF( rc->heap );
-	FREE( rc );
+	IM_FREE( rc );
 }
 
 /* Max cells function for main reduce engine. Read from Preferences, and scale
@@ -1820,9 +1831,10 @@ reduce_heap_max_fn( Heap *heap )
 Reduce *
 reduce_new( void )
 {
-	/* Initial heap size.
+	/* Initial heap size. Big enough that we won't need to grow just
+	 * loading prefs and standard stuff.
 	 */
-	const int stsz = 20000;
+	const int stsz = 100000;
 
 	/* Heap increment..
 	 */
