@@ -43,13 +43,34 @@
 #define EVENT
  */
 
+/* See paint events.
+#define DEBUG_PAINT
+ */
+
 /* Define this to make region drags default to no-update during drag/resize.
 #define NO_UPDATE
  */
 
 #include "ip.h"
 
-static ViewClass *parent_class = NULL;
+typedef void *(*regionview_rect_fn)( Regionview *, Rect *, void * );
+typedef void (*regionview_paint_fn)( Regionview * );
+
+/* Cursor shape for each resize type.
+ */
+iWindowShape regionview_cursors[REGIONVIEW_RESIZE_LAST] = {
+	IWINDOW_SHAPE_EDIT,		/* REGIONVIEW_RESIZE_NONE */
+	IWINDOW_SHAPE_MOVE, 		/* REGIONVIEW_RESIZE_MOVE */
+	IWINDOW_SHAPE_MOVE,		/* REGIONVIEW_RESIZE_EDIT */
+	IWINDOW_SHAPE_TOPLEFT,		/* REGIONVIEW_RESIZE_TOPLEFT */
+	IWINDOW_SHAPE_TOP,		/* REGIONVIEW_RESIZE_TOP */
+	IWINDOW_SHAPE_TOPRIGHT,		/* REGIONVIEW_RESIZE_TOPRIGHT */
+	IWINDOW_SHAPE_RIGHT,		/* REGIONVIEW_RESIZE_RIGHT */
+	IWINDOW_SHAPE_BOTTOMRIGHT,	/* REGIONVIEW_RESIZE_BOTTOMRIGHT */
+	IWINDOW_SHAPE_BOTTOM,		/* REGIONVIEW_RESIZE_BOTTOM */
+	IWINDOW_SHAPE_BOTTOMLEFT,	/* REGIONVIEW_RESIZE_BOTTOMLEFT */
+	IWINDOW_SHAPE_LEFT		/* REGIONVIEW_RESIZE_LEFT */
+};
 
 /* Region border width, without shadows.
  */
@@ -67,13 +88,12 @@ static const int regionview_crosshair_length = 5;
  */
 static const int regionview_morph_threshold = 20;
 
+static ViewClass *parent_class = NULL;
+
 /* Just one popup for all regions.
  */
 static GtkWidget *regionview_popup_menu = NULL;
 
-typedef void *(*regionview_rect_fn)( Regionview *, Rect *, void * );
-typedef void (*regionview_paint_fn)( Regionview * );
-
 /* Paint a rectangle. 
  */
 void
@@ -354,10 +374,10 @@ regionview_border_foreach( Regionview *regionview,
 static void *
 regionview_queue_draw_area( Regionview *regionview, Rect *area )
 {
-#ifdef DEBUG
+#ifdef DEBUG_PAINT
 	printf( "regionview_queue_draw_area: at %dx%d size %dx%d\n",
 		area->left, area->top, area->width, area->height );
-#endif /*DEBUG*/
+#endif /*DEBUG_PAINT*/
 
 	imagedisplay_queue_draw_area( regionview->ip->id, area );
 
@@ -506,10 +526,21 @@ regionview_paint( Regionview *regionview )
 static void
 regionview_detach( Regionview *regionview )
 {
-	regionview->state = REGIONVIEW_WAIT;
-	regionview->paint_state = GTK_STATE_PRELIGHT;
-	regionview->grabbed = FALSE;
-	imagepresent_scroll_stop( regionview->ip );
+	if( regionview->grabbed ) {
+		assert( regionview->ip->grabbed == regionview );
+
+#ifdef DEBUG_GRAB
+		printf( "regionview_detach: 0x%x\n", 
+			(unsigned int) regionview );
+#endif /*DEBUG_GRAB*/
+
+		regionview->state = REGIONVIEW_WAIT;
+		regionview->paint_state = GTK_STATE_PRELIGHT;
+		regionview->grabbed = FALSE;
+		regionview->ip->grabbed = NULL;
+
+		imagepresent_scroll_stop( regionview->ip );
+	}
 }
 
 static void
@@ -519,7 +550,7 @@ regionview_destroy( GtkObject *object )
 	Imagedisplay *id;
 
 #ifdef DEBUG_MAKE
-	printf( "regionview_destroy\n" );
+	printf( "regionview_destroy: 0x%x\n", (unsigned int) regionview );
 #endif /*DEBUG_MAKE*/
 
 	g_return_if_fail( object != NULL );
@@ -540,14 +571,18 @@ regionview_destroy( GtkObject *object )
 	FREESID( regionview->changed_sid, id->conv ); 
 	FREESID( regionview->conv_destroy_sid, id->conv ); 
 	FREESID( regionview->model_changed_sid, regionview->classmodel ); 
-	FREEFI( g_source_remove, regionview->dash_crawl );
-	FREEF( iwindow_cursor_context_destroy, regionview->cntxt );
+	IM_FREEF( g_source_remove, regionview->dash_crawl );
+	IM_FREEF( iwindow_cursor_context_destroy, regionview->cntxt );
 	buf_destroy( &regionview->caption );
 
 	if( regionview->ip ) {
+		if( regionview->ip->regionview == regionview )
+			regionview->ip->regionview = NULL;
+
 		regionview->ip->regionviews = 
 			g_slist_remove( regionview->ip->regionviews, 
 			regionview );
+
 		regionview->ip = NULL;
 	}
 
@@ -734,6 +769,12 @@ regionview_update_from_model( Regionview *regionview )
 {
 	Rect *model_area = regionview_get_model( regionview );
 
+#ifdef DEBUG
+	printf( "regionview_update_from_model: model is %dx%d size %dx%d\n",
+		model_area->left, model_area->top, 
+		model_area->width, model_area->height );
+#endif /*DEBUG*/
+
 	regionview->our_area = *model_area;
 
 	if( regionview_get_compatibility( regionview ) ) {
@@ -743,6 +784,12 @@ regionview_update_from_model( Regionview *regionview )
 		regionview->our_area.left += im->Xoffset;
 		regionview->our_area.top += im->Yoffset;
 	}
+
+#ifdef DEBUG
+	printf( "regionview_update_from_model: set regionview to %dx%d size %dx%d\n",
+		regionview->our_area.left, regionview->our_area.top, 
+		regionview->our_area.width, regionview->our_area.height );
+#endif /*DEBUG*/
 }
 
 /* Update the model from our_area.
@@ -754,14 +801,20 @@ regionview_model_update( Regionview *regionview )
 	Imagepresent *ip = regionview->ip;
 	Rect *model_area = regionview_get_model( regionview );
 
+#ifdef DEBUG
+	printf( "regionview_model_update: regionview is %dx%d size %dx%d\n",
+		regionview->our_area.left, regionview->our_area.top, 
+		regionview->our_area.width, regionview->our_area.height );
+#endif /*DEBUG*/
+
 	*model_area = regionview->our_area;
 
 	if( regionview_get_compatibility( regionview ) ) {
 		Conversion *conv = ip->id->conv;
 		IMAGE *im = imageinfo_get( FALSE, conv->ii );
 
-		regionview->our_area.left -= im->Xoffset;
-		regionview->our_area.top -= im->Yoffset;
+		model_area->left -= im->Xoffset;
+		model_area->top -= im->Yoffset;
 	}
 
 	if( classmodel ) {
@@ -781,6 +834,12 @@ regionview_model_update( Regionview *regionview )
 	/* Refresh immediately .. gives faster feedback during drag.
 	 */
 	vobject_refresh( VOBJECT( regionview ) );
+
+#ifdef DEBUG
+	printf( "regionview_model_update: set model to %dx%d size %dx%d\n",
+		model_area->left, model_area->top, 
+		model_area->width, model_area->height );
+#endif /*DEBUG*/
 }
 
 /* Our model has changed ... undraw in the old position, draw in the new
@@ -921,11 +980,11 @@ regionview_class_init( RegionviewClass *class )
         /* Other init.
          */
 	pane = regionview_popup_menu = popup_build( _( "Region menu" ) );
-	popup_add_but( pane, _( "Edit ..." ), 
+	popup_add_but( pane, _( "_Edit" ), 
 		POPUP_FUNC( regionview_edit_cb ) );
 	popup_add_but( pane, STOCK_DUPLICATE,
 		POPUP_FUNC( regionview_clone_cb ) );
-	popup_add_but( pane, _( "Reset" ), 
+	popup_add_but( pane, _( "_Reset" ), 
 		POPUP_FUNC( regionview_clear_edited_cb ) );
 	menu_add_sep( pane );
 	popup_add_but( pane, GTK_STOCK_DELETE,
@@ -945,7 +1004,7 @@ regionview_init( Regionview *regionview )
 	regionview->frozen = TRUE;
 
 	regionview->state = REGIONVIEW_WAIT;
-	regionview->shape = IWINDOW_SHAPE_NONE;
+	regionview->resize = REGIONVIEW_RESIZE_NONE;
 	regionview->dx = -1;
 	regionview->dy = -1;
 	regionview->grabbed = FALSE;
@@ -1119,10 +1178,10 @@ regionview_rect_touches_region( Regionview *regionview, Rect *expose )
 static void
 regionview_expose( Regionview *regionview, Rect *expose )
 {
-#ifdef DEBUG
+#ifdef DEBUG_PAINT
 	printf( "regionview_expose: at %dx%d size %dx%d\n",
 		expose->left, expose->top, expose->width, expose->height );
-#endif /*DEBUG*/
+#endif /*DEBUG_PAINT*/
 
 	assert( expose->width >= 0 && expose->height >= 0 );
 
@@ -1175,10 +1234,10 @@ regionview_point_in_region( Regionview *regionview, int x, int y )
 	return( regionview_rect_touches_region( regionview, &r ) );
 }
 
-/* Given a position, find the sort of cursor we should show.
+/* Given a position, find the sort of resize we should allow.
  */
-static iWindowShape
-regionview_find_shape( Regionview *regionview, int x, int y )
+static RegionviewResize
+regionview_find_resize( Regionview *regionview, int x, int y )
 {
 	Conversion *conv = regionview->ip->id->conv;
 
@@ -1186,7 +1245,7 @@ regionview_find_shape( Regionview *regionview, int x, int y )
 	int dx, dy; 
 
 	if( im_rect_includespoint( &regionview->label, x, y ) )
-		return( IWINDOW_SHAPE_EDIT );
+		return( REGIONVIEW_RESIZE_EDIT );
 
 	conversion_im_to_disp_rect( conv, &regionview->area, &canvas_area );
 	dx = x - canvas_area.left;
@@ -1196,31 +1255,31 @@ regionview_find_shape( Regionview *regionview, int x, int y )
 	case REGIONVIEW_REGION:
 		if( dx > canvas_area.width - 10 ) {
 			if( dy > canvas_area.height - 10 )
-				return( IWINDOW_SHAPE_BOTTOMRIGHT );
+				return( REGIONVIEW_RESIZE_BOTTOMRIGHT );
 			else if( dy < 10 )
-				return( IWINDOW_SHAPE_TOPRIGHT );
+				return( REGIONVIEW_RESIZE_TOPRIGHT );
 			else 
-				return( IWINDOW_SHAPE_RIGHT );
+				return( REGIONVIEW_RESIZE_RIGHT );
 		}
 		else if( dx < 10 ) {
 			if( dy > canvas_area.height - 10 )
-				return( IWINDOW_SHAPE_BOTTOMLEFT );
+				return( REGIONVIEW_RESIZE_BOTTOMLEFT );
 			else if( dy < 10 )
-				return( IWINDOW_SHAPE_TOPLEFT );
+				return( REGIONVIEW_RESIZE_TOPLEFT );
 			else 
-				return( IWINDOW_SHAPE_LEFT );
+				return( REGIONVIEW_RESIZE_LEFT );
 		}
 		else {
 			if( dy < canvas_area.height / 2 )
-				return( IWINDOW_SHAPE_TOP );
+				return( REGIONVIEW_RESIZE_TOP );
 			else
-				return( IWINDOW_SHAPE_BOTTOM );
+				return( REGIONVIEW_RESIZE_BOTTOM );
 		}
 		break;
 
 	case REGIONVIEW_MARK:
 	case REGIONVIEW_AREA:
-		return( IWINDOW_SHAPE_MOVE );
+		return( REGIONVIEW_RESIZE_MOVE );
 
 	case REGIONVIEW_ARROW:
 		tiny.left = x;
@@ -1230,12 +1289,12 @@ regionview_find_shape( Regionview *regionview, int x, int y )
 		if( regionview_crosshair_foreach( regionview,
 			canvas_area.left, canvas_area.top,
 			(regionview_rect_fn) regionview_rect_touching, &tiny ) )
-			return( IWINDOW_SHAPE_TOPLEFT );
+			return( REGIONVIEW_RESIZE_TOPLEFT );
 		if( regionview_crosshair_foreach( regionview,
 			IM_RECT_RIGHT( &canvas_area ), 
 			IM_RECT_BOTTOM( &canvas_area ),
 			(regionview_rect_fn) regionview_rect_touching, &tiny ) )
-			return( IWINDOW_SHAPE_BOTTOMRIGHT );
+			return( REGIONVIEW_RESIZE_BOTTOMRIGHT );
 
 		/* Extra tests ... allow grabs in the centre of the crosshairs
 		 * too.
@@ -1246,7 +1305,7 @@ regionview_find_shape( Regionview *regionview, int x, int y )
 		tiny.height = 1;
 		im_rect_marginadjust( &tiny, regionview_border_width );
 		if( im_rect_includespoint( &tiny, x, y ) )
-			return( IWINDOW_SHAPE_BOTTOMRIGHT );
+			return( REGIONVIEW_RESIZE_BOTTOMRIGHT );
 
 		tiny.left = canvas_area.left;
 		tiny.top = canvas_area.top;
@@ -1254,21 +1313,21 @@ regionview_find_shape( Regionview *regionview, int x, int y )
 		tiny.height = 1;
 		im_rect_marginadjust( &tiny, regionview_border_width );
 		if( im_rect_includespoint( &tiny, x, y ) )
-			return( IWINDOW_SHAPE_TOPLEFT );
+			return( REGIONVIEW_RESIZE_TOPLEFT );
 
 		break;
 
 	case REGIONVIEW_HGUIDE:
 		im_rect_marginadjust( &canvas_area, 1 );
 		if( im_rect_includespoint( &canvas_area, x, y ) )
-			return( IWINDOW_SHAPE_MOVE );
+			return( REGIONVIEW_RESIZE_MOVE );
 
 		break;
 
 	case REGIONVIEW_VGUIDE:
 		im_rect_marginadjust( &canvas_area, 1 );
 		if( im_rect_includespoint( &canvas_area, x, y ) )
-			return( IWINDOW_SHAPE_MOVE );
+			return( REGIONVIEW_RESIZE_MOVE );
 
 		break;
 
@@ -1280,7 +1339,7 @@ regionview_find_shape( Regionview *regionview, int x, int y )
 		assert( FALSE );
 	}
 
-	return( IWINDOW_SHAPE_NONE );
+	return( REGIONVIEW_RESIZE_NONE );
 }
 
 /* Right button press event.
@@ -1302,19 +1361,25 @@ regionview_right_press( Regionview *regionview, GdkEvent *ev, int x, int y )
 void
 regionview_attach( Regionview *regionview, int x, int y )
 {
-	Conversion *conv = regionview->ip->id->conv;
+	Imagepresent *ip = regionview->ip;
+	Conversion *conv = ip->id->conv;
 	int dx, dy;
 
 	assert( !regionview->grabbed );
+	assert( !regionview->ip->grabbed );
 
-	switch( regionview->shape ) {
-	case IWINDOW_SHAPE_NONE:
-		regionview->shape = IWINDOW_SHAPE_BOTTOMRIGHT;
+#ifdef DEBUG_GRAB
+	printf( "regionview_attach: 0x%x\n", (unsigned int) regionview );
+#endif /*DEBUG_GRAB*/
+
+	switch( regionview->resize ) {
+	case REGIONVIEW_RESIZE_NONE:
+		regionview->resize = REGIONVIEW_RESIZE_BOTTOMRIGHT;
 		regionview->state = REGIONVIEW_RESIZE;
 		break;
 
-	case IWINDOW_SHAPE_MOVE:
-	case IWINDOW_SHAPE_EDIT:
+	case REGIONVIEW_RESIZE_MOVE:
+	case REGIONVIEW_RESIZE_EDIT:
 		regionview->state = REGIONVIEW_MOVE;
 		break;
 
@@ -1326,9 +1391,10 @@ regionview_attach( Regionview *regionview, int x, int y )
 	regionview->paint_state = GTK_STATE_ACTIVE;
 
 	iwindow_cursor_context_set_cursor( regionview->cntxt, 
-		regionview->shape );
+		regionview_cursors[regionview->resize] );
 
 	regionview->grabbed = TRUE;
+	regionview->ip->grabbed = regionview;
 
 	conversion_im_to_disp( conv, 
 		regionview->our_area.left, regionview->our_area.top, &dx, &dy );
@@ -1348,9 +1414,9 @@ regionview_left_press( Regionview *regionview, GdkEvent *ev, int x, int y )
 
 	switch( regionview->state ) {
 	case REGIONVIEW_WAIT:
-		regionview->shape = regionview_find_shape( regionview, x, y );
+		regionview->resize = regionview_find_resize( regionview, x, y );
 
-		if( regionview->shape != IWINDOW_SHAPE_NONE ) {
+		if( regionview->resize != REGIONVIEW_RESIZE_NONE ) {
 			regionview_attach( regionview, x, y );
 			handled = TRUE;
 		}
@@ -1407,53 +1473,53 @@ regionview_resize_area( Regionview *regionview, int ix, int iy )
 	/* If we're not frozen, do an unconstrained resize.
 	 */
 	if( !regionview->frozen ) {
-		switch( regionview->shape ) {
-		case IWINDOW_SHAPE_RIGHT:
+		switch( regionview->resize ) {
+		case REGIONVIEW_RESIZE_RIGHT:
 			our_area->width = IM_CLIP( -our_area->left,
 				rx, im->Xsize - our_area->left );
 			break;
 
-		case IWINDOW_SHAPE_BOTTOM:
+		case REGIONVIEW_RESIZE_BOTTOM:
 			our_area->height = IM_CLIP( -our_area->top, 
 				ry, im->Ysize - our_area->top );
 			break;
 
-		case IWINDOW_SHAPE_MOVE:
+		case REGIONVIEW_RESIZE_MOVE:
 			/* Get this for POINT on create ... treat as 
 			 * BOTTOMRIGHT.
 			 */
-		case IWINDOW_SHAPE_BOTTOMRIGHT:
+		case REGIONVIEW_RESIZE_BOTTOMRIGHT:
 			our_area->width = IM_CLIP( -our_area->left,
 				rx, im->Xsize - our_area->left );
 			our_area->height = IM_CLIP( -our_area->top, 
 				ry, im->Ysize - our_area->top );
 			break;
 
-		case IWINDOW_SHAPE_LEFT:
+		case REGIONVIEW_RESIZE_LEFT:
 			our_area->left = IM_CLIP( 0, ix, im->Xsize - 1 );
 			our_area->width = ri - our_area->left;
 			break;
 
-		case IWINDOW_SHAPE_TOP:
+		case REGIONVIEW_RESIZE_TOP:
 			our_area->top = IM_CLIP( 0, iy, im->Ysize - 1 );
 			our_area->height = bot - our_area->top;
 			break;
 
-		case IWINDOW_SHAPE_TOPLEFT:
+		case REGIONVIEW_RESIZE_TOPLEFT:
 			our_area->top = IM_CLIP( 0, iy, im->Ysize - 1 );
 			our_area->left = IM_CLIP( 0, ix, im->Xsize - 1 );
 			our_area->width = ri - our_area->left;
 			our_area->height = bot - our_area->top;
 			break;
 
-		case IWINDOW_SHAPE_TOPRIGHT:
+		case REGIONVIEW_RESIZE_TOPRIGHT:
 			our_area->top = IM_CLIP( 0, iy, im->Ysize - 1 );
 			our_area->height = bot - our_area->top;
 			our_area->width = IM_CLIP( -our_area->left,
 				rx, im->Xsize - our_area->left );
 			break;
 
-		case IWINDOW_SHAPE_BOTTOMLEFT:
+		case REGIONVIEW_RESIZE_BOTTOMLEFT:
 			our_area->left = IM_CLIP( 0, ix, im->Xsize - 1 );
 			our_area->width = ri - our_area->left;
 			our_area->height = IM_CLIP( -our_area->top, 
@@ -1484,49 +1550,49 @@ regionview_resize_area( Regionview *regionview, int ix, int iy )
 		 */
 		switch( regionview->type ) {
 		case REGIONVIEW_REGION:
-			switch( regionview->shape ) {
-			case IWINDOW_SHAPE_RIGHT:
+			switch( regionview->resize ) {
+			case REGIONVIEW_RESIZE_RIGHT:
 				our_area->width = IM_CLIP( 1, rx, 
 					im->Xsize - our_area->left );
 				break;
 
-			case IWINDOW_SHAPE_BOTTOM:
+			case REGIONVIEW_RESIZE_BOTTOM:
 				our_area->height = IM_CLIP( 1, ry, 
 					im->Ysize - our_area->top );
 				break;
 
-			case IWINDOW_SHAPE_BOTTOMRIGHT:
+			case REGIONVIEW_RESIZE_BOTTOMRIGHT:
 				our_area->width = IM_CLIP( 1, rx, 
 					im->Xsize - our_area->left );
 				our_area->height = IM_CLIP( 1, ry, 
 					im->Ysize - our_area->top );
 				break;
 
-			case IWINDOW_SHAPE_TOP:
+			case REGIONVIEW_RESIZE_TOP:
 				our_area->top = IM_CLIP( 0, iy, bot - 1 );
 				our_area->height = bot - our_area->top;
 				break;
 
-			case IWINDOW_SHAPE_LEFT:
+			case REGIONVIEW_RESIZE_LEFT:
 				our_area->left = IM_CLIP( 0, ix, ri - 1 );
 				our_area->width = ri - our_area->left;
 				break;
 
-			case IWINDOW_SHAPE_TOPLEFT:
+			case REGIONVIEW_RESIZE_TOPLEFT:
 				our_area->left = IM_CLIP( 0, ix, ri - 1 );
 				our_area->width = ri - our_area->left;
 				our_area->top = IM_CLIP( 0, iy, bot - 1 );
 				our_area->height = bot - our_area->top;
 				break;
 
-			case IWINDOW_SHAPE_TOPRIGHT:
+			case REGIONVIEW_RESIZE_TOPRIGHT:
 				our_area->top = IM_CLIP( 0, iy, bot - 1 );
 				our_area->height = bot - our_area->top;
 				our_area->width = IM_CLIP( 1, rx, 
 					im->Xsize - our_area->left );
 				break;
 
-			case IWINDOW_SHAPE_BOTTOMLEFT:
+			case REGIONVIEW_RESIZE_BOTTOMLEFT:
 				our_area->left = IM_CLIP( 0, ix, ri - 1 );
 				our_area->width = ri - our_area->left;
 				our_area->height = IM_CLIP( 1, ry, 
@@ -1541,15 +1607,15 @@ regionview_resize_area( Regionview *regionview, int ix, int iy )
 		case REGIONVIEW_ARROW:
 		case REGIONVIEW_LINE:
 		case REGIONVIEW_BOX:
-			switch( regionview->shape ) {
-			case IWINDOW_SHAPE_TOPLEFT:
+			switch( regionview->resize ) {
+			case REGIONVIEW_RESIZE_TOPLEFT:
 				our_area->left = IM_CLIP( 0, ix, im->Xsize );
 				our_area->width = ri - our_area->left;
 				our_area->top = IM_CLIP( 0, iy, im->Ysize );
 				our_area->height = bot - our_area->top;
 				break;
 
-			case IWINDOW_SHAPE_BOTTOMRIGHT:
+			case REGIONVIEW_RESIZE_BOTTOMRIGHT:
 				our_area->width = 
 					IM_CLIP( -our_area->left, rx, 
 						im->Xsize - our_area->left );
@@ -1613,8 +1679,8 @@ regionview_motion_grab( Regionview *regionview, int x, int y )
 #ifdef DEBUG
 	printf( "regionview_motion_grab:\n" );
 	printf( "cods: %dx%d size %dx%d\n",
-		regionview->model_area->left, regionview->model_area->top, 
-		regionview->model_area->width, regionview->model_area->height );
+		our_area->left, our_area->top, 
+		our_area->width, our_area->height );
 #endif /*DEBUG*/
 
 	switch( regionview->state ) {
@@ -1717,7 +1783,7 @@ static gint
 regionview_motion( Regionview *regionview, GdkEvent *ev, int x, int y )
 {
 	GdkWindow *win = GTK_WIDGET( regionview->ip->id )->window;
-	iWindowShape shape;
+	RegionviewResize resize;
 
 #ifdef EVENT
 	printf( "regionview_motion: 0x%x, %d x %d\n", 
@@ -1731,9 +1797,9 @@ regionview_motion( Regionview *regionview, GdkEvent *ev, int x, int y )
 	switch( regionview->state ) {
 	case REGIONVIEW_WAIT:
 		if( regionview_point_in_region( regionview, x, y ) ) {
-			shape = regionview_find_shape( regionview, x, y );
+			resize = regionview_find_resize( regionview, x, y );
 			iwindow_cursor_context_set_cursor( regionview->cntxt, 
-				shape );
+				regionview_cursors[resize] );
 			regionview_set_paint_state( regionview, 
 				GTK_STATE_PRELIGHT );
 		}
@@ -1783,6 +1849,12 @@ regionview_event_cb( GtkWidget *widget, GdkEvent *ev, Regionview *regionview )
 	if( imagemodel->state != IMAGEMODEL_SELECT && regionview->classmodel )
 		return( FALSE );
 
+	/* If there's a regionview grabbed, only that regionview responds to
+	 * events.
+	 */
+	if( regionview->ip->grabbed && regionview->ip->grabbed != regionview )
+		return( FALSE );
+
 	switch( ev->type ) {
 	case GDK_BUTTON_PRESS:
 		switch( ev->button.button ) {
@@ -1806,7 +1878,7 @@ regionview_event_cb( GtkWidget *widget, GdkEvent *ev, Regionview *regionview )
 		switch( ev->button.button ) {
 		case 1:
 			if( regionview->state == REGIONVIEW_MOVE &&
-				regionview->shape == IWINDOW_SHAPE_EDIT ) {
+				regionview->resize == REGIONVIEW_RESIZE_EDIT ) {
 				model_edit( GTK_WIDGET( ip ), 
 					MODEL( regionview->classmodel ) );
 				handled = TRUE;
@@ -1874,11 +1946,6 @@ regionview_dash_crawl_cb( Regionview *regionview )
 	if( regionview->type != REGIONVIEW_REGION &&
 		regionview->type != REGIONVIEW_MARK &&
 		regionview->type != REGIONVIEW_AREA ) {
-#ifdef DEBUG
-		printf( "regionview_dash_crawl_cb: 0x%x\n", 
-			(unsigned int) regionview );
-#endif /*DEBUG*/
-
 		regionview->dash_offset += 3;
 
 		/* Don't repaint before the first expose. last_type etc.
@@ -1931,7 +1998,8 @@ regionview_setup( Regionview *regionview,
 		G_CALLBACK( regionview_conv_destroy_cb ), regionview );
 
 	iwnd = IWINDOW( gtk_widget_get_toplevel( GTK_WIDGET( ip ) ) );	
-	regionview->cntxt = iwindow_cursor_context_new( iwnd );
+	regionview->cntxt = iwindow_cursor_context_new( iwnd, 
+		1, "regionview" );
 
 	popup_link( GTK_WIDGET( regionview ), regionview_popup_menu, ip );
 
@@ -1946,11 +2014,11 @@ regionview_new( Classmodel *classmodel, Rect *model_area, Imagepresent *ip )
 
 	regionview_setup( regionview, classmodel, model_area, ip );
 
-#ifdef DEBUG
+#ifdef DEBUG_MAKE
 	printf( "regionview_new: %dx%d size %dx%d\n",
 		model_area->left, model_area->top, 
 		model_area->width, model_area->height );
-#endif /*DEBUG*/
+#endif /*DEBUG_MAKE*/
 
 	return( regionview );
 }
