@@ -64,8 +64,8 @@ rowview_destroy( GtkObject *object )
 	printf( "\n" );
 #endif /*DEBUG*/
 
-	FREEFI( g_source_remove, rview->set_name_tid );
-	FREE( rview->last_tooltip );
+	IM_FREEF( g_source_remove, rview->set_name_tid );
+	IM_FREE( rview->last_tooltip );
 
 	/* Kill children ... must do this ourselves, since we are not a
 	 * self-contained widget.
@@ -135,7 +135,7 @@ rowview_set_name( Rowview *rview, const char *set_name )
 	if( changed ) {
 		/* Reset the timeout.
 		 */
-		FREEFI( g_source_remove, rview->set_name_tid );
+		IM_FREEF( g_source_remove, rview->set_name_tid );
 		rview->set_name_tid = g_timeout_add( 200, 
 			(GSourceFunc) rowview_set_name_cb, rview );
 	}
@@ -654,7 +654,7 @@ rowview_class_init( RowviewClass *class )
         /* Other init.
          */
 	pane = rowview_popup_menu = popup_build( _( "Row menu" ) );
-	popup_add_but( pane, _( "_Edit ..." ), 
+	popup_add_but( pane, _( "_Edit" ), 
 		POPUP_FUNC( rowview_edit_cb ) );
 	popup_add_but( pane, STOCK_DUPLICATE,
 		POPUP_FUNC( rowview_clone_cb ) );
@@ -662,7 +662,7 @@ rowview_class_init( RowviewClass *class )
 		POPUP_FUNC( rowview_ungroup_cb ) );
 	popup_add_but( pane, GTK_STOCK_SAVE_AS,
 		POPUP_FUNC( rowview_save_cb ) );
-	popup_add_but( pane, _( "Replace From _File ..." ), 
+	popup_add_but( pane, _( "Replace From _File" ), 
 		POPUP_FUNC( rowview_replace_cb ) );
 	popup_add_but( pane, _( "_Recalculate" ), 
 		POPUP_FUNC( rowview_recalc_cb ) );
