@@ -358,7 +358,7 @@ symbol_strip( Symbol *sym )
 	/* Can free the patch list. We should not have to resolve off this
 	 * name again.
 	 */
-	FREEF( g_slist_free, sym->patch );
+	IM_FREEF( g_slist_free, sym->patch );
 
 	/* Workspacegroup? Unlink from wsg.
 	 */
@@ -552,8 +552,8 @@ symbol_dispose( GObject *gobject )
 	assert( !sym->topparents );
 	assert( !sym->topchildren );
 
-	FREEF( g_slist_free, sym->patch );
-	FREEF( g_slist_free, sym->parents );
+	IM_FREEF( g_slist_free, sym->patch );
+	IM_FREEF( g_slist_free, sym->parents );
 
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
