@@ -236,15 +236,15 @@ toolitem_free( Toolitem *toolitem )
 		toolitem->parent = NULL;
 	}
 
-	FREE( toolitem->label );
-	FREE( toolitem->name );
-	FREE( toolitem->icon );
-	FREE( toolitem->tooltip );
-	FREE( toolitem->help );
-	FREE( toolitem->action );
-	FREE( toolitem->path );
-	FREE( toolitem->user_path );
-	FREE( toolitem );
+	IM_FREE( toolitem->label );
+	IM_FREE( toolitem->name );
+	IM_FREE( toolitem->icon );
+	IM_FREE( toolitem->tooltip );
+	IM_FREE( toolitem->help );
+	IM_FREE( toolitem->action );
+	IM_FREE( toolitem->path );
+	IM_FREE( toolitem->user_path );
+	IM_FREE( toolitem );
 
 	return( NULL );
 }
@@ -304,12 +304,14 @@ toolitem_set_name( Toolitem *toolitem, PElement *root )
 			 */
 			SETSTR( toolitem->label, _( value ) );
 
-			/* Strip underscores (they mark mnenonics).
+			/* Strip underscores (they mark mnemonics). Can't use
+			 * strrcpy(), we have overlapping blocks.
 			 */
 			im_strncpy( value, toolitem->label, 100 );
 			for( i = 0; value[i]; i++ )
 				if( value[i] == '_' )
-					strcpy( value + i, value + i + 1 );
+					memccpy( value + i, value + i + 1,
+						0, 100 - i );
 			SETSTR( toolitem->name, value );
 		}
 
@@ -319,7 +321,7 @@ toolitem_set_name( Toolitem *toolitem, PElement *root )
 	}
 	else {
 		/* Remove underscores from the object name ... we don't want
-		 * them to be menonics.
+		 * them to be mnemonics.
 		 */
 		im_strncpy( value, 
 			IOBJECT( toolitem->compile->sym )->name, 100 );
@@ -645,7 +647,7 @@ toolitem_print_all( Toolitem *toolitem )
 static void
 tool_toolitem_rebuild( Tool *tool )
 {
-	FREEF( toolitem_free, tool->toolitem );
+	IM_FREEF( toolitem_free, tool->toolitem );
 
 	switch( tool->type ) {
 	case TOOL_SYM:
@@ -657,13 +659,8 @@ tool_toolitem_rebuild( Tool *tool )
 		break;
 
 	case TOOL_DIA:
-		if( (tool->toolitem = toolitem_new( NULL, NULL, tool )) ) {
-			char value[100];
-
-			im_snprintf( value, 100, "%s ...", 
-				IOBJECT( tool )->name );
-			SETSTR( tool->toolitem->label, value );
-		}
+		if( (tool->toolitem = toolitem_new( NULL, NULL, tool )) ) 
+			SETSTR( tool->toolitem->label, IOBJECT( tool )->name );
 		break;
 
 	case TOOL_SEP:
@@ -782,9 +779,9 @@ tool_new_dia( Toolkit *kit, int pos,
 	if( (tool = tool_find( kit, name )) ) {
 		if( tool->type != TOOL_DIA ) {
 			error_top( _( "Name clash." ) );
-			error_sub( _( "Can't create dialog with name \"%s\"\n"
+			error_sub( _( "Can't create dialog with name \"%s\", "
 				"an object with that name already exists in "
-				"kit \"%s\"" ), 
+				"kit \"%s\"." ), 
 				name, IOBJECT( kit )->name );
 			return( NULL );
 		}
