@@ -165,7 +165,17 @@ toolview_refresh_sub( Toolitem *toolitem, Workspace *ws, GtkWidget *menu )
 		}
 	}
 
-	gtk_menu_shell_append( GTK_MENU_SHELL( menu ), item );
+	/* Is a top-level toolitem?
+ 	 */
+	if( toolitem == toolitem->tool->toolitem ) 
+		gtk_menu_shell_insert( GTK_MENU_SHELL( menu ), item, 
+			ICONTAINER( toolitem->tool )->pos + 1 );
+	else
+		/* Submenus are always completely rebuilt, so we can just 
+		 * append.
+		 */
+		gtk_menu_shell_append( GTK_MENU_SHELL( menu ), item );
+
 	gtk_widget_show( item );
 
 	return( item );
@@ -216,6 +226,7 @@ toolview_refresh( vObject *vobject )
 			g_quark_from_static_string( "toolview_ws_quark" );
 
 	DESTROY_GTK( tview->item );
+
 	if( tool->toolitem ) 
 		tview->item = toolview_refresh_sub( tool->toolitem, 
 			ws, kview->menu );
