@@ -35,7 +35,7 @@ void
 tree_const_destroy( ParseConst *pc )
 {
 	if( pc->type == PARSE_CONST_STR )
-		FREE( pc->val.str );
+		IM_FREE( pc->val.str );
 	pc->type = PARSE_CONST_NONE;
 }
 
@@ -46,7 +46,7 @@ tree_node_destroy( ParseNode *n )
 {
 	switch( n->type ) {
 	case NODE_TAG:
-		FREE( n->tag );
+		IM_FREE( n->tag );
 		break;
 
 	case NODE_CONST:
@@ -55,7 +55,7 @@ tree_node_destroy( ParseNode *n )
 
 	case NODE_LISTCONST:
 	case NODE_SUPER:
-		FREEF( g_slist_free, n->elist );
+		IM_FREEF( g_slist_free, n->elist );
 		break;
 
 	case NODE_APPLY:
@@ -72,7 +72,7 @@ tree_node_destroy( ParseNode *n )
 		assert( FALSE );
 	}
 
-	FREE( n );
+	IM_FREE( n );
 
 	return( NULL );
 }
