@@ -278,7 +278,7 @@ get_sprop( xmlNode *xnode, const char *name, char *buf, int sz )
 		return( FALSE );
 
 	im_strncpy( buf, value, sz );
-	FREEF( xmlFree, value );
+	IM_FREEF( xmlFree, value );
 
 	return( TRUE );
 }
@@ -292,7 +292,7 @@ get_spropb( xmlNode *xnode, const char *name, BufInfo *buf )
 		return( FALSE );
 
 	buf_appends( buf, value );
-	FREEF( xmlFree, value );
+	IM_FREEF( xmlFree, value );
 
 	return( TRUE );
 }
@@ -828,7 +828,7 @@ void
 buf_destroy( BufInfo *buf )
 {
 	if( buf->dynamic ) {
-		FREE( buf->base );
+		IM_FREE( buf->base );
 	}
 
 	buf_init( buf );
@@ -1216,7 +1216,11 @@ strcasestr( const char *haystack, const char *needle )
 }
 #endif /*!HAVE_STRCASESTR*/
 
-/* Fix this! copy a string, interpreting (a few) C-language escape codes.
+/* Copy a string, interpreting (a few) C-language escape codes.
+
+	FIXME ... yuk! dangerous and not that useful, needs a proper function
+	to do this
+
  */
 char *
 my_strccpy( char *output, const char *input )
@@ -1235,6 +1239,10 @@ my_strccpy( char *output, const char *input )
 				q[0] = '\t';
 				break;
 
+			case 'r':
+				q[0] = '\r';
+				break;
+
 			case '"':
 				q[0] = '\"';
 				break;
@@ -1288,6 +1296,12 @@ my_strecpy( char *output, const char *input, gboolean quote )
 			q++;
 			break;
 
+		case '\r':
+			q[0] = '\\';
+			q[1] = 'r';
+			q++;
+			break;
+
 		case '\"':
 			if( quote ) {
 				q[0] = '\\';
@@ -1656,7 +1670,8 @@ expand_once( char *in, char *out )
 		if( *p == '$' ) {
 			char vname[FILENAME_MAX];
 			char *r;
-			char *subst;
+			const char *subst;
+			const char *s;
 
 			/* Extract the variable name.
 			 */
@@ -1671,14 +1686,14 @@ expand_once( char *in, char *out )
 
 			/* Look up variable.
 			 */
-			subst = getenv( vname );
+			subst = g_getenv( vname );
 
 			/* Copy variable contents.
 			 */
 			if( subst ) {
-				for( r = subst; 
-					(*q = *r) && (q - out) < FILENAME_MAX; 
-					r++, q++ )
+				for( s = subst; 
+					(*q = *s) && (q - out) < FILENAME_MAX; 
+					s++, q++ )
 					;
 			}
 			q--;
@@ -1690,8 +1705,8 @@ expand_once( char *in, char *out )
 		/* Or a '~' at the start of the string?
 		 */
 		else if( *p == '~' && p == in ) {
-			char *r;
-			char *subst = getenv( "HOME" );
+			const char *subst = g_getenv( "HOME" );
+			const char *r;
 
 			/* Copy variable contents.
 			 */
@@ -1712,7 +1727,7 @@ expand_once( char *in, char *out )
 }
 
 /* Expand all variables! Don't touch in, assume out[] is at least
- * FILENAME_MAX bytes.
+ * FILENAME_MAX bytes. in and out must not point to the same place!
  */
 void
 expand_variables( const char *in, char *out )
@@ -1721,6 +1736,8 @@ expand_variables( const char *in, char *out )
 	char *p1 = (char *) in;		/* Discard const, but safe */
 	char *p2 = out;
 
+	assert( in != out );
+
 	/* Expand any environment variables in component.
 	 */
 	while( expand_once( p1, p2 ) ) 
@@ -1782,6 +1799,8 @@ swap_string( char *buffer, const char *from, const char *to )
 void
 canonicalize_path( char *path )
 {
+	gboolean found;
+
 	assert( is_absolute( path ) );
 
 	/* Any "/./" can go.
@@ -1789,14 +1808,39 @@ canonicalize_path( char *path )
 	swap_string( path, 
 		IM_DIR_SEP_STR "." IM_DIR_SEP_STR, IM_DIR_SEP_STR );
 
-	/* 
-
-		FIXME ... should swap /[^/]+/../ for "" as well, but just
-		removing "." is enough to fix filechooser probs for now
-
-		Could use realpath ... but it's not very portable :-( 
-	
+	/* Any "//" can go.
 	 */
+	swap_string( path, 
+		IM_DIR_SEP_STR IM_DIR_SEP_STR, IM_DIR_SEP_STR );
+
+	/* Repeatedly search for "/[^/]+/../" and remove it.
+
+		FIXME ... yuk, should search backwards from the end of the
+		string
+
+	 */
+	do {
+		char *p;
+
+		found = FALSE;
+
+		for( p = path; (p = strchr( p, IM_DIR_SEP )); p++ ) {
+			char *q;
+
+			q = strchr( p + 1, IM_DIR_SEP );
+
+			if( q && isprefix( IM_DIR_SEP_STR "..", q ) ) {
+				memmove( p, q + 3, strlen( q + 3 ) + 1 );
+				found = TRUE;
+				break;
+			}
+		}
+	} while( found );
+
+	/* We may have the empty string ... that's just '/'.
+	 */
+	if( strcmp( path, "" ) == 0 )
+		strcpy( path, IM_DIR_SEP_STR );
 }
 
 /* Absoluteize a path. Must be FILENAME_MAX chars available.
@@ -1817,6 +1861,32 @@ absoluteize_path( char *path )
 	}
 }
 
+/* Guess VIPSHOME, if we can.
+ */
+const char *
+get_vipshome( const char *argv0 )
+{
+	static char prefix_buffer[FILENAME_MAX];
+	static const char *prefix;
+
+	if( !prefix ) {
+		if( !(prefix = im_guess_prefix( argv0, "VIPSHOME" )) ) {
+			error_top( _( "Unable to find install area." ) );
+			error_vips();
+
+			return( NULL );
+		}
+
+		im_strncpy( prefix_buffer, prefix, FILENAME_MAX );
+
+		absoluteize_path( prefix_buffer );
+		canonicalize_path( prefix_buffer );
+		setenvf( "VIPSHOME", "%s", prefix_buffer );
+	}
+
+	return( prefix_buffer );
+}
+
 /* Call a function, building a string arg. We expand env. variables, but
  * that's all.
  */
@@ -1907,19 +1977,16 @@ f2utf8( const char *filename )
 }
 
 void
-putenvf( const char *name, ... )
+setenvf( const char *name, const char *fmt, ... )
 {
 	va_list ap;
 	char buf[FILENAME_MAX];
 
-        va_start( ap, name );
-        (void) im_vsnprintf( buf, FILENAME_MAX, name, ap );
+        va_start( ap, fmt );
+        (void) im_vsnprintf( buf, FILENAME_MAX, fmt, ap );
         va_end( ap );
 
-	/* On some platform we should free, on some we shouldn't :-( ignore
-	 * this potential leak, it's not too bad.
-	 */
-	putenv( g_strdup( buf ) );
+	g_setenv( name, buf, TRUE );
 }
 
 /* File exists? 
@@ -1931,8 +1998,9 @@ existsf( const char *name, ... )
 	gboolean res;
 
         va_start( ap, name );
-        res = (int) call_string_filenameva( (call_string_fn) im_existsf, 
-		name, ap, NULL, NULL, NULL );
+        res = GPOINTER_TO_INT( call_string_filenameva( 
+		(call_string_fn) im_existsf, 
+			name, ap, NULL, NULL, NULL ) );
         va_end( ap );
 
 	return( res );
@@ -1945,8 +2013,10 @@ mkdirf( const char *name, ... )
 	gboolean res;
 
         va_start( ap, name );
-        res = call_string_filenameva( (call_string_fn) mkdir, 
-		name, ap, (void *) 0755, NULL, NULL ) == NULL;
+        res = GPOINTER_TO_INT( call_string_filenameva( 
+		(call_string_fn) mkdir, 
+			name, ap, GINT_TO_POINTER( 0755 ), NULL, NULL ) == 
+			NULL );
         va_end( ap );
 
 	return( res );
@@ -1961,8 +2031,8 @@ systemf( const char *fmt, ... )
 	int res;
 
         va_start( ap, fmt );
-        res = (int) call_stringva( (call_string_fn) system, 
-		fmt, ap, NULL, NULL, NULL );
+        res = GPOINTER_TO_INT( call_stringva( 
+		(call_string_fn) system, fmt, ap, NULL, NULL, NULL ) );
         va_end( ap );
 
 	return( res );
@@ -1975,8 +2045,10 @@ touchf( const char *fmt, ... )
 	int fd;
 
         va_start( ap, fmt );
-        fd = (int) call_string_filenameva( (call_string_fn) creat, 
-		fmt, ap, (void *) (S_IRUSR | S_IWUSR), NULL, NULL );
+        fd = GPOINTER_TO_INT( call_string_filenameva( 
+		(call_string_fn) creat, 
+			fmt, ap, GINT_TO_POINTER( S_IRUSR | S_IWUSR ), 
+				NULL, NULL ) );
         va_end( ap );
 	(void) close( fd );
 
@@ -1990,8 +2062,9 @@ unlinkf( const char *fmt, ... )
 	int res;
 
         va_start( ap, fmt );
-        res = (int) call_string_filenameva( (call_string_fn) unlink, 
-		fmt, ap, NULL, NULL, NULL );
+        res = GPOINTER_TO_INT( call_string_filenameva( 
+		(call_string_fn) unlink, 
+			fmt, ap, NULL, NULL, NULL ) );
         va_end( ap );
 
 	return( res );
@@ -2020,14 +2093,6 @@ is_valid_filename( const char *name )
 {
 	const char *p;
 
-#ifndef HAVE_WINDOWS_H
-	if( strchr( name, ':' ) ) {
-		error_top( _( "Bad filename." ) );
-		error_sub( _( "Filenames may not contain ':' characters." ) );
-		return( FALSE );
-	}
-#endif
-
 	if( strlen( name ) > FILENAME_MAX ) {
 		error_top( _( "Bad filename." ) );
 		error_sub( _( "Filename is too long." ) );
@@ -2052,10 +2117,10 @@ char *im_strdupn( const char *txt ) { return( im_strdup( NULL, txt ) ); }
 void
 file_close( iOpenFile *of )
 {
-	FREEF( fclose, of->fp );
-	FREE( of->fname );
-	FREE( of->fname_real );
-	FREE( of );
+	IM_FREEF( fclose, of->fp );
+	IM_FREE( of->fname );
+	IM_FREE( of->fname_real );
+	IM_FREE( of );
 }
 
 /* Make an iOpenFile*.
@@ -2228,12 +2293,12 @@ file_read( iOpenFile *of )
 
 	/* Allocate memory and fill.
 	 */
-	if( !(str = im_malloc( NULL, len + 1 )) ) 
+	if( !(str = imalloc( NULL, len + 1 )) ) 
 		return( NULL );
 	if( fread( str, sizeof( char ), (size_t) len, of->fp ) != 
 		(size_t) len ) {
 		of->last_errno = errno;
-		FREE( str );
+		IM_FREE( str );
 		error_top( _( "Unable to read." ) );
 		error_sub( _( "Unable to read from file \"%s\".\n%s." ),
 			of->fname_real, g_strerror( of->last_errno ) );
@@ -2273,8 +2338,9 @@ statf( const char *fmt, ... )
 	int result;
 
         va_start( ap, fmt );
-        result = (int) call_string_filenameva( (call_string_fn) stat, 
-		fmt, ap, &st, NULL, NULL );
+        result = GPOINTER_TO_INT( call_string_filenameva( 
+		(call_string_fn) stat, 
+			fmt, ap, &st, NULL, NULL ) );
         va_end( ap );
 
 	if( result == -1 || S_ISDIR( st.st_mode ) )
@@ -2433,27 +2499,6 @@ number_to_string( int n, char *buf )
 	*buf ='\0';
 }
 
-/* Hash from a pointer into an index.
- */
-unsigned int
-hash_pointer( void *p )
-{
-	return( 0xffffff & (unsigned int) p );
-}
-
-/* Hash from a character string into an index.
- */
-unsigned int
-hash_name( const char *n )
-{
-	unsigned int h;
-
-	for( h = 0; *n != '\0'; n += 1 ) 
-		h = (h << 5) - h + *n;
-
-	return( h ); 
-}
-
 /* Find the space remaining in a directory, in bytes. A double for >32bit
  * problem avoidance. <0 for error.
  */
@@ -2572,35 +2617,26 @@ temp_name( char *name, const char *type )
 	static int n = 0;
 
 	const char *dir;
-#ifdef HAVE_MKSTEMP
 	int fd;
-#endif /*HAVE_MKSTEMP*/
 	char buf[FILENAME_MAX];
 
 	dir = PATH_TMP;
 	if( !existsf( "%s", dir ) )
-		dir = IM_DIR_SEP_STR "tmp";
+		dir = IM_DIR_SEP_STR;
 
 	im_snprintf( name, FILENAME_MAX, "%s" IM_DIR_SEP_STR 
 		"untitled-" PACKAGE "-%d-XXXXXXX", 
 		dir, n++ );
 	expand_variables( name, buf );
 
-#ifdef HAVE_MKSTEMP
-	fd = mkstemp( buf );
+	fd = g_mkstemp( buf );
 	if( fd == -1 ) {
-		error( _( "unable to make temporary file \"%s\"\n%s" ),
+		error_top( _( "Unable to create temporary file." ) );
+		error_sub( _( "Unable to make file \"%s\"\n%s" ),
 			buf, g_strerror( errno ) );
 		return( FALSE );
 	}
 	close( fd );
-#else /*HAVE_MKSTEMP*/
-#ifdef HAVE_MKTEMP
-	mktemp( buf );
-#else /*HAVE_MKTEMP*/
-#error "no temporary file name maker found"
-#endif /*HAVE_MKTEMP*/
-#endif /*HAVE_MKSTEMP*/
 	unlinkf( "%s", buf );
 
 	im_snprintf( name, FILENAME_MAX, "%s.%s", buf, type );
@@ -2792,9 +2828,8 @@ imalloc( IMAGE *im, size_t len )
 		buf_init_static( &buf, txt, 256 );
 		to_size( &buf, len );
 		error_top( _( "Out of memory." ) );
-		error_sub( _( 
-			"Request for %s of RAM triggered memory allocation\n"
-			"failure." ) );
+		error_sub( _( "Request for %s of RAM triggered memory "
+			"allocation failure." ), buf_all( &buf ) );
 		error_vips();
 
 		return( NULL );
@@ -2853,7 +2888,8 @@ recent_load( const char *filename )
 
 	recent = NULL;
 
-	if( (of = file_open_read( "%s", filename )) ) {
+	if( (of = file_open_read( "%s" IM_DIR_SEP_STR "%s", 
+		get_savedir(), filename )) ) {
 		char buf[256];
 
 		while( fgets( buf, 256, of->fp ) ) {
@@ -2916,7 +2952,8 @@ recent_save( GSList *recent, const char *filename )
 	slist_map_rev( recent, 
 		(SListMapFn) recent_save_sub, &old_recent );
 
-	if( (of = file_open_write( "%s", filename )) ) {
+	if( (of = file_open_write( "%s" IM_DIR_SEP_STR "%s", 
+		get_savedir(), filename )) ) {
 		slist_map_rev( old_recent,
 			(SListMapFn) recent_save_sub2, of );
 		file_close( of );
@@ -2958,3 +2995,28 @@ set_symbol_drag_type( GtkWidget *widget )
 		targets, IM_NUMBER( targets ),
 		GDK_ACTION_COPY | GDK_ACTION_MOVE );
 }
+
+/* Return the name of the save dir we use ... eg. "/home/john/.nip2-7.10.8",
+ * or maybe "C:\Documents and Settings\john\Application Data"
+ */
+const char *
+get_savedir( void )
+{
+#ifdef HAVE_WINDOWS_H
+	/* If APPDATA is not defined, default to HOME, we know that will
+	 * exist (since we make it if necessary in main()).
+	 */
+	if( g_getenv( "APPDATA" ) && existsf( "%s", g_getenv( "APPDATA" ) ) )
+		return( "$APPDATA" IM_DIR_SEP_STR IP_NAME );
+	else
+		return( "$HOME" IM_DIR_SEP_STR "." IP_NAME );
+#else /*!HAVE_WINDOWS_H*/
+	/* *nix-style system .. .dot file in home area.
+
+		FIXME ... do something better for Mac
+
+	 */
+	return( "$HOME" IM_DIR_SEP_STR "." IP_NAME );
+#endif /*HAVE_WINDOWS_H*/
+}
+
