@@ -30,9 +30,9 @@
 #include "ip.h"
 
 /*
+#define DEBUG_HISTORY_MISS
 #define DEBUG
 #define DEBUG_HISTORY
-#define DEBUG_HISTORY_MISS
  */
 
 /* Maxiumum number of args to a VIPS function.
@@ -62,6 +62,7 @@ typedef struct _VipsInfo {
 	int ninii;			
 	Imageinfo *inii[MAX_VIPS_ARGS];	
 	unsigned int inii_destroy_sid[MAX_VIPS_ARGS];
+	unsigned int inii_paint_sid[MAX_VIPS_ARGS];
 
 	/* Output images. 
 	 */
@@ -174,7 +175,7 @@ vips_hash( VipsInfo *vi )
  */
 #define HASH_I( I ) hash = (hash << 1) | ((unsigned int) (I));
 #define HASH_D( D ) hash = (hash << 1) | ((unsigned int) (D));
-#define HASH_P( P ) hash = (hash << 1) | ((unsigned int) (P));
+#define HASH_P( P ) hash = (hash << 1) | (GPOINTER_TO_UINT( P ));
 #define HASH_S( S ) hash = (hash << 1) | g_str_hash( S );
 
         for( i = 0; i < vi->fn->argc; i++ ) {
@@ -592,6 +593,11 @@ vips_destroy( VipsInfo *vi )
 	for( i = 0; i < vi->noutii; i++ ) 
 		FREESID( vi->outii_destroy_sid[i], vi->outii[i] ); 
 
+	/* Break paint links.
+	 */
+	for( i = 0; i < vi->ninii; i++ ) 
+		FREESID( vi->inii_paint_sid[i], vi->inii[i] ); 
+
 	/* Free any VIPS args we built and haven't used.
 	 */
 	for( i = 0; i < vi->fn->argc; i++ ) {
@@ -617,7 +623,7 @@ vips_destroy( VipsInfo *vi )
 			break;
 
 		case VIPS_STRING:
-			FREE( obj );
+			IM_FREE( obj );
 			break;
 
 		case VIPS_IMAGE:
@@ -627,22 +633,24 @@ vips_destroy( VipsInfo *vi )
 			if( ty->flags & IM_TYPE_OUTPUT ) {
 				IMAGE **im = (IMAGE **) &obj;
 
-				FREEF( im_close, *im );
+				IM_FREEF( im_close, *im );
 			}
 			break;
 
 		case VIPS_DOUBLEVEC:
-			FREE( ((im_doublevec_object *) obj)->vec );
+			IM_FREE( ((im_doublevec_object *) obj)->vec );
 			break;
 
 		case VIPS_DMASK:
-			FREE( ((im_mask_object *) obj)->name );
-			FREEF( im_free_dmask, ((im_mask_object *) obj)->mask );
+			IM_FREE( ((im_mask_object *) obj)->name );
+			IM_FREEF( im_free_dmask, 
+				((im_mask_object *) obj)->mask );
 			break;
 
 		case VIPS_IMASK:
-			FREE( ((im_mask_object *) obj)->name );
-			FREEF( im_free_imask, ((im_mask_object *) obj)->mask );
+			IM_FREE( ((im_mask_object *) obj)->name );
+			IM_FREEF( im_free_imask, 
+				((im_mask_object *) obj)->mask );
 			break;
 
 		default:
@@ -652,22 +660,33 @@ vips_destroy( VipsInfo *vi )
 
 	if( vi->vargv ) {
 		im_free_vargv( vi->fn, vi->vargv );
-		FREE( vi->vargv );
+		IM_FREE( vi->vargv );
 	}
-	FREE( vi );
+	IM_FREE( vi );
 }
 
 static void
 vips_history_destroy_cb( Imageinfo *ii, VipsInfo *vi )
 {
 #ifdef DEBUG_HISTORY
-	printf( "vips_history_destroy_cb: on death of ii, killing \"%s\"\n", 
+	printf( "vips_history_destroy_cb: on death of ii, uncaching \"%s\"\n", 
 		vi->name );
 #endif /*DEBUG_HISTORY*/
 
 	vips_destroy( vi );
 }
 
+static void
+vips_history_painted_cb( Imageinfo *ii, Rect *dirty, VipsInfo *vi )
+{
+#ifdef DEBUG_HISTORY
+	printf( "vips_history_painted_cb: "
+		"on paint of ii, uncaching \"%s\"\n", vi->name );
+#endif /*DEBUG_HISTORY*/
+
+	vips_destroy( vi );
+}
+
 /* Add a function call to the history. 
  */
 static void
@@ -684,8 +703,8 @@ vips_history_add( VipsInfo *vi )
 	}
 
 #ifdef DEBUG_HISTORY
-	printf( "vips_history_add: adding \"%s\", hash = %x\n", 
-		vi->name, vips->hash );
+	printf( "vips_history_add: adding \"%s\", hash = 0x%x\n", 
+		vi->name, vi->hash );
 #endif /*DEBUG_HISTORY*/
 
 	vips_history_insert( vi );
@@ -694,10 +713,19 @@ vips_history_add( VipsInfo *vi )
 	 */
 	for( i = 0; i < vi->ninii; i++ )
 		vi->inii_destroy_sid[i] = g_signal_connect( vi->inii[i], 
-			"destroy", G_CALLBACK( vips_history_destroy_cb ), vi );
+			"destroy", 
+			G_CALLBACK( vips_history_destroy_cb ), vi );
 	for( i = 0; i < vi->noutii; i++ )
 		vi->outii_destroy_sid[i] = g_signal_connect( vi->outii[i], 
-			"destroy", G_CALLBACK( vips_history_destroy_cb ), vi );
+			"destroy", 
+			G_CALLBACK( vips_history_destroy_cb ), vi );
+
+	/* If any of our input ii are painted on, we must also invalidate.
+	 */
+	for( i = 0; i < vi->ninii; i++ )
+		vi->inii_paint_sid[i] = g_signal_connect( vi->inii[i], 
+			"area_painted", 
+			G_CALLBACK( vips_history_painted_cb ), vi );
 
 	/* History too big? Flush!
 	 */
@@ -977,11 +1005,13 @@ vips_new( Reduce *rc, im_function *fn )
 	}
 
 	/* Need to zero all the signals ... we don't always have signals for
-	 * vi that aren't in the history yet.
+	 * vi that aren't in the history yet. Can't just loop to argc, since
+	 * some args may be IMAGEVEC.
 	 */
-	for( i = 0; i < vi->fn->argc; i++ ) {
+	for( i = 0; i < MAX_VIPS_ARGS; i++ ) {
 		vi->outii_destroy_sid[i] = 0;
 		vi->inii_destroy_sid[i] = 0;
+		vi->inii_paint_sid[i] = 0;
 	}
 
 	/* Make the call spine, alloc memory. 
