@@ -388,7 +388,7 @@ workspace_add_symbol( Workspace *ws )
 
 	name = column_name_new( col );
 	sym = symbol_new( ws->sym->expr->compile, name );
-	FREE( name );
+	IM_FREE( name );
 
 	return( sym );
 }
@@ -446,6 +446,36 @@ workspace_add_def( Workspace *ws, const char *str )
 	return( sym );
 }
 
+Symbol *
+workspace_load_file( Workspace *ws, const char *filename )
+{
+	char txt[MAX_STRSIZE];
+	BufInfo buf;
+	Symbol *sym;
+
+	buf_init_static( &buf, txt, MAX_STRSIZE );
+	buf_appends( &buf, "Image_file \"" );
+	buf_appendsc( &buf, TRUE, filename );
+	buf_appends( &buf, "\"" );
+	if( (sym = workspace_add_def( ws, buf_all( &buf ) )) ) {
+		mainw_recent_add( &mainw_recent_image, filename );
+		return( sym );
+	}
+	error_clear();
+
+	buf_init_static( &buf, txt, MAX_STRSIZE );
+	buf_appends( &buf, "Matrix_file \"" );
+	buf_appendsc( &buf, TRUE, filename );
+	buf_appends( &buf, "\"" );
+	if( (sym = workspace_add_def( ws, buf_all( &buf ) )) ) {
+		mainw_recent_add( &mainw_recent_matrix, filename );
+		return( sym );
+	}
+	error_clear();
+
+	return( NULL );
+}
+
 /* Save just the selected objects.
  */
 gboolean
@@ -556,7 +586,7 @@ workspace_checkmark( Workspace *ws )
 	if( FILEMODEL( ws )->auto_load )
 		return;
 
-	FREEFI( g_source_remove, ws->auto_save_timeout );
+	IM_FREEF( g_source_remove, ws->auto_save_timeout );
 	ws->auto_save_timeout = g_timeout_add( 1000, 
 		(GSourceFunc) workspace_checkmark_timeout, ws );
 }
@@ -571,7 +601,7 @@ workspace_retain_clean( void )
 	for( i = 0; i < WS_RETAIN; i++ ) {
 		if( retain_files[i] ) {
 			unlinkf( "%s", retain_files[i] );
-			FREE( retain_files[i] );
+			IM_FREE( retain_files[i] );
 		}
 	}
 }
@@ -632,7 +662,7 @@ workspace_auto_recover_load( iWindow *iwnd,
 		 * "~/.nip2-7.9.6/tmp/untitled-nip2-0-3904875.ws", very
 		 * unhelpful.
 		 */
-		FREE( filemodel->filename );
+		IM_FREE( filemodel->filename );
 		iobject_changed( IOBJECT( filemodel ) );
 
 		nfn( sys, IWINDOW_TRUE );
@@ -671,8 +701,8 @@ workspace_auto_recover( GtkWidget *parent )
 			box_info( parent, 
 				_( "No backup workspaces found." ),
 				_( "You need to enable \"Auto workspace "
-				"save\" in Preferences\n"
-				"Before automatic recovery works" ) );
+				"save\" in Preferences "
+				"before automatic recovery works." ) );
 		else
 			box_info( parent, 
 				_( "No backup workspaces found." ),
@@ -685,13 +715,13 @@ workspace_auto_recover( GtkWidget *parent )
 	/* Tricksy ... free str with notify callack from yesno.
 	 */
 	name = im_strdupn( buf2 );
+
 	box_yesno( parent, 
 		workspace_auto_recover_load, iwindow_true_cb, name, 
 		(iWindowNotifyFn) im_free, name,
 		GTK_STOCK_OPEN, 
 		_( "Open workspace backup?" ),
-		_( "Found workspace \"%s\",\n"
-		"dated %s"
+		_( "Found workspace \"%s\", dated %s. "
 		"Do you want to recover this workspace?" ),
 		name, ctime( &date_sofar ) );
 }
@@ -708,7 +738,7 @@ workspace_dispose( GObject *gobject )
 
 	ws->last_column = NULL;
 	IDESTROY( ws->sym );
-	FREEFI( g_source_remove, ws->auto_save_timeout );
+	IM_FREEF( g_source_remove, ws->auto_save_timeout );
 
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
@@ -729,7 +759,7 @@ workspace_finalize( GObject *gobject )
 
 	UNREF( ws->kitg );
 
-	FREE( ws->status );
+	IM_FREE( ws->status );
 	workspace_all = g_slist_remove( workspace_all, ws );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
@@ -831,6 +861,11 @@ workspace_load( Model *model,
 			ws->mode = mode;
 	}
 
+	/* Also optional.
+	 */
+	(void) get_dprop( xnode, "scale", &ws->scale );
+	(void) get_dprop( xnode, "offset", &ws->offset );
+
 	if( !MODEL_CLASS( parent_class )->load( model, state, parent, xnode ) )
 		return( FALSE );
 
@@ -846,7 +881,9 @@ workspace_save( Model *model, xmlNode *xnode )
 	if( !(xthis = MODEL_CLASS( parent_class )->save( model, xnode )) )
 		return( NULL );
 
-	if( !set_prop( xthis, "view", workspacemode_to_char( ws->mode ) ) )
+	if( !set_prop( xthis, "view", workspacemode_to_char( ws->mode ) ) ||
+		!set_prop( xthis, "scale", "%g", ws->scale ) ||
+		!set_prop( xthis, "offset", "%g", ws->offset ) )
 		return( NULL );
 
 	return( xthis );
@@ -879,7 +916,7 @@ workspace_rename_row_node( ModelLoadState *state, Column *col, xmlNode *xnode )
 		new_name = column_name_new( col );
 		(void) set_prop( xnode, "name", new_name );
 		(void) model_loadstate_rename_new( state, name, new_name );
-		FREE( new_name );
+		IM_FREE( new_name );
 	}
 }
 
@@ -911,7 +948,7 @@ workspace_rename_column_node( Workspace *ws,
 #endif /*DEBUG*/
 
 		(void) set_prop( xnode, "name", new_name );
-		FREE( new_name );
+		IM_FREE( new_name );
 
 		/* And allocate new names for all rows in the subcolumn.
 		 */
@@ -1038,7 +1075,7 @@ workspace_top_load( Filemodel *filemodel,
 			state->minor != filemodel->minor ) {
 			box_info( NULL,
 				_( "Version mismatch." ),
-				_( "File \"%s\" was saved from %s-%d.%d.%d.\n"
+				_( "File \"%s\" was saved from %s-%d.%d.%d. "
 				"You may see compatibility problems." ),
 				state->filename, PACKAGE,
 				state->major, state->minor, state->micro );
@@ -1100,9 +1137,12 @@ workspace_save_all( Filemodel *filemodel, const char *filename )
 
 	if( (result = FILEMODEL_CLASS( parent_class )->save_all( filemodel,
 		filename )) )
-		/* This will add save-as files to recent too.
+		/* This will add save-as files to recent too. Don't note
+		 * auto_load on recent, since it won't have been loaded by the
+		 * user.
 		 */
-		mainw_recent_add( &mainw_recent_workspace, filename );
+		if( !filemodel->auto_load )
+			mainw_recent_add( &mainw_recent_workspace, filename );
 
 	return( result );
 }
@@ -1180,9 +1220,12 @@ workspace_init( Workspace *ws )
 
 	ws->status = NULL;
 	im_snprintf( txt, 256, 
-		_( "%s: (c) 2004 The National Gallery" ), PACKAGE );
+		_( "%s: (c) 2005 The National Gallery" ), PACKAGE );
 	SETSTR( ws->status, txt );
 
+	ws->scale = 1.0;
+	ws->offset = 0.0;
+
 	filemodel_register( FILEMODEL( ws ) );
 
 	workspace_all = g_slist_prepend( workspace_all, ws );
@@ -1224,7 +1267,7 @@ workspace_new( Workspacegroup *wsg, const char *name )
 
 	if( compile_lookup( wsg->sym->expr->compile, name ) ) {
 		error_top( _( "Name clash." ) );
-		error_sub( _( "Can't create workspace \"%s\".\n"
+		error_sub( _( "Can't create workspace \"%s\". "
 			"A symbol with that name already exists." ), name );
 		return( NULL );
 	}
@@ -1459,7 +1502,7 @@ workspace_selected_recalc( Workspace *ws )
 {
 	if( workspace_selected_map( ws,
 		(row_map_fn) workspace_row_dirty, 
-		(void *) link_serial_new(), NULL ) ) 
+		 GINT_TO_POINTER( link_serial_new() ), NULL ) ) 
 		return( FALSE );
 
 	/* Recalc even if autorecomp is off.
@@ -1545,7 +1588,7 @@ workspace_selected_remove( Workspace *ws )
 	(void) slist_map( cs, 
 		(SListMapFn) workspace_selected_remove5, NULL );
 
-	FREEF( g_slist_free, cs );
+	IM_FREEF( g_slist_free, cs );
 	symbol_recalculate_all();
 	filemodel_set_modified( FILEMODEL( ws ), TRUE );
 
